***********************************************************************
*     file montecarlo.F                                               *
*     adapted from COFFERGG by Markus Roth                            *
*---------------------------------------------------------------------*
*     30.01.06  Ansgar Denner     last changed  09.06.10              *
***********************************************************************

* date and time, flush
c#define G77
c#define PGF77

#define PRINTWEIGHTMAX

#define RANLUX

* switch subprocesses on
c#define SUBPROCESSES


***********************************************************************
* NOTES                                                               *
*                                                                     *
*   LOOPS OVER EMITTER AND SPECTATOR ASSUME UNCHARGED HIGGS!!!!       *
*   need to steer charge flow from composite particles!!!!!           *
*                                                                     *
*       fac(phasespace)=0.5d0/pi/gsum(phasespace)*wconv               *
*                                                                     *
*       factor(gen)=mcgevtofb/(2d0*pi)**(3*next(gen)-10)              *
*    &    /(2d0*sqrt(energy**4                                       *
*    &      -mcm(iabs(hepnum(1,gen)))**2*mcm(iabs(hepnum(2,gen)))**2))*
*                                                                     *
*     weight(1,phasespace)=m2pdfpdf_0(phasespace)*wconv               *
*                     *factor(generator)/gsum(phasespace)             *
*                                                                     *
*                                                                     *
*   exit 1:      generation: s < sppmin                               *
*   exit 2:      density:    sinv < 0                                 *
*   exit 3:      inv:        smin > smax                              *
*   exit 4:      inv:        smin < 0                                 *
*   exit 5:      inv:        1/density <= 0 (width>0)                 *
*   exit 6:      inv:        1/density <= 0 (width=0)                 *
*   exit 7:      process:    dabs(q1(1:2)+q2(1:2)) > acc              *
*   exit 8:      process:    lambda < 0                               *
*   exit 9:      process:    d < 0                                    *
*   exit 10:     process:    lambdas <= 0                             *
*   exit 11:     process:    lambdat <= 0                             *
*   exit 12:     process:    tmin >= tmax                             *
*   exit 13:     process:    tmax > 0                                 *
*   exit 14:     process:    1/density = 0 (width>0)                  *
*   exit 15:     process:    1/density = 0 (width=0)                  *
*   exit 16:     process:    s <= 0                                   *
*   exit 17:     decay:      lambda <= 0                              *
*   exit 18:     decay:      s <= 0                                   *
*   exit 19:     decay:      1/density <= 0                           *
*   exit 20:     mapinv:     xmin-m2, xmax-m2 < 0                     *
*   exit 21:     jacobianinv:xmin-m2, xmax-m2 < 0                     *
*   exit 22:     rotation:   |cost| > 1                               *
*   exit 23:     boost:      m2 <= 0                                  *
*   exit 24:     mapin:      1/g <= 0                                 *
*   exit 25:     mapout:     1/g <= 0                                 *
*   exit 26:     transverse: pvec = 0                                 *
*   exit 27:     integrand:  x = 1                                    *
*   exit 28:     integrand:  gsum = 0                                 *
*   exit 29:     integrand:  s(em,sp,...) < 0                         *
*   exit 30:     integrand:  s(em,sp,...) < 0                         *
*   exit 31:     integrand:  s(em,sp,1) <= 0                          *
*   exit 32:     conv:       x=1                                      *
*   exit 33:     dli2gen:    converges badly                          *
*   exit 34:     genbrems  : m2 <= 0                                  *
*   exit 35:     momentum conservation violated:                      *
*   exit 36:     onshellness violated:                                *
*   exit 40:     mapx:       xmin-m2, xmax-m2 < 0                     *
*   exit 41:     jacobianx:  xmin-m2, xmax-m2 < 0                     *
*   exit 42:     mapout:     z = 1 (eliminated)                       *
*   exit 43:     mapout:     z = 0 (eliminated)                       *
*   exit 44:     dets:       sinv = 0 (eliminated)                    *
*   exit 45:     ktcluster:  |cos(deltaphi)| > 0                      *
*   exit 46:     recombination:  |cos(deltaphi)| > 0                  *
*   exit 47:     integrand:   integrand: s(1,2,1) < 0                 *
*   exit 48:     integrand:   integrand: s(em,sp,1) < 0               *
*                                                                     *
***********************************************************************
* CHANGES with respect to MAR                                         *
*                                                                     *
*   nsteps                                                            *
*                                                                     *
***********************************************************************
* TO BE ADAPTED                                                       *
*                                                                     *
*    SWITCH weak                                                      *
*                                                                     *
*    histogram to weight(contr)                                       *
*                                                                     *
***********************************************************************
*                                                                     *
*     Monte Carlo generator                                           *
*                                                                     *
*     written by Markus Roth          2005                            *
*     adapted by Ansgar Denner                                        *
*                                                                     *
*     last changed  31.03.2010                                        *
*                                                                     *
***********************************************************************
      program montecarlo
      implicit none

#ifdef mpiuse
      include "mpif.h"
#endif

#include "mcparams.h"
#include "mccommon.h"

c local variables
      integer unweighting
      real*8  s,m2,ps3,ps4

      integer iNumProcs, iErr, iMyRank
      common/mpistuff/iNumProcs,iMyRank

#ifdef mpiuse
      call MPI_Init(iErr)
      call MPI_Comm_size(MPI_COMM_WORLD, iNumProcs, iErr)
      call MPI_Comm_rank(MPI_COMM_WORLD, iMyRank, iErr)
#else
      iMyRank=0
      iNumProcs=1
#endif

      if(iMyRank.EQ.0) then
	Print *,""
        print *,
     &      '======================================================='
        print *,' This is HAWK, version 1.1                         '
        print *,' released June 10, 2010                            '
        print *,' authors: A. Denner, S. Dittmaier, A. Mueck        '
        print *,
     &      '======================================================='
#ifdef mpiuse
	Print *,"(running on ",iNumProcs," cores)"
#endif
	Print *,""
      endif

c     write(*,*) 'start program'
c initialization
      call initialization
c numerical integration
      unweighting=0
      call integration()
c unweighting events
c      unweighting=1
c      nunwevents=0
c      do i1=1,nunwevents
c        call eventgeneration(unweighting)
c      enddo
c write result
      call writeresult
#ifdef G77
      call flush
#endif
#ifdef PGF77
      call flush(nout)
#endif

#ifdef mpiuse
       call MPI_Finalize(iErr)
#endif

      stop

      s = energy**2
      m2 = mcm(25)**2

      if (m2.ne.0d0) then
        ps3 = 1d0/(2*pi)**5*(pi/2)**2/2/s
     &      *(s**2-m2**2+2*m2*s*log(m2/s))
        ps4 = 1d0/(2*pi)**8*(pi/2)**3/12/s
     &      *((s-m2)*(s**2+10*m2*s+m2**2)+6*m2*s*(s+m2)*log(m2/s))
      else
        ps3 = 1d0/(2*pi)**5*(pi/2)**2/2*s
        ps4 = 1d0/(2*pi)**8*(pi/2)**3/12*s**2
      endif

      write(*,*) 's   = ',s
      write(*,*) 'mh2 = ',m2
      write(*,*) 'ps3 = ',ps3*mcgevtofb/2/s
      write(*,*) 'ps4 = ',ps4*mcgevtofb/2/s

      end


***********************************************************************
      subroutine initialization
***********************************************************************
*     initialization of phase space integration                       *
*     adapted from COFFERGG by Markus Roth                            *
*---------------------------------------------------------------------*
*     08.03.06  Ansgar Denner     last changed  22.02.10              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

#ifdef RANLUX
      integer ranluxseed
      common/ranluxseed/ranluxseed
#endif

#ifdef multpdf
#include "multpdf.h"
      integer nset
#endif
#ifdef mpiuse
      include "mpif.h"
#endif
      integer iNumProcs, iErr, iMyRank
      common/mpistuff/iNumProcs,iMyRank

c local variables
c      real*8 scale,gamma,sppmin,dir(maxe)
      real*8 sppmin,mtotcut
      logical endname
      integer i1
      integer switchinitrans
      character*3 madname(-maxv:maxv)

c date and time
      real*8     rcpu
c     integer*4  int,itime,iminit,ihour,icpu,icpuh,icpum,icpus
c     integer*4  int,itime,icpu,icpuh,icpum,icpus
      integer*4  int,icpu,icpuh,icpum,icpus
c     integer    idate
#ifdef PGF77
      integer    imonth,iyear,iday
      real tarray(2),etime
#endif
c      integer*4  no,lpar,il,iu,id
c      character  parstr*6,prostr*22
      character  datum*10,da1*2,da2*2,da3*4
c      character  fdatum*10,fdate
      character  chtime*8
      integer iarray(3)

      real*8  qu,qd,ql,qn,qf,mu,mu2,md,md2
      complex*16 guu,gdd
      common/qf/qu,qd,ql,qn,qf(4),guu(-1:1),gdd(-1:1),mu,mu2,md,md2

      real*8 constff,constfi,constif,constii
      common/constsub/constff,constfi,constif,constii

      save switchinitrans

c initialize
      av=0d0
      si=0d0
      average=0d0
      sigma=0d0
      sigmaa=0d0
      avborn=0d0
      siborn=0d0
#ifdef multpdf
          do nset=1,pdfmemberto-pdfmemberfrom+1
	    maverage(nset)=0d0
	    msigma(nset)=0d0
	    msigmaa(nset)=0d0
            mavborn(nset)=0d0
            msiborn(nset)=0d0
	  enddo
#endif
      wmax=0d0
      weighttotmax=0d0
      nunw=0
      nneg=0
      nevt=1
      nmaxtot=0
      do icut=1,maxcut
        mccutevt_0(icut)=0d0
      enddo
c      do iexit=1,maxexit
c        mcexitevtborn(iexit)=0d0
c      enddo
      do gen=0,maxg
        do icon=1,maxcon
          do icut=1,maxcut
            do iphsp=1,maxps
              mccutsubevt(icut,gen,icon,iphsp)=0d0
            enddo
            mccutevt(icut,gen,icon)=0d0
          enddo
        enddo
        do iexit=1,maxexit
          mcexitevt(iexit,gen)=0d0
        enddo
        do icon=1,maxcon
          processname(gen)=' '
          avgen(gen,icon)=0d0
          sigen(gen,icon)=0d0
#ifdef multpdf
          do nset=1,pdfmemberto-pdfmemberfrom+1
            mavgen(nset,gen,icon)=0d0
            msigen(nset,gen,icon)=0d0	
	  enddo
#endif
          nevtgen(gen,icon)=0d0
          weightmax(gen,icon)=0d0
          rejgen(gen,icon)=0
          nmax(gen,icon)=0
          channelmax(gen,icon)=0
          do cha=1,maxch
            avgencha(gen,icon,cha)=0d0
            sigencha(gen,icon,cha)=0d0
            weightmaxcha(gen,icon,cha)=0d0
            nevtgencha(gen,icon,cha)=0
            rejcha(gen,icon,cha)=0
            nmaxcha(gen,icon,cha)=0
          enddo
        enddo
      enddo
#ifdef SUBPROCESSES
      do parton1=-4,4
      do parton2=-4,4
        averageii(parton1,parton2)=0d0
        sigmaii(parton1,parton2)=0d0
        avbornii(parton1,parton2)=0d0
        sibornii(parton1,parton2)=0d0
        do gen=0,maxg
          do icon=1,maxcon
            avgenii(gen,icon,parton1,parton2)=0d0
            sigenii(gen,icon,parton1,parton2)=0d0
          enddo
        enddo
      enddo
      enddo
#endif

      do ipart=-maxip,maxip
        hepname(ipart)=' '
      enddo
      do ipart=-maxip,maxip
        class(ipart)=' '
      enddo
      do mcpart=1,maxv
        mcm(mcpart)=0d0
        mcw(mcpart)=0d0
      enddo
      do gen=0,maxg
      do ext=1,maxe
        hepnum(ext,gen)=0
        name(ext,gen)=' '
        ecut(ext,gen)=0d0
        ptcut(ext,gen)=0d0
        ycut(ext,gen)=1d10
        ecutgen(ext,gen)=0d0
        do ext2=1,maxe
          dycut(ext,ext2,gen)=0d0
          dRcut(ext,ext2,gen)=0d0
          scut(ext,ext2,gen)=0d0
          ccut(ext,ext2,gen)=0d0
          scutgen(ext,ext2,gen)=0d0
          ccutgen(ext,ext2,gen)=0d0
        enddo
      enddo
      enddo
      mtotcut=0d0

      xpmax=0
      xpmin=1
      x1max=0
      x1min=1
      x2max=0
      x2min=1
      x3max=0
      x3min=1


      ecuth=0d0          ! Higgs energy cut
      ecutj=0d0          ! jet energy cut
      ecutp=0d0          ! photon energy cut
      ptcuth=0d0         ! Higgs pt cut
      ptcutj=0d0         ! jet pt cut
      ptcutp=0d0         ! photon pt cut
      ptcuts=0d0         ! singlet pt cut
      ycuth=1d10         ! Higgs rapidity cut
      ycutj=1d10         ! jet rapidity cut
      ycutp=1d10         ! photon rapidity cut
      ycuts=1d10         ! singlet rapidity cut
      ccuthb=0d0         ! angular cut between Higgs and beam
      ccutjb=0d0         ! angular cut between jet and beam
      ccutpb=0d0         ! angular cut between photon and beam
      ccutjj=0d0         ! angular cut between two jets
      ccutjh=0d0         ! angular cut between Higgs and jet
      ccutjp=0d0         ! angular cut between Photon and jet
      dycutjj=0d0        ! rapidity cut between two jets
      dycutjh=0d0        ! rapidity cut between Higgs and jet
      dycutjp=0d0        ! rapidity cut between Photon and jet
      dRcutjj=0d0        ! R cut between two jets
      dRcutjh=0d0        ! R cut between Higgs and jet
      dRcutjp=0d0        ! R cut between Photon and jet
      dRcutjs=0d0        ! R cut between Singlet and jet
      mcutjj=0d0         ! invariant mass cut between two jets
      mcutjh=0d0         ! invariant mass cut between Higgs and jet
      mcutjp=0d0         ! invariant mass cut between Photon and jet
      mlcuth=1d-2        ! lower cut on Higgs invariant mass
      mucuth=1d30        ! upper cut on Higgs invariant mass

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     setting parameters                                          c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c fixing SM input
      call inputparameter

c reading input
      call readinput

c open outputfile
      endname=.false.
      do i1=1,50
        if(outputfile(i1:i1).eq.' ') endname=.true.
        if(endname)outputfile(i1:i1)=' '
      enddo
      if(outputfile(1:1).eq.' ')then
        nout=6
      else
        nout=22
        open(unit=nout,file=outputfile,status='unknown')
      endif


c fixing dependent parameters
      call fixparameter

 97   format(i2.2,'.',i2.2,'.20',i2.2)
 98   format(i2.2,'.',i2.2,'.',i2.2)
#ifdef G77
      call date_and_time(datum)
      da1=datum(7:8)
      da2=datum(5:6)
      da3=datum(1:4)
      datum=da1//'.'//da2//'.'//da3
      call itime(iarray)
      write(chtime,98)iarray(1),iarray(2),iarray(3)
      call cpu_time(rcpu)
#endif
#ifdef PGF77
      call idate(imonth,iday,iyear)
      write(datum,97) iday,imonth,iyear
      call itime(iarray)
      write(chtime,98)iarray(1),iarray(2),iarray(3)
      rcpu = etime(tarray)
#endif
#ifdef IFORT77
      call date_and_time(datum)
      da1=datum(7:8)
      da2=datum(5:6)
      da3=datum(1:4)
      datum=da1//'.'//da2//'.'//da3
      call itime(iarray)
      write(chtime,98)iarray(1),iarray(2),iarray(3)
      call cpu_time(rcpu)
#endif
      icpuh=int(rcpu/3600.)
      icpum=int(rcpu/60.-icpuh*60)
      icpus=int(rcpu-icpuh*3600-icpum*60)
      icpu =icpuh*60+icpum
      If(imyRank.EQ.0) Then
        write(nout,100) datum,chtime,icpuh,icpum,icpus
100     format(/'  date: ',a10,' ,    time: ',a8,' hour,',
     &       '    cpu: ',i3,' h',i3,' m',i3,' s')
      Endif

c setting parameters for amplitudes
      call initqqqqhamps
      If(imyRank.EQ.0) Then
        call paramout
      Endif

c initialize names of possible external particles
      call init_particles()

c setting parmeters for monte carlo generator
      call mcparameter


      If(imyRank.EQ.0) Then
        write(nout,'(a)')' '
        write(nout,'(a)')'                       Monte Carlo'
        write(nout,'(a)')'                       ==========='
      Endif

      do gen=0,ngenerator
c setting external particle parameters
        processname(gen)=name(1,gen)//' '//name(2,gen)//' -> '
        do ext=3,nextgen(gen)
          processname(gen)=processname(gen)(1:54)//name(ext,gen)
          do i1=70,2,-1
            if(processname(gen)(i1-1:i1).eq.'  ')
     &          processname(gen)=processname(0)(1:i1-2)//
     &          processname(gen)(i1:70)
          enddo
        enddo

        do ext=1,nextgen(gen)
          do ipart=-maxip,maxip
            if(name(ext,gen).eq.hepname(ipart)) hepnum(ext,gen)=ipart
c     write(*,*) name(ext,0),ipart,hepname(ipart)
          enddo
          if(hepnum(ext,gen).eq.0)then
            write(*,'(a24,a16)')' Unknown particle name: ',name(ext,gen)
            write(*,'(a16,2x,i5)')' Model: smodel= ',smodel
            stop
          endif
      enddo
      enddo

      If(imyRank.EQ.0) Then
        write(nout,200)  processname(0)
 200    format(' Input process:  ',a60)

        if(lnoutmc.ge.3)then
          write(nout,300) hepnum(1,0),hepnum(2,0),hepnum(3,0),
     &        (hepnum(ext,0),ext=4,nextgen(0))
 300      format(' Input process:  ',i3,' + ',i3,' -> ',
     &        i3,(9(:,' + ',i3,:)))
        endif
      Endif

c ADAPT
c input-parameter scheme
      if(salp.eq.0.or.salp.eq.3)then
	mcalpha=mcalpha0
      elseif(salp.eq.1)then
	mcalpha=mcalphaz
      elseif(salp.eq.2)then
	mcalpha=sqrt(2d0)*mcgf/pi*mcm(24)**2*(1d0-mccw**2)
      else
        write(*,'(a)')
     &      ' Unknown input for input-parameter scheme (salp)!'
        stop
      endif
c      mcel=sqrt(4d0*pi*mcalpha)

c inititalize beam momenta
      pbeam(1,0)=0.5d0*energy
      pbeam(1,1)=0d0
      pbeam(1,2)=0d0
      pbeam(1,3)=0.5d0*energy
      pbeam(2,0)=0.5d0*energy
      pbeam(2,1)=0d0
      pbeam(2,2)=0d0
      pbeam(2,3)=(-0.5d0)*energy

c definition of end-point part of subtraction functions (const=Cij-1/2)
      do ext=1,maxe
      do ext2=1,maxe
        if(ssoft.eq.1)then
c racoonww paper eq(4.29)
          if(ext.ge.3.and.ext2.ge.3)const(ext,ext2)=1d0-pi2/3d0
          if(ext.ge.3.and.ext2.le.2)const(ext,ext2)=1d0-pi2/2d0
          if(ext.le.2.and.ext2.ge.3)const(ext,ext2)=-1.5d0+pi2/6d0
          if(ext.le.2.and.ext2.le.2)const(ext,ext2)=1.5d0-pi2/3d0
        elseif(ssoft.eq.2)then
          const(ext,ext2)=-0.5d0
c Cij = 0
        elseif(ssoft.eq.3)then
c YFS inspired
          if(ext.ge.3.and.ext2.ge.3)const(ext,ext2)=0.5d0-pi2*2d0/3d0
          if(ext.ge.3.and.ext2.le.2)const(ext,ext2)=0.5d0-pi2/6d0
          if(ext.le.2.and.ext2.ge.3)const(ext,ext2)=0.5d0-pi2/6d0
          if(ext.le.2.and.ext2.le.2)const(ext,ext2)=0.5d0-pi2*2d0/3d0
        endif
      enddo
      enddo

c output
      numout=0
c     maxout=100       ! maximal number of error messages

c initialize phase-space generator
      mcmass(0)=0d0
      mcwidth(0)=0d0
      do mcpart=1,maxv
        mcmass(mcpart)=mcm(mcpart)
        mcwidth(mcpart)=mcw(mcpart)
      enddo


c neglect light-fermion masses
      if(slightfermions.eq.1)then
        do mcpart=1,4
          mcmass(mcpart)=0d0
        enddo
        do mcpart=11,13,2
          mcmass(mcpart)=0d0
        enddo
      elseif(slightfermions.eq.2)then
        do mcpart=1,5
          mcmass(mcpart)=0d0
        enddo
        do mcpart=11,15,2
          mcmass(mcpart)=0d0
        enddo
      endif

c initialize histograms


      call settings(pbeam,pevent,weight(1,1),0,0,1)


ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     initialize subcontributions                                 c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      do gen=0,ngenerator
      if(sgen(gen).ne.0)then
c process name
        processname(gen)=name(1,gen)//' '//name(2,gen)//' -> '
        do ext=3,nextgen(gen)
          processname(gen)=processname(gen)(1:54)//name(ext,gen)
          do i1=70,2,-1
            if(processname(gen)(i1-1:i1).eq.'  ')
     &          processname(gen)=processname(gen)(1:i1-2)//
     &          processname(gen)(i1:70)
          enddo
        enddo

c initialize flux and phasespacefactor
        factor(gen)=mcgevtofb/(2d0*pi)**(3*nextgen(gen)-10)
     &    /(2d0*sqrt(energy**4
     &      -mcm(iabs(hepnum(1,gen)))**2*mcm(iabs(hepnum(2,gen)))**2))


c definitions for subtraction functions
        lambda2(gen)=mclambda*mclambda

c properties of external particles
        do ext=1,nextgen(gen)

c masses of external particles (for subtraction functions)
c does not work for composite particles!!!!
          mass2(ext,gen)=mcm(abs(hepnum(ext,gen)))**2


c direction of external particles

c charges of external fermions
          i1=abs(hepnum(ext,gen))


          mccharge(ext,gen)=0d0
          if(i1.eq.101.or.i1.eq.102) then
            mccharge(ext,gen)=1d20   ! only switch!
          else if(i1.eq.1.or.i1.eq.3.or.i1.eq.5) then
            mccharge(ext,gen)=-(1d0/3d0)
          else if(i1.eq.2.or.i1.eq.4.or.i1.eq.6) then
            mccharge(ext,gen)=2d0/3d0
          else if(i1.eq.11.or.i1.eq.13.or.i1.eq.15) then
            mccharge(ext,gen)=-1d0
          endif

        enddo

c photon recombination
        if(srecomb(gen).ne.0)then
          do ext=1,nextsub(gen)
            recparticle(ext,gen)=0
            if(mccharge(ext,gen).ne.0d0)recparticle(ext,gen)=1
          enddo
          if(srecomb(gen).eq.1.or.srecomb(gen).eq.2)then
            continue
          elseif(srecomb(gen).eq.3)then
            recparam(3,gen)=recparam(3,gen)*recparam(3,gen)
            recparam(4,gen)=recparam(4,gen)*recparam(4,gen)
          elseif(srecomb(gen).eq.4)then
            recparam(1,gen)=dcos(pi/180d0*recparam(1,gen))
            recparam(3,gen)=dcos(pi/180d0*recparam(3,gen))
            recparam(4,gen)=recparam(4,gen)*recparam(4,gen)
          else
            write(*,*)' initialization: wrong input for srecomb:',
     &          srecomb(gen),gen
            stop
          endif
        endif


*#####################################################################
* shift to other subroutine and adapt!  init_cuts
c standard cuts for pp -> qqH
        if(scuts(gen).eq.0.and.shtr.eq.1)then
          sucuth=mucuth*mucuth
          slcuth=mlcuth*mlcuth
        elseif(scuts(gen).eq.1.or.scuts(gen).eq.2)then
          ptcut(3,gen)=ptcutj
          ptcut(4,gen)=ptcutj
          ycut(3,gen)=ycutj
          ycut(4,gen)=ycutj
          dycut(3,4,gen)=dycutjj
          dycut(4,3,gen)=dycutjj
          scut(3,4,gen)=mcutjj
          scut(4,3,gen)=mcutjj
          if(shtr.ge.2)then
            ptcut(5,gen)=ptcuts
            ptcut(6,gen)=ptcuts
            ycut(5,gen)=ycuts
            ycut(6,gen)=ycuts
            dRcut(3,5,gen)=dRcutjs
            dRcut(3,6,gen)=dRcutjs
            dRcut(4,5,gen)=dRcutjs
            dRcut(4,6,gen)=dRcutjs
          endif
          sucuth=mucuth*mucuth
          slcuth=mlcuth*mlcuth

        elseif(scuts(gen).eq.3)then
c         ptcutj=20d0
c         ycutj=4.5d0
          ptcut(3,gen)=ptcutj
          ptcut(4,gen)=ptcutj
          ycut(3,gen)=ycutj
          ycut(4,gen)=ycutj

c cuts for tests for p p -> q q H
        elseif(scuts(gen).eq.8)then
          mtotcut=1d3
        elseif(scuts(gen).eq.9)then
c         ptcutH=50d0        ! Higgs pT cut
        elseif(scuts(gen).ge.4.and.scuts(gen).le.10)then
          write(*,'(a30,i3)')' Unknown input: scuts = ',scuts
          stop
        endif
c filling cut parameters


        if(scuts(gen).gt.3.and.scuts(gen).le.10)then
          do ext=3,nextgen(gen)


            if(class(hepnum(ext,gen)).eq.'charged lepton')then
            elseif(class(hepnum(ext,gen)).eq.'quark')then
            elseif(hepname(hepnum(ext,gen)).eq.'photon')then
              ecut(ext,gen)=ecutp
              ptcut(ext,gen)=ptcutp
              ycut(ext,gen)=ycutp
              ccut(1,ext,gen)=ccutpb
              ccut(2,ext,gen)=ccutpb
            elseif(hepname(hepnum(ext,gen)).eq.'jet')then
              ecut(ext,gen)=ecutj
              ptcut(ext,gen)=ptcutj
              ycut(ext,gen)=ycutj
              ccut(1,ext,gen)=ccutjb
              ccut(2,ext,gen)=ccutjb
            elseif(hepname(hepnum(ext,gen)).eq.'gluon')then
              ecut(ext,gen)=ecutj
              ptcut(ext,gen)=ptcutj
              ycut(ext,gen)=ycutj
              ccut(1,ext,gen)=ccutjb
              ccut(2,ext,gen)=ccutjb
            elseif(hepname(hepnum(ext,gen)).eq.'h0')then
              ecut(ext,gen)=ecutH
              ptcut(ext,gen)=ptcutH
              ycut(ext,gen)=ycutH
              ccut(1,ext,gen)=ccutHb
              ccut(2,ext,gen)=ccutHb
            endif
            do ext2=3,nextgen(gen)
              if(hepname(hepnum(ext,gen)).eq.'jet'.and.
     &          hepname(hepnum(ext2,gen)).eq.'jet')then
                scut(ext,ext2,gen)=mcutjj
                ccut(ext,ext2,gen)=ccutjj
                dycut(ext,ext2,gen)=dycutjj
                dRcut(ext,ext2,gen)=dRcutjj
              elseif(hepname(hepnum(ext,gen)).eq.'jet'.and.
     &          hepname(hepnum(ext2,gen)).eq.'h0')then
                scut(ext,ext2,gen)=mcutjh
                ccut(ext,ext2,gen)=ccutjh
                dycut(ext,ext2,gen)=dycutjh
                dRcut(ext,ext2,gen)=dRcutjh
              elseif(hepname(hepnum(ext2,gen)).eq.'jet'.and.
     &          hepname(hepnum(ext,gen)).eq.'h0')then
                scut(ext,ext2,gen)=mcutjh
                ccut(ext,ext2,gen)=ccutjh
                dycut(ext,ext2,gen)=dycutjh
                dRcut(ext,ext2,gen)=dRcutjh
              elseif(hepname(hepnum(ext,gen)).eq.'jet'.and.
     &          hepname(hepnum(ext2,gen)).eq.'photon')then
                scut(ext,ext2,gen)=mcutjp
                ccut(ext,ext2,gen)=ccutjp
                dycut(ext,ext2,gen)=dycutjp
                dRcut(ext,ext2,gen)=dRcutjp
              elseif(hepname(hepnum(ext2,gen)).eq.'jet'.and.
     &          hepname(hepnum(ext,gen)).eq.'photon')then
                scut(ext,ext2,gen)=mcutjp
                ccut(ext,ext2,gen)=ccutjp
                dycut(ext,ext2,gen)=dycutjp
                dRcut(ext,ext2,gen)=dRcutjp
              endif
            enddo
          enddo
        endif
c calculating cut parameters
        do ext=1,nextgen(gen)
        do ext2=1,nextgen(gen)
          ccut(ext,ext2,gen)=dcos(ccut(ext,ext2,gen)*pi/180d0)
          scut(ext,ext2,gen)=scut(ext,ext2,gen)**2
        enddo
        enddo
        do ext=1,nextgen(gen)
        do ext2=1,nextgen(gen)
          ccut(ext,ext2,gen)=min(ccut(ext,ext2,gen),ccut(ext2,ext,gen))
          scut(ext,ext2,gen)=max(scut(ext,ext2,gen),scut(ext2,ext,gen))
        enddo
        enddo

c calculating cut parameters to be given to the generator
        mtotcutgen=0d0
        do ext=3,nextgen(gen)
          mtotcutgen=mtotcutgen+sqrt(ptcut(ext,gen)**2+mass2(ext,gen))
          do ext2=1,nextgen(gen)
c ccutgen must be set to 1 if there are no cuts!
            ccutgen(ext2,ext,gen)=ccut(ext2,ext,gen)
            scutgen(ext2,ext,gen)=scut(ext2,ext,gen)
          enddo
        enddo
        mtotcutgen=max(mtotcutgen,mtotcut)


*#######################################################################


c initializing phase space generators
        if(lnoutgen.gt.0)then
          If(imyRank.EQ.0) Then
            write(nout,'(a)') ' '
            write(nout,'(a)') ' Initializing phase-space generators'
          Endif
        endif
        nchannel(gen)=0
        nphsp(gen)=maxps       ! to ensure correct initialization of
                               ! arrays in integrand for first event
        ncontr(gen)=maxcon     ! to ensure correct initialization of
                               ! arrays in integrand for first event

c determine partonic processes
        if(shtr.eq.0)then
          call init_subprocesses(energy,sppmin,gen,
     &        nextgen(gen),smodel,sincludecuts(gen),
     &        ssub(gen)+ssli(gen)+seff(gen))
        else
          call init_subprocesses(energy,sppmin,gen,
     &        nextgen(gen),smodel,sincludecuts(gen),
     &        ssub(gen)+ssli(gen)+seff(gen))
        endif


       if(sincludecuts(gen).ge.2) sppmin=max(sppmin,mtotcutgen**2)

       if(lnoutgen.gt.0)then
         If(imyRank.EQ.0) Then
           write(nout,*) 'initgenerator sppmin=',sppmin,sqrt(sppmin)
         endif
       endif

        xmin(gen)=sppmin/energy**2
        if(xmin(gen).gt.1d0)then
          write(*,'(a)')' error: energy too low for particle production'
          stop
        endif

c initialize adaptation
        noptran(gen)=2000*nchannel(gen)
        alminran=0.2d0
        do igrv=1,maxr
          beran(0,igrv,gen)=0d0
          do igrp=1,maxgr
            wran(igrp,igrv,gen)=0d0
            beran(igrp,igrv,gen)=dble(igrp)/dble(maxgr)
            alran(igrp,igrv,gen)=1d0/dble(maxgr)
          enddo
        enddo
      endif
      enddo

c initialize random number generator
      if(switchinitrans.eq.0)then
          If(imyRank.EQ.0) Then
            write(nout,'(a)')
            write(nout,'(a)') " Initializing Random Number Generator"
          Endif
#ifdef RANLUX
        call rluxgo(3,ranluxseed,0,0)
c       call rluxgo(0,ranluxseed,0,0)
#else
        call initcarry
#endif
        switchinitrans=1
      endif

c write output
      If(imyRank.EQ.0) Then
        call writeoutput
      Endif

#ifdef G77
      call flush
#endif
#ifdef PGF77
      call flush(nout)
#endif

      end

***********************************************************************
      subroutine writeoutput
***********************************************************************
*     writes output                                                   *
*     adapted from COFFERGG by Markus Roth                            *
*---------------------------------------------------------------------*
*     21.02.06  Ansgar Denner     last changed  31.03.10              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"
#ifdef multpdf
#include "multpdf.h"
#endif

c local variables
      integer i1
      character*70 contentname


c output
      write(nout,'(a)')' '
      write(nout,'(a)')' Input parameters:'
      write(nout,'(a)')' '
 1000 format(9x,'energy = ',f18.12,',',/,
     *  7x,'alpha(0) = 1/', f16.12,',',2x,'alpha(MZ) = 1/',f16.12,',',/,
     *  12x,'G_F = ',g18.8,',',6x,'alpha = 1/',f16.12,',',/,
     *  4x,'alpha_s(MZ) = ',f18.12,',',4x,'alpha_s = ',f18.12,'.',/)
 2000 format(12x,'mgl = ',f18.12,',',8x,'mmu = ',f18.12,',',/,
     *  12x,'ma0 = ',f18.12,',',9x,'tb = ',f18.12,',',/,
     *  13x,'m1 = ',f18.12,',',9x,'m2 = ',f18.12,',',/,
     *  10x,'msusy = ',f18.12,',',7x,'atau = ',f18.12,',',/,
     *  13x,'at = ',f18.12,',',9x,'ab = ',f18.12,'.',/)
 3000 format(9x,'msl(1) = ',f9.4,',  msl(2) = ',f9.4,
     *  ',  msl(3) = ',f9.4,',',/,
     *  9x,'mse(1) = ',f9.4,',  mse(2) = ',f9.4,
     *  ',  mse(3) = ',f9.4,',',/,
     *  9x,'msq(1) = ',f9.4,',  msq(2) = ',f9.4,
     *  ',  msq(3) = ',f9.4,',',/,
     *  9x,'msu(1) = ',f9.4,',  msu(2) = ',f9.4,
     *  ',  msu(3) = ',f9.4,',',/,
     *  9x,'msd(1) = ',f9.4,',  msd(2) = ',f9.4,
     *  ',  msd(3) = ',f9.4,'.',/)
      write(nout,1000)energy,1d0/mcalpha0,1d0/mcalphaz,mcgf,1d0/mcalpha,
     *  mcalphasz,mcalphas
      if(smodel.eq.3.or.smodel.eq.4.or.smodel.eq.34)then
        write(nout,2000)mgl,mue,ma0,tb,mgo1,mgo2,msusy,atau,at,ab
        write(nout,3000)msl(1),msl(2),msl(3),mse(1),mse(2),mse(3),
     &    msq(1),msq(2),msq(3),msu(1),msu(2),msu(3),msd(1),msd(2),msd(3)
      endif

      if(lnoutmc.ge.3)then
c output particles
        write(nout,'(a)')' '
        write(nout,'(a)')
     &      ' Particle           Mass                 Width'
        do ipart=1,ncomp
          if(hepname(ipart).ne.' ')then
            if(mcm(ipart).ge.1d0.and.mcw(ipart).ne.0d0)then
              write(nout,'(1x,a15,f15.10," GeV  ",f15.10," GeV")')
     &            hepname(ipart),mcm(ipart),mcw(ipart)
            elseif(mcm(ipart).gt.0d0.and.mcw(ipart).ne.0d0)then
              write(nout,'(1x,a15,f15.10," MeV  ",f15.10," MeV")')
     &            hepname(ipart),1d3*mcm(ipart),1d3*mcw(ipart)
            elseif(mcm(ipart).ge.1d0.and.mcw(ipart).eq.0d0)then
              write(nout,'(1x,a15,f15.10," GeV")')
     &            hepname(ipart),mcm(ipart)
            elseif(mcm(ipart).gt.0d0.and.mcw(ipart).eq.0d0)then
              write(nout,'(1x,a15,f15.10," MeV")')
     &            hepname(ipart),1d3*mcm(ipart)
            elseif(mcm(ipart).eq.0d0.and.mcw(ipart).eq.0d0)then
              write(nout,'(1x,a15,"    massless")')hepname(ipart)
            else
              write(*,'(a)')' writeoutput: width for massless particle!'
              stop
            endif
          endif
        enddo
        write(nout,'(a)')' '
        write(nout,'(a)')' Composite particle contains partons        '
        do ipart=ncomp,maxip
          if(hepname(ipart).ne.' ')then
            contentname = ' '
            do mcpart=-naux+1,naux-1
              if (partcont(ipart,mcpart)) then
                contentname=contentname(1:54)//hepname(mcpart)
                do i1=70,2,-1
                  if(contentname(i1-1:i1).eq.'  ')
     &                contentname=contentname(1:i1-2)//
     &                contentname(i1:70)
                enddo
              endif
            enddo
            write(nout,'(1x,a15,a50)')
     &          hepname(ipart),contentname(1:50)
          endif
        enddo
      endif

c output factorization scales
      call getqcdfacscale(mcm(24))
      call getqedfacscale(mcm(24))
      write(nout,*)
      write(nout,'(a28,1g20.10)')'QCD factorization scale   =',
     &    qcdfacscale
      write(nout,'(a28,1g20.10)')'QED factorization scale   =',
     &    qedfacscale

c output renormalization scales
      call getqcdrenscale(mcm(24))
      write(nout,'(a28,1g20.10)')'QCD renormalization scale =',
     &    qcdrenscale

      if(lnoutmc.ge.5) then
c output mapping parameters
      write(nout,*)	
      write(nout,'(a33,1g20.10)')'propagator mapping with power = ',
     &    powerprop
      write(nout,'(a33,1g20.10)')'propagator mapping with cut   = ',
     &    techparam(1)
      write(nout,'(a33,1g20.10)')'allowed uncertainty for tmax  = ',
     &    techparam(2)
      write(nout,'(a33,1g20.10)')'allowed uncertainty for P_T,in= ',
     &    techparam(3)
      write(nout,'(a33,1g20.10)')'mapping of x with cut         = ',
     &    techparam(4)
      write(nout,'(a33,1g20.10)')'1/(1-x) mapping with power = ',
     &    powersplitf
      write(nout,'(a33,1g20.10)')'ln(1-x) mapping with power = ',
     &    powersplitp
        write(nout,'(a33,1g20.10)')'phase-space mapping with power= ',
     &      powerexmap
      endif

c output technical parameters
      write(nout,*)
      if(ssli(0).ne.0)then
        write(nout,'(a33,1e20.10)')'slicing cut:           deltas = ',
     &      mcslideltas
        write(nout,'(a33,1e20.10)')'slicing cut:           deltac = ',
     &      mcslideltac
      endif
      if(seff(0).ne.0)then
        write(nout,'(a33,1e20.10)')'coll. mass param.:  mcollreg2 = ',
     &      mcollreg2
        write(nout,'(a33,1e20.10)')'technical cut:  deltaspk/shat = ',
     &      mceffcut
      endif
      if(lnoutmc.ge.5) then
      if(ssub(0).ne.0)then
        write(nout,'(a33,1e20.10)')'subtraction cut:       deltas = ',
     &      tcdeltas
        write(nout,'(a33,1e20.10)')'subtraction cut:       deltae = ',
     &      tcdeltae
        write(nout,'(a33,1e20.10)')'subtraction cut:       deltac = ',
     &      tcdeltac
      endif
      write(nout,'(a33,1e20.10)')'splittingfunction cut:omxmaxf = ',
     &    omxmaxf
      write(nout,'(a33,1e20.10)')'splittingfunction cut:omxmaxp = ',
     &    omxmaxp
      endif

      if(lnoutmc.ge.3) then
        write(nout,'(a29,i2)')'mapping of pdfs: pdfmap = ',
     &      pdfmap
      endif

c output options
      write(nout,'(a)')' '

      if(spdf.eq.1)then
        write(nout,'("           spdf=",i2,(a))') spdf,
     &      ': MRST MRST2004QED [with O(alpha) effects]'
      elseif(spdf.eq.2)then
        write(nout,'("           spdf=",i2,(a))') spdf,
     &      ': MRST MRST2004QED [without O(alpha) effects]'
      elseif(spdf.eq.11)then
        write(nout,'("           spdf=",i2,(a))') spdf,
     &      ': CTEQ6M [Standard MSbar scheme (alphas(mz)=0.118)]'
      elseif(spdf.eq.14)then
        write(nout,'("           spdf=",i2,(a))') spdf,
     &      ': CTEQ6L1 [Leading Order (alphas(mz)=0.130)]'
#ifdef includeLHAPDF
      elseif(spdf.eq.99)then
        if(pdfpath.ne.'')then
          write(nout,'("           spdf=",i2,(a),/20x,"pdfset ",i2)')
     &        spdf,': LHApdf: '//pdfpath,pdfmember
        else
          write(nout,'("           spdf=",i2,(a),/20x,"pdfset ",i2)')
     &        spdf,': LHApdf: '//pdfname,pdfmember
        endif
#endif
      elseif(sconv(gen).ne.0)then
        write(*,'(a23,i4)')' Unknown input: spdf = ',spdf
        stop
      endif

#ifdef multpdf
      If(pdfmemberto.GE.pdfmemberfrom) Then
        write(nout,*) ''
        write(nout,*) 'Usage of multiple PDF sets requested:'
	write(nout,'(" for sets ",i4," to ",i4,
     &              " results will be provided")'),
     &                pdfmemberfrom,pdfmemberto
        write(nout,*) ''
      Endif
#endif
      if(lnoutmc.ge.3)then
c model
        if(smodel.eq.12)then
          write(nout,'(a15,i3,(a))')'smodel=',smodel,
     &        ': Standard Model with QCD'
        elseif(smodel.eq.1)then
          write(nout,'(a15,i3,(a))')'smodel=',smodel,
     &        ': Standard Model without QCD'
        elseif(smodel.eq.2)then
          write(nout,'(a15,i3,(a))')'smodel=',smodel,
     &        ': QCD'
        elseif(smodel.eq.15)then
          write(nout,'(a16,i3,(a))')'smodel=',smodel,
     &        ': Standard Model with Higgs singlet without: QCD'
        elseif(smodel.eq.125)then
          write(nout,'(a16,i3,(a))')'smodel=',smodel,
     &        ': Standard Model with Higgs singlet and QCD'
        elseif(smodel.eq.34)then
          write(nout,'(a15,i3,(a))')'smodel=',smodel,
     &        ': MSSM with SQCD'
        elseif(smodel.eq.3)then
          write(nout,'(a15,i3,(a))')'smodel=',smodel,
     &        ': MSSM without SQCD'
        elseif(smodel.eq.4)then
          write(nout,'(a15,i3,(a))')'smodel=',smodel,
     &        ': SQCD'
        else
          write(*,'(a30,i3)')' Unknown input: smodel = ',smodel
          stop
        endif

c input-parameter scheme
        if(salp.eq.0)then
          write(nout,'(a16,i2,(a))')'salp=',salp,
     &        ': alpha(0)-parametrization scheme'
        elseif(salp.eq.1)then
          write(nout,'(a16,i2,(a))')'salp=',salp,
     &        ': alpha(MZ)-parametrization scheme'
        elseif(salp.eq.2)then
          write(nout,'(a16,i2,(a))')'salp=',salp,
     &        ': GF-parametrization scheme'
        elseif(salp.eq.3)then
          write(nout,'(a16,i2,(a))')'salp=',salp,
     &        ': mixed alpha(0)-alpha_GF-parametrization scheme'
        else
          write(*,'(a)')' Unkown input: salp = ',salp
          stop
        endif

        if(scp.eq.0)then
          write(nout,'(a16,i2,(a))')'scp=',scp,
     &        ': CP symmetry not used'
        elseif(scp.eq.1)then
          write(nout,'(a16,i2,(a))')'scp=',scp,
     &        ': CP symmetry used in lowest order matrix elements'
        else
          write(*,'(a30,i3)')' Unknown input: scp = ',scp
          stop
        endif

        if(swdiag.eq.0)then
          write(nout,'(a16,i2,(a))')'swdiag=',swdiag,
     &        ': W-exchange diagrams excluded '
        elseif(swdiag.eq.1)then
          write(nout,'(a16,i2,(a))')'swdiag=',swdiag,
     &        ': W-exchange diagrams included '
        else
          write(*,'(a30,i3)')' Unknown input: swdiag = ',swdiag
          stop
        endif

        if(szdiag.eq.0)then
          write(nout,'(a16,i2,(a))')'szdiag=',szdiag,
     &        ': Z-exchange diagrams excluded '
        elseif(szdiag.eq.1)then
          write(nout,'(a16,i2,(a))')'szdiag=',szdiag,
     &        ': Z-exchange diagrams included '
        else
          write(*,'(a30,i3)')' Unknown input: szdiag = ',szdiag
          stop
        endif

        if(swidth.eq.0)then
          write(nout,'(a16,i2,(a))')'swidth=',swidth,
     &        ': zero width'
        elseif(swidth.eq.1)then
          write(nout,'(a16,i2,(a))')'swidth=',swidth,
     &        ': complex mass scheme'
        elseif(swidth.eq.2)then
          write(nout,'(a16,i2,(a))')'swidth=',swidth,
     &        ': naive fixed width scheme'
        else
          write(*,'(a30,i3)')' Unknown input: swidth = ',swidth
          stop
        endif
      endif

c ir-treatment
          if(sirtr.eq.0)then
            write(nout,'(a16,i2,(a))')'sirtr=',sirtr,
     &          ': subtraction method used'
          elseif(sirtr.eq.1)then
            write(nout,'(a16,i2,(a))')'sirtr=',sirtr,
     &          ': phase-space slicing  used'
          else
            write(*,'(a)')' Unknown input: sirtr = ',sirtr
            stop
          endif

c weak virtual corrections
      if(sew.eq.1)then
        write(nout,'(a16,i2,(a))')'sew=',sew,
     &      ': electroweak corrections included'
      elseif(sew.ne.0)then
        write(*,'(a30,i3)')' Unknown input: sew = ',sew
        stop
      endif

      if(lnoutmc.ge.3)then
c weak virtual corrections
        if(sweak.eq.1)then
          write(nout,'(a16,i2,(a))')'sweak=',sweak,
     &        ': electroweak virtual corrections included'
        elseif(sweak.eq.2)then
          write(nout,'(a16,i2,(a))')'sweak=',sweak,
     &        ': only fermionic virtual corrections included'
        elseif(sweak.eq.3)then
          write(nout,'(a16,i2,(a))')'sweak=',sweak,
     &        ': only bosonic virtual corrections included'
        elseif(sweak.ne.0)then
          write(*,'(a30,i3)')' Unknown input: sweak = ',sweak
          stop
        endif
      endif

c incoming photon corrections
      if(spinc(0).eq.1)then
        write(nout,'(a16,i2,(a))')'spinc=',spinc(0),
     &      ': incoming photon corrections included'
      elseif(spinc(0).ne.0)then
        write(*,'(a30,i3)')' Unknown input: spinc = ',spinc(0)
        stop
      endif

c two-loop heavy Higgs corrections
      if(shh2.eq.1)then
        write(nout,'(a16,i2,(a))')'shh2=',shh2,
     &    ': 2-loop heavy Higgs corrections included'
      elseif(shh2.ne.0)then
        write(*,'(a30,i3)')' Unknown input: shh2 = ',shh2
        stop
      endif

c qcd corrections
      if(sqcd.eq.1)then
        write(nout,'(a16,i2,(a))')'sqcd=',sqcd,
     &    ': QCD corrections included'
      elseif(sqcd.ne.0)then
        write(*,'(a30,i3)')' Unknown input: sqcd = ',sqcd
        stop
      endif
c diagonal qcd corrections
      if(sqcddiag.eq.1)then
        write(nout,'(a16,i2,(a))')'sqcddiag=',sqcddiag,
     &    ': diagonal QCD corrections included'
      elseif(sqcddiag.ne.0)then
        write(*,'(a30,i3)')' Unknown input: sqcddiag = ',sqcddiag
        stop
      endif
c nondiagonal qcd corrections
      if(sqcdnondiag.eq.1)then
        write(nout,'(a16,i2,(a))')'sqcdnondiag=',sqcdnondiag,
     &    ': nondiagonal QCD corrections included'
      elseif(sqcdnondiag.ne.0)then
        write(*,'(a30,i3)')' Unknown input: sqcdnondiag = ',sqcdnondiag
        stop
      endif
c qqg qcd corrections
      if(sqcdgsplit.eq.1)then
        write(nout,'(a16,i2,(a))')'sqcdgsplit=',sqcdgsplit,
     &    ': qqg QCD corrections included'
      elseif(sqcdgsplit.ne.0)then
        write(*,'(a30,i3)')' Unknown input: sqcdgsplit = ',sqcdgsplit
        stop
      endif
c gg fusion qcd corrections
      if(sqcdggfus.eq.1)then
        write(nout,'(a16,i2,(a))')'sqcdggfus=',sqcdggfus,
     &    ': gg fusion QCD corrections included'
      elseif(sqcdggfus.ne.0)then
        write(*,'(a30,i3)')' Unknown input: sqcdggfus = ',sqcdggfus
        stop
      endif

c t-channel diagrams
      if(stcha.eq.0)then
        write(nout,'(a16,i2,(a))')'stcha=',stcha,
     &    ': t-channel diagrams not included'
      elseif(stcha.ne.1)then
        write(*,'(a30,i3)')' Unknown input: stcha = ',stcha
        stop
      endif
c s-channel diagrams
      if(sscha.eq.0)then
        write(nout,'(a16,i2,(a))')'sscha=',sscha,
     &    ': s-channel diagrams not included'
      elseif(sscha.ne.1)then
        write(*,'(a30,i3)')' Unknown input: sscha = ',sscha
        stop
      endif
c squared matrix elements
      if(sch2.eq.0)then
        write(nout,'(a16,i2,(a))')'sch2=',sch2,
     &    ': squared matrix elements not included'
      elseif(sch2.ne.1)then
        write(*,'(a30,i3)')' Unknown input: sch2 = ',sch2
        stop
      endif
c interferences
      if(schint.eq.0)then
        write(nout,'(a16,i2,(a))')'schint=',schint,
     &    ': interferences not included'
      elseif(schint.ne.1)then
        write(*,'(a30,i3)')' Unknown input: schint = ',schint
        stop
      endif

c bottom quark contributions in initial state
      if(sbini.eq.1)then
        write(nout,'(a16,i2,(a))')'sbini=',sbini,
     &    ': initial bottoms included in LO'
      elseif(sbini.ne.0)then
        write(*,'(a30,i3)')' Unknown input: sbini = ',sbini
        stop
      endif
c bottom quark contributions in final state
      if(sbfin.eq.1)then
        write(nout,'(a16,i2,(a))')'sbfin=',sbfin,
     &    ': final bottoms included in LO'
      elseif(sbfin.ne.0)then
        write(*,'(a30,i3)')' Unknown input: sbfin = ',sbfin
        stop
      endif

      if(lnoutmc.ge.3)then
c end-point part of subtraction terms
        if(ssoft.eq.1.)then
          write(nout,'(a16,i2,(a))')'ssoft=',ssoft,
     &        ': end-point part of subtraction functions of STD'
        elseif(ssoft.ne.0)then
          write(*,'(a30,i3)')' Unknown input: ssoft = ',ssoft
          stop
        endif

c light fermion masses neglected
        if(slightfermions.eq.1)then
          write(nout,'(a16,i2,(a))')' slightfermions=',slightfermions,
     &        ': small fermion masses neglected (el,mu,uq,dq,cq,sq)'
        elseif(slightfermions.eq.2)then
          write(nout,'(a16,i2,(a))')' slightfermions=',slightfermions,
     &        ': small fermion masses neglected'//
     &        ' (el,mu,ta,uq,dq,cq,sq,bq)'
        elseif(slightfermions.ne.0)then
          write(*,'(a30,i3)')' Unknown input: slightfermions = ',
     &        slightfermions
          stop
        endif

c real part of bremsstrahlung corrections
        if(sreal.eq.1)then
          write(*,'(a)')' sreal = 1 not implemented'
          stop
          write(nout,'(a16,i2,(a))')'sreal=',sreal,
     &        ': imaginary part of virtual corrections neglected'
        elseif(sreal.ne.0)then
          write(*,'(a30,i3)')' Unknown input: sreal = ',sreal
          stop
        endif
      endif

c qed factorization
      if(sfactqed.eq.1)then
        write(nout,'("       sfactqed=",i2,(a))'),sfactqed,
     &      ': MSbar factorization for QED'
      else if(sfactqed.eq.2)then
        write(nout,'("       sfactqed=",i2,(a))'),sfactqed,
     &      ': DIS factorization for QED  '
      elseif(sfactqed.ne.0)then
        write(*,'(a30,i3)')' Unknown input: sfactqed = ',sfactqed
        stop
      endif
c qcd factorization
      if(sfactqcd.eq.1)then
        write(nout,'("       sfactqcd=",i2,(a))'),sfactqcd,
     &      ': MSbar factorization for QCD'
      else if(sfactqcd.eq.2)then
        write(nout,'("       sfactqcd=",i2,(a))'),sfactqcd,
     &      ': DIS factorization for QCD  '
      elseif(sfactqcd.ne.0)then
        write(*,'(a30,i3)')' Unknown input: sfactqcd = ',sfactqcd
        stop
      endif
      if(sfactqcd.eq.0.and.sfactqcd.ne.0.or.
     &    sfactqcd.ne.0.and.sfactqcd.eq.0) then
        write(*,'(a30,i3)')' Unknown input: sfactqed = ',sfactqed
        write(*,'(a30,i3)')' Unknown input: sfactqcd = ',sfactqcd
        stop
      endif

      if(lnoutmc.ge.3)then
c frame for momentum generation
        if(lcmsgen)then
          write(nout,'("        lcmsgen=",l2,(a))'),lcmsgen,
     &        ': momenta generated in CMS'
        else
          write(nout,'("        lcmsgen=",l2,(a))'),lcmsgen,
     &        ': momenta generated in LS'
        endif
      endif

      if(shtr.eq.0)then
        write(nout,'("           shtr=",i2,(a))'),shtr,
     &      ': on-shell Higgs boson'
      elseif(shtr.eq.1)then
        write(nout,'("           shtr=",i2,(a))'),shtr,
     &      ': off-shell Higgs boson'
      elseif(shtr.eq.2)then
        write(nout,'("           shtr=",i2,(a))'),shtr,
     &      ': Higgs decaying to two singlets'
      else
        write(*,'(a30,i3)')' Unknown input: shtr = ',shtr
        stop
      endif

      if(shtr.gt.0)then
        if(sonsproj.eq.0)then
          write(nout,'("       sonsproj=",i2,(a))'),sonsproj,
     &        ': no on-shell projection'
        elseif(sonsproj.eq.1)then
          write(nout,'("       sonsproj=",i2,(a))'),sonsproj,
     &        ': on-shell projection via rescaling '//
     &        'of final-state momenta'
        elseif(sonsproj.eq.2)then
          write(nout,'("       sonsproj=",i2,(a))'),sonsproj,
     &        ': on-shell projection via rescaling '//
     &        'of all momenta'
        else
          write(*,'(a30,i3)')' Unknown input: sonsproj = ',sonsproj
          stop
        endif

        if(sborn(0).gt.0.and.lonsborn)then
          write(nout,'("       lonsborn=",l2,(a))'),lonsborn,
     &        ': LO ME projected on-shell'
        endif

        if(sqcd.gt.0.and.lonsqcd)then
          write(nout,'("        lonsqcd=",l2,(a))'),lonsqcd,
     &        ': NLO QCD ME projected on-shell'
        endif

        if(sew.gt.0.and.lonsew)then
          write(nout,'("         lonsew=",l2,(a))'),lonsew,
     &        ': NLO EW ME projected on-shell'
        endif
      endif

c subcontributions
      do gen=0,ngenerator
      if(sgen(gen).ne.0)then
        write(nout,*)' '
c subcontributions
        if(gen.eq.0)then
          write(nout,'("        sgen(",i1,")=",i2,(a))')gen,sgen(gen),
     &      ': branch for Born or virtual corrections'
        elseif(gen.eq.1)then
          write(nout,'("        sgen(",i1,")=",i2,(a))')gen,sgen(gen),
     &      ': branch for real photonic corrections'
        elseif(gen.eq.2)then
          write(nout,'("        sgen(",i1,")=",i2,(a))')gen,sgen(gen),
     &      ': branch for real gluonic corrections'
        elseif(gen.eq.3)then
          write(nout,'("        sgen(",i1,")=",i2,(a))')gen,sgen(gen),
     &      ': branch for initial photon contributions 1'
        elseif(gen.eq.4)then
          write(nout,'("        sgen(",i1,")=",i2,(a))')gen,sgen(gen),
     &      ': branch for initial photon contributions 2'
        elseif(gen.eq.5)then
          write(nout,'("        sgen(",i1,")=",i2,(a))')gen,sgen(gen),
     &      ': branch for initial gluon contributions 1'
        elseif(gen.eq.6)then
          write(nout,'("        sgen(",i1,")=",i2,(a))')gen,sgen(gen),
     &      ': branch for initial gluon contributions 2'
        else
          write(*,'(a)')' Unknown input: sgen = ',sgen(gen)
          stop
        endif
c event numbers
        if(freqgencon(0,6).ne.freqgencon(0,7))then
          write(*,'(a30,2i5)')' Unknown input: freqgencon 6=/=7 ',
     &        freqgencon(0,6),freqgencon(0,7)
          stop
        endif
        if(freqgencon(0,8).ne.freqgencon(0,9))then
          write(*,'(a30,2i5)')' Unknown input: freqgencon 8=/=9 ',
     &        freqgencon(0,8),freqgencon(0,9)
          stop
        endif
        if(freqgencon(0,2).lt.freqgencon(0,5))then
          write(*,*)' ew frequency freqgencon(0,2) > ',
     &        ' strong frequency freqgencon(0,5)',
     &        ' strong frequency reset to electroweak frequency',
     &        freqgencon(0,2),freqgencon(0,5)
          freqgencon(0,5)=freqgencon(0,2)
        endif

        ncontr(0)=9
        if(gen.eq.0)then
          do icon=1,ncontr(gen)
            if(sgencon(gen,icon).eq.1)then
              write(nout,'("   nevents(",i1,",",i1,")=",i10)')
     &            gen,icon,nevents/freqgencon(gen,icon)
            endif
          enddo
        else
          if(sgen(gen).eq.1)then
            write(nout,'("     nevents(",i1,")=",i10)')gen,
     &          nevents/freqgencon(gen,1)
          endif
        endif
c born amplitude
        if(sborn(gen).eq.1)then
          write(nout,'("       sborn(",i1,")=",i2,(a))')gen,sborn(gen),
     &      ': tree-level amplitude included'
        elseif(sborn(gen).ne.0)then
          write(*,'(a25,i3)')' Unknown input: sborn = ',sborn(gen)
          stop
        endif
c additional information for tree-level cross section
        if(swborn(gen).eq.1)then
          write(nout,'("      swborn(",i1,")=",i2,(a))')gen,swborn(gen),
     &      ': additional result for tree-level cross section'
        elseif(swborn(gen).ne.0)then
          write(*,'(a30,i3)')' Unknown input: swborn = ',swborn
          stop
        endif

        if(lnoutmc.ge.3)then
c convolution
          if(sconv(gen).eq.1)then
            write(nout,'("       sconv(",i1,")=",i2,(a))')
     &          gen,sconv(gen),
     &          ': convolution over parton distributions included'
            if(spdf.eq.1)then
              write(nout,'("           spdf=",i2,(a))') spdf,
     &            ': MRST MRST2004QED [with O(alpha) effects]'
            elseif(spdf.eq.2)then
              write(nout,'("           spdf=",i2,(a))') spdf,
     &            ': MRST MRST2004QED [without O(alpha) effects]'
            elseif(spdf.eq.11)then
              write(nout,'("           spdf=",i2,(a))') spdf,
     &            ': CTEQ6M [Standard MSbar scheme (alphas(mz)=0.118)]'
            elseif(spdf.eq.14)then
              write(nout,'("           spdf=",i2,(a))') spdf,
     &            ': CTEQ6L1 [Leading Order (alphas(mz)=0.130)]'
#ifdef includeLHAPDF
            elseif(spdf.eq.99)then
              if(pdfpath.ne.'')then
                write(nout,'("           spdf=",i2,(a),
     &              /20x,"pdfset ",i2)')
     &              spdf,': LHApdf: '//pdfpath,pdfmember
              else
                write(nout,'("           spdf=",i2,(a),
     &              /20x,"pdfset ",i2)')
     &              spdf,': LHApdf: '//pdfname,pdfmember
              endif
#endif
            elseif(sconv(gen).ne.0)then
              write(*,'(a)')' Unknown input: spdf = ',spdf
              stop
            endif
          elseif(sconv(gen).ne.0)then
            write(*,'(a30,i3)')' Unknown input: sconv = ',sconv(gen)
            stop
          endif

c virtual corrections
          if(svirt(gen).eq.1)then
            write(nout,'("       svirt(",i1,")=",i2,(a))')
     &          gen,svirt(gen),
     &          ': virtual corrections included'
          elseif(svirt(gen).ne.0)then
            write(*,'(a30,i3)')' Unknown input: svirt = ',svirt(gen)
            stop
          endif
c photonic bremsstrahlung corrections
          if(spbrems(gen).eq.1)then
            write(nout,'("     spbrems(",i1,")=",i2,(a))')
     &          gen,spbrems(gen),
     &          ': real photonic corrections included'
          elseif(spbrems(gen).ne.0)then
            write(*,'(a30,i3)')' Unknown input: spbrems = ',spbrems(gen)
            stop
          endif
c gluonic bremsstrahlung corrections
          if(sgbrems(gen).eq.1)then
            write(nout,'("     sgbrems(",i1,")=",i2,(a))')
     &          gen,sgbrems(gen),
     &          ': real gluonic corrections included'
          elseif(sgbrems(gen).ne.0)then
            write(*,'(a30,i3)')' Unknown input: sgbrems = ',sgbrems(gen)
            stop
          endif
c incoming photon corrections
          if(spinc(gen).eq.1)then
            write(nout,'("       spinc(",i1,")=",i2,(a))')
     &          gen,spinc(gen),
     &          ': incoming photon corrections included'
          elseif(spinc(gen).ne.0)then
            write(*,'(a30,i3)')' Unknown input: spinc = ',spinc(gen)
            stop
          endif
c incoming gluon  corrections
          if(sginc(gen).eq.1)then
            write(nout,'("       sginc(",i1,")=",i2,(a))')
     &          gen,sginc(gen),
     &          ': incoming gluon corrections included'
          elseif(sginc(gen).ne.0)then
            write(*,'(a30,i3)')' Unknown input: sginc = ',sginc(gen)
            stop
          endif
c slicing terms
          if(ssli(gen).eq.1)then
            write(nout,'("        ssli(",i1,")=",i2,(a))')gen,ssli(gen),
     &          ': slicing terms included'
          elseif(ssli(gen).ne.0)then
            write(*,'(a)')' Unknown input: ssli = ',ssli(gen)
            stop
          endif
c subtraction terms
          if(ssub(gen).eq.1.or.ssub(gen).eq.2)then
            write(nout,'("        ssub(",i1,")=",i2,(a))')gen,ssub(gen),
     &          ': subtraction terms included'
          elseif(ssub(gen).ne.0)then
            write(*,'(a30,i3)')' Unknown input: ssub = ',ssub(gen)
            stop
          endif
c collinear effective factors
          if(seff(gen).eq.1)then
            write(nout,'("        seff(",i1,")=",i2,(a))')gen,seff(gen),
     &          ': collinear effective factors included'
          elseif(seff(gen).ne.0)then
            write(*,'(a30,i3)')' Unknown input: sseff = ',seff(gen)
            stop
          endif
c end-point contributions
          if(sendp(gen).eq.1)then
            write(nout,'("       sendp(",i1,")=",i2,(a))')
     &          gen,sendp(gen),
     &          ': end-point contributions included in Monte Carlo'
          elseif(sendp(gen).eq.-1)then
            write(nout,'("       sendp(",i1,")=",i2,(a))')
     &          gen,sendp(gen),
     &          ': end-point contributions included in |M|^2 '
          elseif(sendp(gen).ne.0)then
            write(*,'(a30,i3)')' Unknown input: sendp = ',sendp(gen)
            stop
          endif
c include cuts in phase-space generator
          if(sincludecuts(gen).eq.1)then
            write(nout,'("sincludecuts(",i1,")=",i2,(a))')gen,
     &          sincludecuts(gen),
     &          ': threshold cuts taken into account'//
     &          ' in phase-space generation'
          elseif(sincludecuts(gen).eq.2)then
            write(nout,'("sincludecuts(",i1,")=",i2,(a))')gen,
     &          sincludecuts(gen),
     &          ': threshold cuts taken into account'//
     &          ' in phase-space and x generation'
          elseif(sincludecuts(gen).eq.3)then
            write(nout,'("sincludecuts(",i1,")=",i2,(a))')gen,
     &          sincludecuts(gen),
     &          ': cuts taken into account in phase-space generation'
          elseif(sincludecuts(gen).eq.0)then
            write(nout,'("sincludecuts(",i1,")=",i2,(a))')gen,
     &          sincludecuts(gen),
     &          ': cuts NOT taken into account in phase-space'//
     &          ' generation'
          else
            write(*,'(a30,i3)')' Unknown input: sincludecuts = ',
     &          sincludecuts(gen)
            stop
          endif
        endif

c recombination cuts
        if(srecomb(gen).eq.1.or.srecomb(gen).eq.2)then
          write(nout,'("     srecomb(",i1,")=",i2,(a))')gen,srecomb(0),
     &      ': with kT jet clustering:'
          write(nout,'(" resolution parameter D  = ",f11.5)')
     &      ktd
          write(nout,'(" partonic cut |y_parton| < ",f11.5)')
     &      partetacut
        else if(srecomb(gen).eq.3.or.srecomb(gen).eq.4)then
          write(nout,'("     srecomb(",i1,")=",i2,(a))')gen,srecomb(0),
     &      ': with photon recombination:'
          if(srecomb(gen).eq.4) then
            write(nout,'(" theta_p    = ",f11.5," deg")')
     &          dacos(recparam(1,gen))*180d0/pi
            write(nout,'(" E_p        = ",f11.5," GeV")')
     &          recparam(2,gen)
            write(nout,'(" theta_pf   = ",f11.5," deg")')
     &          dacos(recparam(3,gen))*180d0/pi
            write(nout,'(" M_pf       = ",f11.5," GeV")')
     &          sqrt(recparam(4,gen))
          elseif (srecomb(gen).eq.3)then
            write(nout,'(" y_p        = ",f11.5)') recparam(1,gen)
            write(nout,'(" E_p        = ",f11.5," GeV")')
     &          recparam(2,gen)
            write(nout,'(" dR_pj      = ",f11.5)')
     &          sqrt(recparam(3,gen))
            write(nout,'(" M_pj       = ",f11.5," GeV")')
     &          sqrt(recparam(4,gen))
          endif
        elseif(srecomb(gen).ne.0)then
          write(*,*)' Unknown input: srecomb= ',srecomb(gen)
          stop
        endif

c separation cuts
        if(scuts(gen).ge.1.and.scuts(gen).le.10)then
          write(nout,'("       scuts(",i1,")=",i2,(a))')gen,scuts(gen),
     &      ': with separation cuts:'
          if(scuts(gen).eq.1)then
            write(nout,'((a))')
     &      ' tagging jets defined via pT ordering'
          elseif(scuts(gen).eq.2)then
            write(nout,'((a))')
     &      ' tagging jets defined via E ordering'
          endif
          do ext=1,nextgen(gen)
            if(ptcut(ext,gen).gt.0d0)then
              write(nout,'(" pT(",i1,")      > ",f11.5," GeV")')
     &          ext,ptcut(ext,gen)
            endif
          enddo
          do ext=1,nextgen(gen)
            if(ecut(ext,gen).gt.0d0)then
              write(nout,'(" energy(",i1,")  > ",f11.5," GeV")')
     &          ext,ecut(ext,gen)
            endif
          enddo
          do ext=1,nextgen(gen)
            if(ycut(ext,gen).lt.1d10)then
              write(nout,'(" y(",i1,")       < ",f11.5)')
     &          ext,ycut(ext,gen)
            endif
          enddo
          do ext=1,nextgen(gen)
          do ext2=ext+1,nextgen(gen)
            if(ccut(ext,ext2,gen).lt.1d0)then
              write(nout,'(" angle(",i1,",",i1,") > ",f11.5," deg")')
     &          ext,ext2,dacos(ccut(ext,ext2,gen))/pi*180d0
            endif
          enddo
          enddo
          do ext=1,nextgen(gen)
          do ext2=ext+1,nextgen(gen)
            if(scut(ext,ext2,gen).gt.0d0)then
              write(nout,'(" mass(",i1,",",i1,")  > ",f11.5," GeV")')
     &          ext,ext2,sqrt(scut(ext,ext2,gen))
            endif
          enddo
          enddo
          do ext=3,nextgen(gen)
          do ext2=ext+1,nextgen(gen)
            if(dycut(ext,ext2,gen).gt.0d0)then
              write(nout,'(" dy(",i1,",",i1,")    > ",f11.5)')
     &          ext,ext2,dycut(ext,ext2,gen)
            endif
          enddo
          enddo
          do ext=3,nextgen(gen)
          do ext2=ext+1,nextgen(gen)
            if(dRcut(ext,ext2,gen).gt.0d0)then
              write(nout,'(" dR(",i1,",",i1,")    > ",f11.5)')
     &          ext,ext2,dRcut(ext,ext2,gen)
            endif
          enddo
          enddo
          if((shtr.ge.1))then
            write(nout,'(" sqrt(s_h)  > ",f11.5, " GeV")') mlcuth
            if(mucuth.lt.1e29)
     &          write(nout,'(" sqrt(s_h)  < ",f11.5, " GeV")') mucuth
          endif
          if((scuts(gen).eq.1.or.scuts(gen).eq.2).and.lhemicut)then
            write(nout,'(" hemisphere cut:  y(jet1)*y(jet2) < 0")')
          endif
          if(shtr.eq.2.and.(scuts(gen).eq.1.or.scuts(gen).eq.2)
     &        .and.lhiggsbjets)then
            write(nout,
     &        '(" rapidity cut:    y(jet_min) < y(decp) < y(jetmax)")')
          endif
        elseif(scuts(gen).eq.0)then
          if((shtr.ge.1))then
            write(nout,'(" sqrt(s_h)  > ",f11.5, " GeV")') mlcuth
            if(mucuth.lt.1e29)
     &          write(nout,'(" sqrt(s_h)  < ",f11.5, " GeV")') mucuth
          endif
        else
          write(*,'(a30,i3)')' Unknown input: scuts = ',scuts(gen)
          stop
        endif
      endif
      enddo
      write(nout,*)

      end

***********************************************************************
      subroutine writeresult
***********************************************************************
*     output of result                                                *
*     adapted from COFFERGG by Markus Roth                            *
*                                                                     *
*---------------------------------------------------------------------*
*     21.02.06  Ansgar Denner     last changed  18.06.07              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

#ifdef multpdf
#include "multpdf.h"
      integer nset
#endif
#ifdef mpiuse
      include "mpif.h"
      integer status(MPI_STATUS_SIZE)
      real*8 keep_sigmaa
      real*8 keep_avborn
      real*8 keep_siborn
#ifdef multpdf
      real*8 keep_mavborn(maxpdf)
      real*8 keep_msiborn(maxpdf)
#endif
      real*8 keep_avbornii(-4:4,-4:4)
      real*8 keep_sibornii(-4:4,-4:4)
      real*8 keep_averageii(-4:4,-4:4)
      real*8 keep_sigmaii(-4:4,-4:4)
      real*8 keep_avgenii(0:maxg,maxcon,-4:4,-4:4)
      real*8 keep_sigenii(0:maxg,maxcon,-4:4,-4:4)
      real*8 keep_avgencha(0:maxg,maxcon,maxch)
      real*8 keep_sigencha(0:maxg,maxcon,maxch)
      real*8 keep_avgen(0:maxg,maxcon)
      real*8 keep_sigen(0:maxg,maxcon)
#ifdef multpdf
      real*8 keep_mavgen(maxpdf,0:maxg,maxcon)
      real*8 keep_msigen(maxpdf,0:maxg,maxcon)
#endif
      real*8 keep_weighttotmax
      real*8 keep_weightmaxcha(0:maxg,maxcon,maxch)
      real*8 keep_weightmax(0:maxg,maxcon)
      integer keep_nmax(0:maxg,maxcon)
      integer keep_channelmax(0:maxg,maxcon)
      integer keep_rejcha(0:maxg,maxcon,maxch)
      integer keep_rejgen(0:maxg,maxcon)
#endif

      integer iNumProcs, iErr, iMyRank,ii
      common/mpistuff/iNumProcs,iMyRank

c local variables
       character*5 part(-4:4)
       integer i1,i2
      real*8 avgenchan(0:maxg,maxcon,maxch)
      real*8 sigenchan(0:maxg,maxcon,maxch)

c date and time
      real*8     rcpu
c     integer*4  int,itime,iminit,ihour,icpu,icpuh,icpum,icpus
c     integer*4  int,itime,icpu,icpuh,icpum,icpus
      integer*4  int,icpu,icpuh,icpum,icpus
c     integer    idate
#ifdef PGF77
      integer    imonth,iyear,iday
      real tarray(2),etime
#endif
c      integer*4  no,lpar,il,iu,id,count
      integer*4  count,nout2
c      character  parstr*6,prostr*22
      character  datum*10,da1*2,da2*2,da3*4
      character  chtime*8
      integer    iarray(3)

#ifdef mpiuse
      if(iMyRank.EQ.0.AND.iNumProcs.NE.1) Then
	do ii=1,iNumProcs-1
	  call MPI_Recv(keep_sigmaa, 1,
     &    MPI_DOUBLE_PRECISION, ii, 10, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_avborn, 1,
     &    MPI_DOUBLE_PRECISION, ii, 11, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_siborn, 1,
     &    MPI_DOUBLE_PRECISION, ii, 12, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_avbornii, 81,
     &    MPI_DOUBLE_PRECISION, ii, 13, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_sibornii, 81,
     &    MPI_DOUBLE_PRECISION, ii, 14, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_averageii, 81,
     &    MPI_DOUBLE_PRECISION, ii, 15, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_sigmaii, 81,
     &    MPI_DOUBLE_PRECISION, ii, 16, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_avgenii, 81*(maxg+1)*(maxcon),
     &    MPI_DOUBLE_PRECISION, ii, 17, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_sigenii, 81*(maxg+1)*(maxcon),
     &    MPI_DOUBLE_PRECISION, ii, 18, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_avgencha, (maxg+1)*(maxcon)*(maxch),
     &    MPI_DOUBLE_PRECISION, ii, 19, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_sigencha, (maxg+1)*(maxcon)*(maxch),
     &    MPI_DOUBLE_PRECISION, ii, 20, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_avgen, (maxg+1)*(maxcon),
     &    MPI_DOUBLE_PRECISION, ii, 21, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_sigen, (maxg+1)*(maxcon),
     &    MPI_DOUBLE_PRECISION, ii, 22, MPI_COMM_WORLD, status, iErr)
#ifdef multpdf
	  call MPI_Recv(keep_mavborn, maxpdf,
     &    MPI_DOUBLE_PRECISION, ii, 11, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_msiborn, maxpdf,
     &    MPI_DOUBLE_PRECISION, ii, 12, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_mavgen, maxpdf*(maxg+1)*(maxcon),
     &    MPI_DOUBLE_PRECISION, ii, 30, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_msigen, maxpdf*(maxg+1)*(maxcon),
     &    MPI_DOUBLE_PRECISION, ii, 31, MPI_COMM_WORLD, status, iErr)
#endif
	  call MPI_Recv(keep_weighttotmax, 1,
     &    MPI_DOUBLE_PRECISION, ii, 23, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_weightmaxcha, (maxg+1)*(maxcon)*(maxch),
     &    MPI_DOUBLE_PRECISION, ii, 24, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_weightmax, (maxg+1)*(maxcon),
     &    MPI_DOUBLE_PRECISION, ii, 25, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_nmax, (maxg+1)*(maxcon),
     &    MPI_INTEGER, ii, 26, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_channelmax, (maxg+1)*(maxcon),
     &    MPI_INTEGER, ii, 27, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_rejcha, (maxg+1)*(maxcon)*(maxch),
     &    MPI_INTEGER, ii, 28, MPI_COMM_WORLD, status, iErr)
	  call MPI_Recv(keep_rejgen, (maxg+1)*(maxcon),
     &    MPI_INTEGER, ii, 29, MPI_COMM_WORLD, status, iErr)
	  sigmaa=sigmaa+keep_sigmaa
	  avborn=avborn+keep_avborn
	  siborn=siborn+keep_siborn
#ifdef multpdf
	  do nset=1,pdfmemberto-pdfmemberfrom+1
	    mavborn(nset)=mavborn(nset)+keep_mavborn(nset)
	    msiborn(nset)=msiborn(nset)+keep_msiborn(nset)
	  enddo
#endif
	  do i1=-4,4
	   do i2=-4,4
	    avbornii(i1,i2)=avbornii(i1,i2)+keep_avbornii(i1,i2)
	    sibornii(i1,i2)=sibornii(i1,i2)+keep_sibornii(i1,i2)
	    averageii(i1,i2)=averageii(i1,i2)+keep_averageii(i1,i2)
	    sigmaii(i1,i2)  =sigmaii(i1,i2)  +keep_sigmaii(i1,i2)
            do gen=0,ngenerator
             do icon=1,ncontr(gen)
	      avgenii(gen,icon,i1,i2)=avgenii(gen,icon,i1,i2)+
     &                keep_avgenii(gen,icon,i1,i2)
	      sigenii(gen,icon,i1,i2)=sigenii(gen,icon,i1,i2)+
     &                keep_sigenii(gen,icon,i1,i2)
	     enddo
	    enddo
	   enddo
	  enddo	  	  	
          do gen=0,ngenerator
           do icon=1,ncontr(gen)
            do cha=1,nchannel(gen)
	     avgencha(gen,icon,cha)=avgencha(gen,icon,cha)+
     &                              keep_avgencha(gen,icon,cha)
	     sigencha(gen,icon,cha)=sigencha(gen,icon,cha)+
     &                              keep_sigencha(gen,icon,cha)
	    enddo
	    avgen(gen,icon)=avgen(gen,icon)+keep_avgen(gen,icon)
	    sigen(gen,icon)=sigen(gen,icon)+keep_sigen(gen,icon)
#ifdef multpdf
	    do nset=1,pdfmemberto-pdfmemberfrom+1
	      mavgen(nset,gen,icon)=mavgen(nset,gen,icon)+
     &                              keep_mavgen(nset,gen,icon)
	      msigen(nset,gen,icon)=msigen(nset,gen,icon)+
     &                              keep_msigen(nset,gen,icon)
            enddo
#endif
           enddo
	  enddo
	  If(keep_weighttotmax.GT.weighttotmax)
     &                           weighttotmax=keep_weighttotmax
          do gen=0,ngenerator
           do icon=1,ncontr(gen)
            do cha=1,nchannel(gen)
	      If(keep_weightmaxcha(gen,icon,cha).GT.
     &            weightmaxcha(gen,icon,cha)) Then
	        weightmaxcha(gen,icon,cha)=
     &                           keep_weightmaxcha(gen,icon,cha)
	      Endif	
	      rejcha(gen,icon,cha)=rejcha(gen,icon,cha)+
     &                             keep_rejcha(gen,icon,cha)	
	    enddo
	     If(keep_weightmax(gen,icon).GT.weightmax(gen,icon)) Then
	       weightmax(gen,icon)=keep_weightmax(gen,icon)
	       nmax(gen,icon)=keep_nmax(gen,icon)	
	       channelmax(gen,icon)=keep_channelmax(gen,icon)	
	     Endif	
	     rejgen(gen,icon)=rejgen(gen,icon)+keep_rejgen(gen,icon)
	   enddo
	  enddo
	enddo
      else if(iMyRank.NE.0.AND.iNumProcs.NE.1) Then
	call MPI_Send(sigmaa, 1,
     &  MPI_DOUBLE_PRECISION, 0, 10, MPI_COMM_WORLD, iErr)
	call MPI_Send(avborn, 1,
     &  MPI_DOUBLE_PRECISION, 0, 11, MPI_COMM_WORLD, iErr)
	call MPI_Send(siborn, 1,
     &  MPI_DOUBLE_PRECISION, 0, 12, MPI_COMM_WORLD, iErr)
	call MPI_Send(avbornii, 81,
     &  MPI_DOUBLE_PRECISION, 0, 13, MPI_COMM_WORLD, iErr)
	call MPI_Send(sibornii, 81,
     &  MPI_DOUBLE_PRECISION, 0, 14, MPI_COMM_WORLD, iErr)
	call MPI_Send(averageii, 81,
     &  MPI_DOUBLE_PRECISION, 0, 15, MPI_COMM_WORLD, iErr)
	call MPI_Send(sigmaii, 81,
     &  MPI_DOUBLE_PRECISION, 0, 16, MPI_COMM_WORLD, iErr)
	call MPI_Send(avgenii, 81*(maxg+1)*(maxcon),
     &  MPI_DOUBLE_PRECISION, 0, 17, MPI_COMM_WORLD, iErr)
	call MPI_Send(sigenii, 81*(maxg+1)*(maxcon),
     &  MPI_DOUBLE_PRECISION, 0, 18, MPI_COMM_WORLD, iErr)
	call MPI_Send(avgencha, (maxg+1)*(maxcon)*(maxch),
     &  MPI_DOUBLE_PRECISION, 0, 19, MPI_COMM_WORLD, iErr)
	call MPI_Send(sigencha, (maxg+1)*(maxcon)*(maxch),
     &  MPI_DOUBLE_PRECISION, 0, 20, MPI_COMM_WORLD, iErr)
	call MPI_Send(avgen, (maxg+1)*(maxcon),
     &  MPI_DOUBLE_PRECISION, 0, 21, MPI_COMM_WORLD, iErr)
	call MPI_Send(sigen, (maxg+1)*(maxcon),
     &  MPI_DOUBLE_PRECISION, 0, 22, MPI_COMM_WORLD, iErr)
#ifdef multpdf
	call MPI_Send(mavborn, maxpdf,
     &  MPI_DOUBLE_PRECISION, 0, 11, MPI_COMM_WORLD, iErr)
	call MPI_Send(msiborn, maxpdf,
     &  MPI_DOUBLE_PRECISION, 0, 12, MPI_COMM_WORLD, iErr)
	call MPI_Send(mavgen, maxpdf*(maxg+1)*(maxcon),
     &  MPI_DOUBLE_PRECISION, 0, 30, MPI_COMM_WORLD, iErr)
	call MPI_Send(msigen, maxpdf*(maxg+1)*(maxcon),
     &  MPI_DOUBLE_PRECISION, 0, 31, MPI_COMM_WORLD, iErr)
#endif
	call MPI_Send(weighttotmax, 1,
     &  MPI_DOUBLE_PRECISION, 0, 23, MPI_COMM_WORLD, iErr)
	call MPI_Send(weightmaxcha, (maxg+1)*(maxcon)*(maxch),
     &  MPI_DOUBLE_PRECISION, 0, 24, MPI_COMM_WORLD, iErr)
	call MPI_Send(weightmax, (maxg+1)*(maxcon),
     &  MPI_DOUBLE_PRECISION, 0, 25, MPI_COMM_WORLD, iErr)
	call MPI_Send(nmax, (maxg+1)*(maxcon),
     &  MPI_INTEGER, 0, 26, MPI_COMM_WORLD, iErr)
	call MPI_Send(channelmax, (maxg+1)*(maxcon),
     &  MPI_INTEGER, 0, 27, MPI_COMM_WORLD, iErr)
	call MPI_Send(rejcha, (maxg+1)*(maxcon)*(maxch),
     &  MPI_INTEGER, 0, 28, MPI_COMM_WORLD, iErr)
	call MPI_Send(rejgen, (maxg+1)*(maxcon),
     &  MPI_INTEGER, 0, 29, MPI_COMM_WORLD, iErr)
      endif
      if(iMyRank.EQ.0) Then
#endif

      part(-4) = 'cbar'
      part(-3) = 'sbar'
      part(-2) = 'ubar'
      part(-1) = 'dbar'
      part(1) = 'd'
      part(2) = 'u'
      part(3) = 's'
      part(4) = 'c'

 1052 format(" Cross section          = ",1g24.14)
 1053 format(" Standard deviation     = ",1g24.14)
      if(nunw.ne.0)then
        write(nout,'(a)')' '
        write(nout,'(a)')' '
        write(nout,'(a)')' Unweighted events'
        write(nout,1008)nunw
        write(nout,1009)nneg
      endif

      average=average/dble(nevt)
      sigma=sigma/dble(nevt)**2
      sigmaa=sigmaa/dble(nevt)**2
      sigma=sqrt(dabs(sigma-average**2/dble(nevt)))
      sigmaa=sqrt(dabs(sigmaa-average**2/dble(nevt)))
      avborn=avborn/dble(nevt)
      siborn=siborn/dble(nevt)**2
      siborn=sqrt(dabs(siborn-avborn**2/dble(nevt)))
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
	maverage(nset)=maverage(nset)/dble(nevt)
	msigma(nset)=msigma(nset)/dble(nevt)**2
	msigmaa(nset)=msigmaa(nset)/dble(nevt)**2
	msigma(nset)=sqrt(dabs(msigma(nset)-maverage(nset)**2/
     &                     dble(nevt)))
	msigmaa(nset)=sqrt(dabs(msigmaa(nset)-maverage(nset)**2/
     &                     dble(nevt)))
	mavborn(nset)=mavborn(nset)/dble(nevt)
	msiborn(nset)=msiborn(nset)/dble(nevt)**2
	msiborn(nset)=sqrt(dabs(msiborn(nset)-
     &                     mavborn(nset)**2/dble(nevt)))
      enddo
#endif
#ifdef SUBPROCESSES
      do i1=-4,4
      do i2=-4,4
        avbornii(i1,i2)=avbornii(i1,i2)/dble(nevt)
        sibornii(i1,i2)=sibornii(i1,i2)/dble(nevt)**2
        sibornii(i1,i2)=sqrt(dabs(sibornii(i1,i2)
     &      -avbornii(i1,i2)**2/dble(nevt)))
        averageii(i1,i2)=averageii(i1,i2)/dble(nevt)
        sigmaii(i1,i2)=sigmaii(i1,i2)/dble(nevt)**2
        sigmaii(i1,i2)=sqrt(dabs(sigmaii(i1,i2)
     &      -averageii(i1,i2)**2/dble(nevt)))
        do gen=0,ngenerator
          do icon=1,ncontr(gen)
            avgenii(gen,icon,i1,i2)=avgenii(gen,icon,i1,i2)/dble(nevt)
            sigenii(gen,icon,i1,i2)=
     &          sigenii(gen,icon,i1,i2)/dble(nevt)**2
            sigenii(gen,icon,i1,i2)=sqrt(dabs(sigenii(gen,icon,i1,i2)
     &          -avgenii(gen,icon,i1,i2)**2/dble(nevt)))
          enddo
        enddo
      enddo
      enddo
#endif
      do gen=0,ngenerator
        do icon=1,ncontr(gen)
          do cha=1,nchannel(gen)
            if (nevtgencha(gen,icon,cha).gt.0) then
              avgencha(gen,icon,cha)=
     &            avgencha(gen,icon,cha)/nevtgencha(gen,icon,cha)
              sigencha(gen,icon,cha)=sigencha(gen,icon,cha)/
     &            dble(nevtgencha(gen,icon,cha))**2
              sigencha(gen,icon,cha)=sqrt(dabs(sigencha(gen,icon,cha)
     &            -avgencha(gen,icon,cha)**2/nevtgencha(gen,icon,cha)))
c     normalize to all events
              avgenchan(gen,icon,cha)=
     &          avgencha(gen,icon,cha)*nevtgencha(gen,icon,cha)/nevents
              sigenchan(gen,icon,cha)=sigencha(gen,icon,cha)*
     &            dble(nevtgencha(gen,icon,cha))/nevents
              weightmaxcha(gen,icon,cha)=weightmaxcha(gen,icon,cha)
            endif
          enddo
          avgen(gen,icon)=avgen(gen,icon)/dble(nevt)
          sigen(gen,icon)=sigen(gen,icon)/dble(nevt)**2
          sigen(gen,icon)=sqrt(dabs(sigen(gen,icon)
     &        -avgen(gen,icon)**2/dble(nevt)))
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
          mavgen(nset,gen,icon)=mavgen(nset,gen,icon)/dble(nevt)
          msigen(nset,gen,icon)=msigen(nset,gen,icon)/dble(nevt)**2
          msigen(nset,gen,icon)=sqrt(dabs(msigen(nset,gen,icon)
     &        -mavgen(nset,gen,icon)**2/dble(nevt)))
      enddo
#endif
        enddo
      enddo

 1000 format(" Total cross section    = ",2g18.10)
 1001 format(" Number of events       = ",i10,:" of ",i10)
 1002 format(" Cross section          = ",2g18.10)
 1003 format(" Standard deviation     = ",2g18.10)
 1004 format(" Relative corrections   = ",1f18.10)
 1005 format(" Maximal weight         = ",1f18.10)
 1006 format(" Rejected events        = ",i10,1f17.4,"%")
 1007 format(" (event = ",i9,", channel = ",i8,")")
 1017 format(" (event = ",i9,")")
 1008 format(" Number of unweighted events          = ",i10)
 1009 format(" Number of negative unweighted events = ",i10)

 1021 format(1x,a5,a5,": Cross section          = ",1g18.10)
 1022 format(12x," Standard deviation     = ",1g18.10)

      write(nout,'(a)')' '
      write(nout,'(a)')' Results:'
      write(nout,'(a)')' --------'
      write(nout,1001)nevt
      write(nout,1000)average
      write(nout,1003)sigma       ! from sum_n (sum_icon w_icon)^2
      write(nout,1003)sigmaa      ! from sum_n sum_icon (w_icon)^2
      write(nout,1005)weighttotmax/dble(nevt)
      if(average.ne.0d0.and.avborn.ne.0d0)then
        ngen=0
        do gen=0,ngenerator
          ngen=ngen+sborn(gen)
        enddo
        write(nout,1004)average/avborn-dble(ngen)
        write(nout,1003)dabs(average/avborn)
     &    *sqrt((sigma/average)**2+(siborn/avborn)**2)
      endif
      write(nout,'(a)')' '
      write(nout,'(a)')' '
      write(nout,'(a)')' Born contribution'
      write(nout,1000)avborn
      write(nout,1003)siborn
      write(nout,*)

#ifdef SUBPROCESSES
      do i1=-4,4
      do i2=-4,4
      if(i1*i2.ne.0d0) then
        write(nout,1021)part(i1),part(i2),avbornii(i1,i2)
        write(nout,1022)sibornii(i1,i2)
      endif
      enddo
      enddo
#endif

      write(nout,'(a)')' '
      write(nout,'(a)')' '
      write(nout,'(a)')' Complete results'
      write(nout,1000)average
      write(nout,1003)sigma
      write(nout,1003)sigmaa
      write(nout,*)

#ifdef SUBPROCESSES
      do i1=-4,4
      do i2=-4,4
      if(i1*i2.ne.0d0.and.averageii(i1,i2).ne.0d0) then
        write(nout,1021)part(i1),part(i2),averageii(i1,i2)
        write(nout,1022)sigmaii(i1,i2)
      endif
      enddo
      enddo
      i1=0
      part(i1)='p/gl'
      do i2=-4,4
      if(i2.ne.0d0.and.averageii(0,i2).ne.0d0) then
        write(nout,1021)part(i1),part(i2),averageii(i1,i2)
        write(nout,1022)sigmaii(i1,i2)
      endif
      enddo
      i2=0
      part(i2)='p/gl'
      do i1=-4,4
      if(i1.ne.0d0.and.averageii(i1,0).ne.0d0) then
        write(nout,1021)part(i1),part(i2),averageii(i1,i2)
        write(nout,1022)sigmaii(i1,i2)
      endif
      enddo
#endif


      write(nout,'(a)')' '
      do gen=0,ngenerator
        do icon=1,ncontr(gen)
          if(sgencon(gen,icon).ne.0)then
            write(nout,'(a)')' '
            write(nout,'(a23,i2)')' Phase-space generator ',gen
            write(nout,'(a23,i2," of ",i2)')
     &          ' contribution ',icon,ncontr(gen)
            write(nout,'(" Process: ",a60)')processname(gen)
            write(nout,1001)nevt/freqgencon(gen,icon),nevt
            write(nout,1000)avgen(gen,icon)
            write(nout,1003)sigen(gen,icon)
            write(nout,1006)rejgen(gen,icon),
     &          1d2*rejgen(gen,icon)/dble(nevt)*freqgencon(gen,icon)
            write(nout,1005)weightmax(gen,icon)/dble(nevt)
            write(nout,1007)nmax(gen,icon),channelmax(gen,icon)

#ifdef SUBPROCESSES
            if(gen.le.2.and.icon.le.5) then
              do i1=-4,4
                do i2=-4,4
                  if(i1*i2.ne.0d0.and.avgenii(gen,icon,i1,i2).ne.0d0)
     &                then
                    write(nout,1021)part(i1),part(i2),
     &                  avgenii(gen,icon,i1,i2)
                    write(nout,1022)sigenii(gen,icon,i1,i2)
                  endif
                enddo
              enddo
            endif

            if(gen.eq.3.or.gen.eq.0.and.icon.eq.6) then
              part(0) = 'phot'
              i1=0
              do i2=-4,4
                if(i2.ne.0d0) then
                  write(nout,1021)part(i1),part(i2),
     &                avgenii(gen,icon,i1,i2)
                  write(nout,1022)sigenii(gen,icon,i1,i2)
                endif
              enddo
            endif
            if(gen.eq.4.or.gen.eq.0.and.icon.eq.7) then
              part(0) = 'phot'
              i2=0
              do i1=-4,4
                if(i1.ne.0d0) then
                  write(nout,1021)part(i1),part(i2),
     &                avgenii(gen,icon,i1,i2)
                  write(nout,1022)sigenii(gen,icon,i1,i2)
                endif
              enddo
            endif
            if(gen.eq.5.or.gen.eq.0.and.icon.eq.8) then
              part(0) = 'gluo'
              i1=0
              do i2=-4,4
                if(i2.ne.0d0) then
                  write(nout,1021)part(i1),part(i2),
     &                avgenii(gen,icon,i1,i2)
                  write(nout,1022)sigenii(gen,icon,i1,i2)
                endif
              enddo
            endif
            if(gen.eq.6.or.gen.eq.0.and.icon.eq.9) then
              part(0) = 'gluo'
              i2=0
              do i1=-4,4
                if(i1.ne.0d0) then
                  write(nout,1021)part(i1),part(i2),
     &                avgenii(gen,icon,i1,i2)
                  write(nout,1022)sigenii(gen,icon,i1,i2)
                endif
              enddo
            endif
#endif

            if(lnoutmc.ge.5) then
              do cha=1,nchannel(gen)
                write(nout,'(a)')' '
                write(nout,'(a23,i3)')' channel        ',cha
                write(nout,'(a23,g23.8)')' alpha_channel  ',
     &              alphaopt(cha,opt(gen),gen)
                write(nout,1001)nevtgencha(gen,icon,cha)
     &              /freqgencon(gen,icon)
                write(nout,1002)avgenchan(gen,icon,cha)
c    &              ,avgencha(gen,icon,cha)
                write(nout,1003)sigenchan(gen,icon,cha)
c    &              ,sigencha(gen,icon,cha)
                write(nout,1006)rejcha(gen,icon,cha),
     &              ((1d2*rejcha(gen,icon,cha))/dble(nevt)
     &              *freqgencon(gen,icon))
                write(nout,1005)weightmaxcha(gen,icon,cha)/dble(nevt)
                write(nout,1017)nmaxcha(gen,icon,cha)
              enddo
            endif
          else
            write(nout,'(a)')' '
            write(nout,'(a23,i2)')' Phase-space generator ',gen
            write(nout,'(a23,i2," of ",i2)')
     &          ' contribution ',icon,ncontr(gen)
            write(nout,'(" Process: ",a60)')processname(gen)
            write(nout,'(a)')' no contribution'
          endif
        enddo
      enddo

 300  format(/' Summary of results')
 301  format(/' Relative corrections')
 400  format(a12,i2,a14,i2,a1,1g18.10,' +/- ',1g18.10)
 500  format(a31,1g18.10,' +/- ',1g18.10)
      write(nout,300)
      write(nout,500)' Born cross section:',
     &    avborn,siborn
      write(nout,500)' complete cross section:',
     &    average,sigma
      write(nout,500)' complete cross section:',
     &    (avgen(0,1)+avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1))
     &    +(avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1)),
     &    sqrt((sigen(0,1))**2+(sigen(0,5))**2+(sigen(0,4))**2
     &         +(sigen(0,8))**2+(sigen(0,9))**2
     &         +(sigen(2,1))**2+(sigen(5,1))**2+(sigen(6,1))**2
     &         +(sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2)
      write(nout,500)' QCD corr. to cross section:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1)),
     &    sqrt((sigen(0,5))**2+(sigen(0,4))**2
     &         +(sigen(0,8))**2+(sigen(0,9))**2
     &         +(sigen(2,1))**2+(sigen(5,1))**2+(sigen(6,1))**2)
      write(nout,500)' incoming gluon cross section:',
     &    (avgen(0,8)+avgen(0,9)+avgen(5,1)+avgen(6,1)),
     &    sqrt((sigen(0,8))**2+(sigen(0,9))**2
     &         +(sigen(5,1))**2+(sigen(6,1))**2)
      write(nout,500)' virt+real qcd cross section:',
     &    (avgen(0,5)+avgen(0,4)+avgen(2,1)),
     &    sqrt((sigen(0,5))**2+(sigen(0,4))**2+(sigen(2,1))**2)
      write(nout,500)' virt. qcd cross section:',
     &    avgen(0,5),sigen(0,5)
      write(nout,500)' outgoing gluon cross section:',
     &    (avgen(0,4)+avgen(2,1)),
     &    sqrt((sigen(0,4))**2+(sigen(2,1))**2)
      write(nout,500) 'ew corr. to cross section:',
     &    (avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1)),
     &    sqrt((sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2)
      write(nout,500) 'incoming photon cross section:',
     &    (avgen(0,6)+avgen(0,7)+avgen(3,1)+avgen(4,1)),
     &    sqrt( (sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(3,1))**2+(sigen(4,1))**2)
      write(nout,500) ' virt+real ew cross section:',
     &    (avgen(0,2)+avgen(0,3)+avgen(1,1)),
     &    sqrt((sigen(0,2))**2+(sigen(0,3))**2+(sigen(1,1))**2)
      write(nout,500) ' virt. ew cross section:',
     &    avgen(0,2),sigen(0,2)
      write(nout,500) 'outgoing photon cross section:',
     &    (avgen(0,3)+avgen(1,1)),
     &    sqrt((sigen(0,3))**2+(sigen(1,1))**2)

      write(nout,500)

      do gen=0,ngenerator
        do icon=1,ncontr(gen)
          if(avgen(gen,icon).ne.0d0)then
            write(nout,400)' generator',gen,
     &    ', contribution',icon,':',avgen(gen,icon),sigen(gen,icon)
          endif
        enddo
      enddo

      write(nout,301)
      write(nout,500) 'total correction:',
     &    average/avborn-1d0,
     &    average/avborn*sqrt((sigma/average)**2+(siborn/avborn)**2)
      write(nout,500)' total correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1)
     &    +avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt( (sigen(0,5))**2+(sigen(0,4))**2
     &         +(sigen(0,8))**2+(sigen(0,9))**2
     &         +(sigen(2,1))**2+(sigen(5,1))**2+(sigen(6,1))**2
     &         +(sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2
     &         +((avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &         +avgen(2,1)+avgen(5,1)+avgen(6,1)
     &         +avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &         +avgen(1,1)+avgen(3,1)+avgen(4,1))*siborn/avborn)**2)
     &       /avborn
      write(nout,500) 'total qcd correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1))/avborn,
     &    sqrt((sigen(0,5))**2+(sigen(0,4))**2
     &        +(sigen(0,8))**2+(sigen(0,9))**2
     &        +(sigen(2,1))**2
     &        +(sigen(5,1))**2+(sigen(6,1))**2
     &        +((avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &        +avgen(2,1)+avgen(5,1)+avgen(6,1))*siborn/avborn)**2)
     &        /avborn
      write(nout,500) 'virt+real qcd correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(2,1))/avborn,
     &    sqrt((sigen(0,5))**2+(sigen(0,4))**2
     &        +(sigen(2,1))**2
     &        +((avgen(0,5)+avgen(0,4)+avgen(2,1))*siborn/avborn)**2)
     &        /avborn
      write(nout,500) ' incoming gluon correction:',
     &    (avgen(0,8)+avgen(0,9)+avgen(5,1)+avgen(6,1))/avborn,
     &    sqrt((sigen(0,8))**2+(sigen(0,9))**2
     &        +(sigen(5,1))**2+(sigen(6,1))**2
     &        +((avgen(0,8)+avgen(0,9)
     &          +avgen(5,1)+avgen(6,1))*siborn/avborn)**2)
     &        /avborn
      write(nout,500) 'total ew correction:',
     &    (avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt((sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2
     &    +((avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))*siborn/avborn)**2)/avborn
      write(nout,500) 'virt+real ew correction:',
     &    (avgen(0,2)+avgen(0,3)+avgen(1,1))/avborn,
     &    sqrt((sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(1,1))**2
     &    +((avgen(0,2)+avgen(0,3)+avgen(1,1))*siborn/avborn)**2)/avborn
      write(nout,500) 'incoming photon correction:',
     &    (avgen(0,6)+avgen(0,7)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt((sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(3,1))**2+(sigen(4,1))**2
     &    +((avgen(0,6)+avgen(0,7)
     &     +avgen(3,1)+avgen(4,1))*siborn/avborn)**2)/avborn

#ifdef multpdf
C include varying alpha_s from PDF-set to PDF-set
      do nset=1,pdfmemberto-pdfmemberfrom+1
	mavgen(nset,0,5)=mavgen(nset,0,5)*alphasratio(nset)
	mavgen(nset,0,4)=mavgen(nset,0,4)*alphasratio(nset)
	mavgen(nset,0,8)=mavgen(nset,0,8)*alphasratio(nset)
	mavgen(nset,0,9)=mavgen(nset,0,9)*alphasratio(nset)
	mavgen(nset,2,1)=mavgen(nset,2,1)*alphasratio(nset)
	mavgen(nset,5,1)=mavgen(nset,5,1)*alphasratio(nset)
	mavgen(nset,6,1)=mavgen(nset,6,1)*alphasratio(nset)
	msigen(nset,0,5)=msigen(nset,0,5)*alphasratio(nset)
	msigen(nset,0,4)=msigen(nset,0,4)*alphasratio(nset)
	msigen(nset,0,8)=msigen(nset,0,8)*alphasratio(nset)
	msigen(nset,0,9)=msigen(nset,0,9)*alphasratio(nset)
	msigen(nset,2,1)=msigen(nset,2,1)*alphasratio(nset)
	msigen(nset,5,1)=msigen(nset,5,1)*alphasratio(nset)
	msigen(nset,6,1)=msigen(nset,6,1)*alphasratio(nset)
C	mavgen(nset,0,5)=(mavgen(nset,0,5)+avgen(0,5))*
C     &                    alphasratio(nset)-avgen(0,5)
C	mavgen(nset,0,4)=(mavgen(nset,0,4)+avgen(0,4))*
C     &                    alphasratio(nset)-avgen(0,4)
C	mavgen(nset,0,8)=(mavgen(nset,0,8)+avgen(0,8))*
C     &                    alphasratio(nset)-avgen(0,8)
C	mavgen(nset,0,9)=(mavgen(nset,0,9)+avgen(0,9))*
C     &                    alphasratio(nset)-avgen(0,9)
C	mavgen(nset,2,1)=(mavgen(nset,2,1)+avgen(2,1))*
C     &                    alphasratio(nset)-avgen(2,1)
C	mavgen(nset,5,1)=(mavgen(nset,5,1)+avgen(5,1))*
C     &                    alphasratio(nset)-avgen(5,1)
C	mavgen(nset,6,1)=(mavgen(nset,6,1)+avgen(6,1))*
C     &                    alphasratio(nset)-avgen(6,1)
C	msigen(nset,0,5)=sqrt(msigen(nset,0,5)**2*alphasratio(nset)**2+
C     &                    (alphasratio(nset)-1D0)**2*sigen(0,5)**2)
C	msigen(nset,0,4)=sqrt(msigen(nset,0,4)**2*alphasratio(nset)**2+
C     &                    (alphasratio(nset)**2-1D0)**2*sigen(0,4)**2)
C	msigen(nset,0,8)=sqrt(msigen(nset,0,8)**2*alphasratio(nset)**2+
C     &                    (alphasratio(nset)**2-1D0)**2*sigen(0,8)**2)
C	msigen(nset,0,9)=sqrt(msigen(nset,0,9)**2*alphasratio(nset)**2+
C     &                    (alphasratio(nset)-1D0)**2*sigen(0,9)**2)
C	msigen(nset,2,1)=sqrt(msigen(nset,2,1)**2*alphasratio(nset)**2+
C     &                    (alphasratio(nset)-1D0)**2*sigen(2,1)**2)
C	msigen(nset,5,1)=sqrt(msigen(nset,5,1)**2*alphasratio(nset)**2+
C     &                    (alphasratio(nset)-1D0)**2*sigen(5,1)**2)
C	msigen(nset,6,1)=sqrt(msigen(nset,6,1)**2*alphasratio(nset)**2+
C     &                    (alphasratio(nset)-1D0)**2*sigen(6,1)**2)
      enddo
      if(pdfmemberto.ge.pdfmemberfrom)then
        write(nout,*) ''
        write(nout,*) 'Results for different PDF sets:'
      endif
      do nset=1,pdfmemberto-pdfmemberfrom+1
      write(nout,*) ''
      write(nout,*) 'PDF set:',pdfmemberfrom-1+nset
      write(nout,500)' Born cross section:',
     &    mavborn(nset),msiborn(nset)
      write(nout,500)' complete cross section:',
     &    (mavgen(nset,0,1)+mavgen(nset,0,5)+mavgen(nset,0,4)+
     &     mavgen(nset,0,8)+mavgen(nset,0,9)
     &    +mavgen(nset,2,1)+mavgen(nset,5,1)+mavgen(nset,6,1))
     &    +(mavgen(nset,0,2)+mavgen(nset,0,3)+mavgen(nset,0,6)+
     &      mavgen(nset,0,7)
     &    +mavgen(nset,1,1)+mavgen(nset,3,1)+mavgen(nset,4,1)),
     &    sqrt((msigen(nset,0,1))**2+(msigen(nset,0,5))**2+
     &         (msigen(nset,0,4))**2
     &         +(msigen(nset,0,8))**2+(msigen(nset,0,9))**2
     &         +(msigen(nset,2,1))**2+(msigen(nset,5,1))**2+
     &          (msigen(nset,6,1))**2
     &         +(msigen(nset,0,2))**2+(msigen(nset,0,3))**2
     &         +(msigen(nset,0,6))**2+(msigen(nset,0,7))**2
     &         +(msigen(nset,1,1))**2+(msigen(nset,3,1))**2+
     &          (msigen(nset,4,1))**2)
      write(nout,500)' QCD corr. to cross section:',
     &    (mavgen(nset,0,5)+mavgen(nset,0,4)+mavgen(nset,0,8)+
     &     mavgen(nset,0,9)
     &    +mavgen(nset,2,1)+mavgen(nset,5,1)+mavgen(nset,6,1)),
     &    sqrt((msigen(nset,0,5))**2+(msigen(nset,0,4))**2
     &         +(msigen(nset,0,8))**2+(msigen(nset,0,9))**2
     &         +(msigen(nset,2,1))**2+(msigen(nset,5,1))**2+
     &          (msigen(nset,6,1))**2)
      write(nout,500)' incoming gluon cross section:',
     &    (mavgen(nset,0,8)+mavgen(nset,0,9)+mavgen(nset,5,1)+
     &     mavgen(nset,6,1)),
     &    sqrt((msigen(nset,0,8))**2+(msigen(nset,0,9))**2
     &         +(msigen(nset,5,1))**2+(msigen(nset,6,1))**2)
      write(nout,500)' virt+real qcd cross section:',
     &    (mavgen(nset,0,5)+mavgen(nset,0,4)+mavgen(nset,2,1)),
     &    sqrt((msigen(nset,0,5))**2+(msigen(nset,0,4))**2+
     &         (msigen(nset,2,1))**2)
      write(nout,500)' virt. qcd cross section:',
     &    mavgen(nset,0,5),msigen(nset,0,5)
      write(nout,500)' outgoing gluon cross section:',
     &    (mavgen(nset,0,4)+mavgen(nset,2,1)),
     &    sqrt((msigen(nset,0,4))**2+(msigen(nset,2,1))**2)
      write(nout,500) 'ew corr. to cross section:',
     &    (mavgen(nset,0,2)+mavgen(nset,0,3)+mavgen(nset,0,6)+
     &     mavgen(nset,0,7)
     &    +mavgen(nset,1,1)+mavgen(nset,3,1)+mavgen(nset,4,1)),
     &    sqrt((msigen(nset,0,2))**2+(msigen(nset,0,3))**2
     &         +(msigen(nset,0,6))**2+(msigen(nset,0,7))**2
     &         +(msigen(nset,1,1))**2+(msigen(nset,3,1))**2+
     &          (msigen(nset,4,1))**2)
      write(nout,500) 'incoming photon cross section:',
     &    (mavgen(nset,0,6)+mavgen(nset,0,7)+mavgen(nset,3,1)+
     &     mavgen(nset,4,1)),
     &    sqrt( (msigen(nset,0,6))**2+(msigen(nset,0,7))**2
     &         +(msigen(nset,3,1))**2+(msigen(nset,4,1))**2)
      write(nout,500) ' virt+real ew cross section:',
     &    (mavgen(nset,0,2)+mavgen(nset,0,3)+mavgen(nset,1,1)),
     &    sqrt((msigen(nset,0,2))**2+(msigen(nset,0,3))**2+
     &         (msigen(nset,1,1))**2)
      write(nout,500) ' virt. ew cross section:',
     &    mavgen(nset,0,2),msigen(nset,0,2)
      write(nout,500) 'outgoing photon cross section:',
     &    (mavgen(nset,0,3)+mavgen(nset,1,1)),
     &    sqrt((msigen(nset,0,3))**2+(msigen(nset,1,1))**2)
      enddo
#endif

      if(outputfile2(1:1).eq.' ')then
        nout2=6
      else
        nout2=23
        open(unit=nout2,file=outputfile2,access='append',
     &	status='unknown')
      endif
c
 2222 format(f12.5,4g18.10,6f14.7)
 2223 format(g12.5,4g18.10,6f14.7)
c     if(outputvar.eq.'mh'.or.outputvar.eq.' ') then
      if(outputvar.eq.'mh') then
         write(nout2,2222) mcm(25),
c      ' Born cross section:',
     &    avborn,siborn,
c      ' complete cross section:',
     &    average,sigma,
c      ' total correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1)
     &    +avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt( (sigen(0,5))**2+(sigen(0,4))**2
     &         +(sigen(0,8))**2+(sigen(0,9))**2
     &         +(sigen(2,1))**2+(sigen(5,1))**2+(sigen(6,1))**2
     &         +(sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2
     &         +((avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &         +avgen(2,1)+avgen(5,1)+avgen(6,1)
     &         +avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &         +avgen(1,1)+avgen(3,1)+avgen(4,1))*siborn/avborn)**2)
     &       /avborn,
c      'total ew correction:',
     &    (avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt((sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2
     &    +((avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))*siborn/avborn)**2)/avborn,
c      'total qcd correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1))/avborn,
     &    sqrt((sigen(0,5))**2+(sigen(0,4))**2
     &        +(sigen(0,8))**2+(sigen(0,9))**2
     &        +(sigen(2,1))**2
     &        +(sigen(5,1))**2+(sigen(5,1))**2
     &        +((avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &        +avgen(2,1)+avgen(5,1)+avgen(6,1))*siborn/avborn)**2)
     &        /avborn
      else if(outputvar.eq.'qcdfacscalefac2') then
         write(nout2,2222) qcdfacscalefac*qcdfacscalefac,
c      ' Born cross section:',
     &    avborn,siborn,
c      ' complete cross section:',
     &    average,sigma,
c      ' total correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1)
     &    +avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt( (sigen(0,5))**2+(sigen(0,4))**2
     &         +(sigen(0,8))**2+(sigen(0,9))**2
     &         +(sigen(2,1))**2+(sigen(5,1))**2+(sigen(6,1))**2
     &         +(sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2
     &         +((avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &         +avgen(2,1)+avgen(5,1)+avgen(6,1)
     &         +avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &         +avgen(1,1)+avgen(3,1)+avgen(4,1))*siborn/avborn)**2)
     &       /avborn,
c      'total ew correction:',
     &    (avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt((sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2
     &    +((avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))*siborn/avborn)**2)/avborn,
c      'total qcd correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1))/avborn,
     &    sqrt((sigen(0,5))**2+(sigen(0,4))**2
     &        +(sigen(0,8))**2+(sigen(0,9))**2
     &        +(sigen(2,1))**2
     &        +(sigen(5,1))**2+(sigen(5,1))**2
     &        +((avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &        +avgen(2,1)+avgen(5,1)+avgen(6,1))*siborn/avborn)**2)
     &        /avborn
      elseif(outputvar.eq.'deltac') then
         write(nout2,2223) mcslideltac,
c      ' Born cross section:',
     &    avborn,siborn,
c      ' complete cross section:',
     &    average,sigma,
c      ' total correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1)
     &    +avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt( (sigen(0,5))**2+(sigen(0,4))**2
     &         +(sigen(0,8))**2+(sigen(0,9))**2
     &         +(sigen(2,1))**2+(sigen(5,1))**2+(sigen(6,1))**2
     &         +(sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2
     &         +((avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &         +avgen(2,1)+avgen(5,1)+avgen(6,1)
     &         +avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &         +avgen(1,1)+avgen(3,1)+avgen(4,1))*siborn/avborn)**2)
     &       /avborn,
c      'total ew correction:',
     &    (avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt((sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2
     &    +((avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))*siborn/avborn)**2)/avborn,
c      'total qcd correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1))/avborn,
     &    sqrt((sigen(0,5))**2+(sigen(0,4))**2
     &        +(sigen(0,8))**2+(sigen(0,9))**2
     &        +(sigen(2,1))**2
     &        +(sigen(5,1))**2+(sigen(5,1))**2
     &        +((avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &        +avgen(2,1)+avgen(5,1)+avgen(6,1))*siborn/avborn)**2)
     &        /avborn
      elseif(outputvar.eq.'deltas2') then
         write(nout2,2223) mcslideltas*mcslideltas,
c      ' Born cross section:',
     &    avborn,siborn,
c      ' complete cross section:',
     &    average,sigma,
c      ' total correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1)
     &    +avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt( (sigen(0,5))**2+(sigen(0,4))**2
     &         +(sigen(0,8))**2+(sigen(0,9))**2
     &         +(sigen(2,1))**2+(sigen(5,1))**2+(sigen(6,1))**2
     &         +(sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2
     &         +((avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &         +avgen(2,1)+avgen(5,1)+avgen(6,1)
     &         +avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &         +avgen(1,1)+avgen(3,1)+avgen(4,1))*siborn/avborn)**2)
     &       /avborn,
c      'total ew correction:',
     &    (avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))/avborn,
     &    sqrt((sigen(0,2))**2+(sigen(0,3))**2
     &         +(sigen(0,6))**2+(sigen(0,7))**2
     &         +(sigen(1,1))**2+(sigen(3,1))**2+(sigen(4,1))**2
     &    +((avgen(0,2)+avgen(0,3)+avgen(0,6)+avgen(0,7)
     &    +avgen(1,1)+avgen(3,1)+avgen(4,1))*siborn/avborn)**2)/avborn,
c      'total qcd correction:',
     &    (avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &    +avgen(2,1)+avgen(5,1)+avgen(6,1))/avborn,
     &    sqrt((sigen(0,5))**2+(sigen(0,4))**2
     &        +(sigen(0,8))**2+(sigen(0,9))**2
     &        +(sigen(2,1))**2
     &        +(sigen(5,1))**2+(sigen(5,1))**2
     &        +((avgen(0,5)+avgen(0,4)+avgen(0,8)+avgen(0,9)
     &        +avgen(2,1)+avgen(5,1)+avgen(6,1))*siborn/avborn)**2)
     &        /avborn
       endif	

      if(nout2.ne.6) close(nout2)

#ifdef mpiuse
       endif
       If(iMyRank.nE.0) gen=1
#endif

c output histograms
c                write(*,*) 'writeresult: settings 4'
      call settings(pbeam,pevent,weight(1,1),nevt,gen,4)

#ifdef mpiuse
      if(iMyRank.EQ.0) Then
#endif

      if(lnoutmc.ge.3)then
c output exit statistics
 600  format(/' Rejected events')
 700  format(a10,i2,a8,i3,a1,i12,a3,f10.5,a3)
 701  format(a10,i2,a12,i12,a3,f10.5,a3)
 702  format(a10,i2,a12,i12,a3,f10.5,a3/)
      write(nout,600)
      endif
      do gen=0,ngenerator
        if(sgen(gen).ne.0)then
          count=0
          do iexit=0,maxexit
            if(mcexitevt(iexit,gen).ne.0)then
              if(lnoutmc.ge.3)then
              write(nout,700)' generator',gen,
     &            ',   exit',iexit,':',mcexitevt(iexit,gen)
     &            ,' = ',
     &            mcexitevt(iexit,gen)
     &            /dble(nevents)*freqgencon(gen,1)*1d2
     &            ,' % '
              endif
              count=count+mcexitevt(iexit,gen)
            endif
          enddo

          if( (count-mcexitevt(0,gen))
     &        /dble(nevents)*freqgencon(gen,1).gt.1d-2) then
            write(*,*) ' HAWK: run unreliable'
            write(*,*) '       please contact the authors'//
     &          ' providing input and output'
            stop
          elseif( (count-mcexitevt(0,gen))
     &        /dble(nevents)*freqgencon(gen,1).gt.1d-3) then
            write(*,*) ' HAWK: run potentially unreliable'
            write(*,*) '       use results with caution'
          endif

          if(lnoutmc.ge.3)then
          write(nout,701)' generator',gen,
     &        ', all exits:',count-mcexitevt(0,gen)
     &            ,' = ',
     &            (count-mcexitevt(0,gen))
     &        /dble(nevents)*freqgencon(gen,1)*1d2
     &            ,' % '
          write(nout,702)' generator',gen,
     &        ', sum exits:',count
     &            ,' = ',
     &            count/dble(nevents)*freqgencon(gen,1)*1d2
     &            ,' % '
          endif
        endif
      enddo

      if(lnoutmc.ge.3)then
c output cut statistics
 800  format(/' Cutted events')
 900  format(a10,i2,a7,i2,a7,i3,a1,i12,a3,f10.6,a3)
 901  format(a10,i2,a7,i2,a11,i12,a3,f10.6,a3)
 902  format(a21,a7,i3,a1,i12,a3,f10.6,a3)
 903  format(a21,a11,i12,a3,f10.6,a3)
 904  format(a10,i2,a7,i2,a11,i12,a3,f10.6,a3/)
 905  format(a21,a11,i12,a3,f10.6,a3/)
      write(nout,800)
      count=0
      if(swborn(0).ne.0)then
        do icut=1,maxcut
          if(mccutevt_0(icut).ne.0)then
            write(nout,902)' Born generator',
     &          ',   cut',icut,':',mccutevt_0(icut)
     &      ,' = '
     &      ,mccutevt_0(icut)/dble(nevents)*freqgencon(0,1)*1d2
     &      ,' % '
            count=count+mccutevt_0(icut)
          endif
        enddo
        write(nout,903)' Born generator',
     &      ', all cuts:',count
     &      ,' = ',
     &      count/dble(nevents)*freqgencon(0,1)*1d2
     &      ,' % '
        write(nout,903)' Born generator',
     &      ',  no cut :',mccutevt_0(0)
     &      ,' = ',
     &      mccutevt_0(0)
     &      /dble(nevents)*freqgencon(0,1)*1d2
     &      ,' % '
        write(nout,905)' Born generator',
     &      ', sum cuts:',count+mccutevt_0(0)
     &      ,' = ',
     &      (count+mccutevt_0(0))/dble(nevents)*freqgencon(0,1)*1d2
     &      ,' % '
      endif
      do gen=0,ngenerator
        if(sgen(gen).ne.0)then
          do icon=1,ncontr(gen)
            count=0
            if(sgencon(gen,icon).ne.0)then
              if(gen.eq.0)then
                do icut=1,maxcut
                  if(mccutevt(icut,gen,icon).ne.0)then
                    write(nout,900)' generator',gen,', con',icon,
     &                  ',   cut',icut,':',mccutevt(icut,gen,icon)
     &            ,' = ',
     &            mccutevt(icut,gen,icon)
     &                  /dble(nevents)*freqgencon(gen,icon)*1d2
     &            ,' % '
                    count=count+mccutevt(icut,gen,icon)
                  endif
                enddo
              else
                do icut=-2*nexternal(gen),maxcut
                  if(mccutevt(icut,gen,icon).ne.0.and.icut.ne.0)then
                    write(nout,900)' generator',gen,', con',icon,
     &                  ',   cut',icut,':',mccutevt(icut,gen,icon)
     &            ,' = ',
     &            mccutevt(icut,gen,icon)
     &                  /dble(nevents)*freqgencon(gen,1)*1d2
     &            ,' % '
                    count=count+mccutevt(icut,gen,icon)
                  endif
                enddo
              endif
              write(nout,901)' generator',gen,', con',icon,
     &            ', all cuts:',count
     &            ,' = ',
     &            (count)/dble(nevents)*freqgencon(gen,icon)*1d2
     &            ,' % '
               write(nout,901)' generator',gen,', con',icon,
     &            ',  no cut :',mccutevt(0,gen,icon)
     &            ,' = ',
     &            mccutevt(0,gen,icon)
     &            /dble(nevents)*freqgencon(gen,icon)*1d2
     &            ,' % '
             write(nout,904)' generator',gen,', con',icon,
     &            ', sum cuts:',count+mccutevt(0,gen,icon)
     &            ,' = ',
     &            (count+mccutevt(0,gen,icon))
     &             /dble(nevents)*freqgencon(gen,icon)*1d2
     &            ,' % '
            endif
          enddo
        endif
      enddo
      endif

 97   format(i2.2,'.',i2.2,'.20',i2.2)
 98   format(i2.2,'.',i2.2,'.',i2.2)
#ifdef G77
      call date_and_time(datum)
      da1=datum(7:8)
      da2=datum(5:6)
      da3=datum(1:4)
      datum=da1//'.'//da2//'.'//da3
      call itime(iarray)
      write(chtime,98)iarray(1),iarray(2),iarray(3)
      call cpu_time(rcpu)
c      write(*,*) 'cpu 1',rcpu
c      call time(chtime)
c      write(*,*) 'cpu 2',chtime
#endif
#ifdef PGF77
      call idate(imonth,iday,iyear)
      write(datum,97) iday,imonth,iyear
      call itime(iarray)
      write(chtime,98)iarray(1),iarray(2),iarray(3)
      rcpu = etime(tarray)
c      write(*,*) 'rcpu = ',etime(tarray),tarray
#endif
#ifdef IFORT77
      call date_and_time(datum)
      da1=datum(7:8)
      da2=datum(5:6)
      da3=datum(1:4)
      datum=da1//'.'//da2//'.'//da3
      call itime(iarray)
      write(chtime,98)iarray(1),iarray(2),iarray(3)
      call cpu_time(rcpu)
#endif
c
      icpuh=int(rcpu/3600.)
      icpum=int(rcpu/60.-icpuh*60)
      icpus=int(rcpu-icpuh*3600-icpum*60)
      icpu =icpuh*60+icpum
      write(nout,100) datum,chtime,icpuh,icpum,icpus
100   format(/'  date: ',a10,' ,    time: ',a8,' hour ,',
     &       '    cpu: ',i3,' h',i3,' m',i3,' s')

#ifdef mpiuse
       endif
#endif

      if(outputfile(1:2).ne.'  ')close(nout)	

      if(lprintweightmax) then
        close(20)
      endif

      end

***********************************************************************
      subroutine integration()
***********************************************************************
*     phase space integration                                         *
*     adapted from COFFERGG by Markus Roth                            *
*---------------------------------------------------------------------*
*     21.02.06  Ansgar Denner     last changed  21.02.06              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

#ifdef mpiuse
      include "mpif.h"
      integer status(MPI_STATUS_SIZE)
      double precision g(maxch,maxps)
      real*8 wi_ext(maxch,0:maxg),wran_ext(maxgr,maxr,0:maxg),dummy
      double precision keep_av,keep_si
      integer do_optimize,proccount
      common/do_optimize_common/do_optimize
#endif

#ifdef multpdf
#include "multpdf.h"
      integer nset
#endif
      integer iNumProcs, iErr, iMyRank
      common/mpistuff/iNumProcs,iMyRank

c local variables
      real*8 srandom(3*maxe+1)
      real*8 random(3*maxe+1),weighttot,sigmatot,weightgen(maxcon)
#ifdef SUBPROCESSES
      real*8 weighttotii(-4:4,-4:4),weightgenii(maxcon,-4:4,-4:4)
#endif
c      integer generator,switch,channel,phasespace,nsteps,generatormax
      integer switch,channel,phasespace
c      integer generatormax
      integer i1,i2

      real*8 sevent(maxe,maxe),kevent(maxe,0:3)

      logical misnan

c variables for collinear momentum output
c      real*8 invminevt,invmin
c      integer coll,outcoll,outsoft
c      logical initcoll
c      data initcoll /.true./, outsoft /50/
c      data invmin /1d20/
c      real*8 scoll(maxe),esoft
c      save scoll,esoft,invmin


c variables for testing
      real*8 weighttest(maxps,maxcon),weightgentest(maxps)
      character*5 part(-4:4)



c      set in public.F
c      data nsteps/10000/        ! output after 'nsteps' times
c      data nsteps/5000/        ! output after 'nsteps' times

c      write(*,*) 'integration: ngenerator =  ',ngenerator

 1050           format(1x,a28,": phasespace,weight = ",i4,g20.10)
 1051           format(1x,a28,":            weight = ",4x,g20.10)
 1029           format(/1x,a25,13x,3a24,a29)
 1030           format(/1x,a50/1x,a25,13x,3a24,a29)
 1031           format(1x,a5,a5,a21,' = ',5g24.14)
 1032           format(1x,a31,' = ',5g24.14)

* initialize hadrons


* initialize parton distributions

#ifdef mpiuse
      do_optimize=0
      proccount=0
#endif

c numerical integration
      do evt=1,nevents
      nevt=evt

c generate random numbers

c random1 choice of channel
c random3 choice of proton direction
c random4 optimization of x1
c random5 optimization of x2
c random6 x1  parton energy fraction
c random7 x2  parton energy fraction
c random8 z   energy fraction after collinear photon emission
c (random9 (1-z) for exclusive final states)
c random10 -- (3*maxe+1)  (3*maxe-8) phase space variable for maxe-2
c                    final state particles [(3maxe - 10) needed]?


c random1 choice of channel
c random2 choice of proton direction
c random5 optimization of x1
c random6 optimization of x2
c random7 x1
c random8 x2
c random9 z
c (random10 (1-z) for exclusive final states)
c random 10- (3*maxe+1)  phase space variable for maxe-1
c                       final state particles


#ifdef RANLUX
        call ranlux(random(1),3*maxe+1)
#else
        do rnr=1,3*maxe+1
c          call rans(random(rnr))
          call ransrcarry(random(rnr))
        enddo
#endif

#ifdef mpiuse
        proccount=proccount+1
        if(proccount.EQ.iNumProcs) proccount=0
        if(iNumProcs.EQ.1) proccount=0

        if(proccount.EQ.iMyRank) Then
#endif

c        write(*,*) 'integration: random =  ',random

        weighttot=0d0
        sigmatot=0d0
#ifdef multpdf
	do nset=1,pdfmemberto-pdfmemberfrom+1
          mweighttot(nset)=0D0
          msigmatot(nset)=0D0
	enddo
#endif
#ifdef SUBPROCESSES
        do i1=-4,4
          do i2=-4,4
            weighttotii(i1,i2)=0d0
          enddo
        enddo
#endif

* partonic energy fractions
* put here???

c        write(*,*) 'integration maxg,sqcdnondiag',maxg,sqcdnondiag

        do gen=0,ngenerator
          cogen=gen
          if(sgen(gen).ne.0)then

* if switch gets > 0 event does not contribute owing to numerical problems.
            switch=0

* choice of channel

c            write(*,*) 'integration: betaopt =',betaopt
c            write(*,*) 'integration use random(1) =',random(1)

            do cha=1,nchannel(gen)
              if(random(1).ge.betaopt(cha-1,gen).and.
     &          random(1).le.betaopt(cha,gen))then
                channel=cha
              endif
            enddo

c            write(*,*) 'integration generator,channel =',
c     &          gen,channel

            cocha=channel

c           if (evt.lt.68000000) goto 999

            call integrand(random(2),evt,channel,
     &        gen,switch)

c--> added AD 1.04.10
c discard unreasonable events
            do phasespace=1,nphsp(gen)
              do icon=1,ncontr(gen)
                if(switch.eq.0.and.
     &              rejectedcon(icon,phasespace).eq.0.and.
     &              misnan(weight(icon,phasespace)))then
                  switch=50
                  if(numout.lt.maxout.and.lnoutmc.ge.5)then
 105                format(/" event = ",i9,", gen = ",i4,
     &                  ", channel = ",i4,
     &                  ", exit = ",i4,(a))
                    write(nout,105) nevt,cogen,cocha,switch,
     &                  ', integration: NaN weight'
                    numout=numout+1
                  endif
                endif
              enddo
            enddo
c--<

c added AD 13.11.06
            if(switch.eq.0)then
              if(swborn(gen).eq.1)then
                if(rejected_0.eq.0)then
                  avborn=avborn+weight_0
                  siborn=siborn+weight_0**2

#ifdef multpdf
                  do nset=1,pdfmemberto-pdfmemberfrom+1
                    mavborn(nset)=mavborn(nset)+mweight_0(nset)
                    msiborn(nset)=msiborn(nset)+mweight_0(nset)**2
		  enddo
#endif
#ifdef SUBPROCESSES
                  do i1=-4,4
                    do i2=-4,4
                      avbornii(i1,i2)=avbornii(i1,i2)
     &                    +weightii_0(i1,i2)
                      sibornii(i1,i2)=sibornii(i1,i2)
     &                    +weightii_0(i1,i2)**2
                    enddo
                  enddo
#endif

                else
                  weight_0=0d0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
                  mweight_0(nset)=0d0
      enddo
#endif
                endif
                mccutevt_0(rejected_0)=mccutevt_0(rejected_0)+1
              endif
              do icon=1,ncontr(gen)
                if(mod(nevt,iNumProcs*freqgencon(gen,icon)).lt.
     &             iNumProcs)
     &              mccutevt(rejected(icon),gen,icon)=
     &                  mccutevt(rejected(icon),gen,icon)+1
              enddo

c              write(*,*) 'integration rej2',rejected(1),rejected(4)

              do icon=1,ncontr(gen)
                weightgen(icon)=0d0
#ifdef multpdf
		do nset=1,pdfmemberto-pdfmemberfrom+1
                  mweightgen(nset,icon)=0d0
		enddo
#endif
#ifdef SUBPROCESSES
                do i1=-4,4
                  do i2=-4,4
                    weightgenii(icon,i1,i2)=0d0
                  enddo
                enddo
#endif
              enddo
              do phasespace=1,nphsp(gen)
                do icon=1,ncontr(gen)

                  if(rejectedcon(icon,phasespace).eq.0)then
                    weightgen(icon)=weightgen(icon)
     &                  +weight(icon,phasespace)
#ifdef multpdf
		    do nset=1,pdfmemberto-pdfmemberfrom+1
                      mweightgen(nset,icon)=mweightgen(nset,icon)
     &                    +mweight(nset,icon,phasespace)
		    enddo
#endif

#ifdef SUBPROCESSES
                    do i1=-4,4
                    do i2=-4,4
                      weightgenii(icon,i1,i2)=weightgenii(icon,i1,i2)
     &                    +weightii(icon,i1,i2,phasespace)
                    enddo
                    enddo
#endif
                  endif
                  mccutsubevt(rejectedcon(icon,phasespace),
     &                                          gen,icon,phasespace)=
     &                mccutsubevt(rejectedcon(icon,phasespace),
     &                                         gen,icon,phasespace)+1
                enddo           ! icon
*     fill histograms with subevents
                call settings(pbeam,pevent(1,0,phasespace),
     &              weight(1,phasespace),evt,gen,2)

c                  if(gen.eq.2.and.nevt.eq.2868)
c     &                write(*,*) 'integration weight =',
c     &                phasespace,weight(1,phasespace)

              enddo             ! phasespace

c              do icon=1,ncontr(gen)
c                write(*,*) 'integration: weightgen: ',gen,icon,
c     &              weightgen(icon)
c              enddo

c                  if(gen.eq.2.and.nevt.eq.2868)
c     &                write(*,*) 'integration weight =',
c     &                weightgen(1)



c calculation of total weight
              do icon=1,ncontr(gen)
                weighttot=weighttot+weightgen(icon)
                sigmatot=sigmatot+weightgen(icon)**2
#ifdef multpdf
		do nset=1,pdfmemberto-pdfmemberfrom+1
                mweighttot(nset)=mweighttot(nset)+mweightgen(nset,icon)
                msigmatot(nset)=msigmatot(nset)+mweightgen(nset,icon)**2
		enddo
#endif
#ifdef SUBPROCESSES
                do i1=-4,4
                  do i2=-4,4
                    weighttotii(i1,i2)=weighttotii(i1,i2)
     &                  +weightgenii(icon,i1,i2)
                    avgenii(gen,icon,i1,i2)=avgenii(gen,icon,i1,i2)
     &                  +weightgenii(icon,i1,i2)
                    sigenii(gen,icon,i1,i2)=sigenii(gen,icon,i1,i2)
     &                  +weightgenii(icon,i1,i2)**2
                  enddo
                enddo
#endif
c     write(*,*) 'integration: weights    =',
c     c     &          (weight(phasespace),phasespace=1,nphsp(gen)),
c     &          weightgen(1),weighttot,gen
                avgencha(gen,icon,channel)=avgencha(gen,icon,channel)
     &              +weightgen(icon)
                sigencha(gen,icon,channel)=sigencha(gen,icon,channel)
     &              +weightgen(icon)**2
#ifdef multpdf
                do nset=1,pdfmemberto-pdfmemberfrom+1
                mavgencha(nset,gen,icon,channel)=
     &               mavgencha(nset,gen,icon,channel)
     &              +mweightgen(nset,icon)
                msigencha(nset,gen,icon,channel)=
     &               msigencha(nset,gen,icon,channel)
     &              +mweightgen(nset,icon)**2
	        enddo
#endif
                nevtgencha(gen,icon,channel)=nevtgencha(gen,icon,channel
     &              )+ 1
c rejected weight
                if(weightgen(icon).eq.0d0.and.
     &              mod(nevt,iNumProcs*freqgencon(gen,icon)).lt.
     &                  iNumProcs)then
                  rejcha(gen,icon,channel)=rejcha(gen,icon,channel)+1
                endif


c     for testing print event
c end testing print event

c maximal weight of generator
                if(dabs(weightgen(icon)).gt.
     &              weightmaxcha(gen,icon,channel))then
                  weightmaxcha(gen,icon,channel)=dabs(weightgen(icon))
                  nmaxcha(gen,icon,channel)=evt
#ifdef PRINTWEIGHTMAX
                  if(lprintweightmax) then
                    if(weightmaxcha(gen,icon,channel).gt.
     &                  weightmax(gen,icon)) then
                      weightmax(gen,icon)=weightmaxcha(gen,icon,channel)
                      channelmax(gen,icon) = channel
                      nmax(gen,icon) = evt

c     open(unit=20,file='dat.weightmax',status='unknown')
                      write(20,*) ' new maximal weight'
                      write(20,*) '      generator = ',gen
                      write(20,*) '   contribution = ',icon
                      write(20,*) '      channel   = ',channel
                      write(20,*) '      event     = ',evt
                      write(20,*) '      weight    = ',
     &                    weightmaxcha(gen,icon,channel)
                      write(20,*) '      gsum      = ',gsumm(1)
c     do cha=1,nchannel(gen)
c     write(20,*) ' g         = ',gcha(cha,1)
c     enddo
                      write(20,*) '      factor    = ',factor(gen)
                      write(20,*) '      wconv     = ',wconv
                      write(20,*) '      m2pdfpdf  = ',
     &                  weightmaxcha(gen,icon,channel)/wconv/factor(gen)
     &                    *gsumm(1)
                      write(20,*) '            x1  = ',x1print
                      write(20,*) '            x2  = ',x2print
                      if (gen.eq.0)then
                        write(20,*) '            x   = ',xprint,omxprint
                      endif
                      do ext=1,nextgen(gen)
                        do vcomp=0,3
                          write(20,100) ext,vcomp,pprint(ext,vcomp)
                        enddo
                      enddo
                      call dets(pprint,sevent,nextgen(gen),0)
                      do ext=1,nextgen(gen)
                        do ext2=ext+1,nextgen(gen)
                          write(20,110) ext,ext2,sevent(ext,ext2)
                        enddo
                      enddo
 100                  format(6x,' p(',i1,',',i1,')    = ',d24.18)
 110                  format(6x,' s(',i1,',',i1,')    = ',d24.18)
                      write(20,*)
c     close(20)
                    endif
                  endif
#endif
                endif


c     end loop over icons
              enddo


c test of convolution contribution to subtraction

c     end test of convolution contribution to subtraction

c     test of convolution contribution to slicing

c     end test of convolution contribution to slicing

c test of incoming photon convolution contribution to subtraction

c test of incoming photon convolution contribution to slicing

c test of incoming photon convolution contribution to eff. coll. fact
c     end test of incoming photon convolution contributions

c test of incoming gluon convolution contribution to subtraction

c     end test of incoming gluon convolution contributions

c test of incoming gluon convolution contribution with eff. coll. factors
c     end test of incoming gluon convolution contributions

c     end output of collinear events for test purposes

              if(mod(nevt,iNumProcs*freqgencon(gen,1)).lt.iNumProcs)
     &            mcexitevt(switch,gen)=mcexitevt(switch,gen)+1
            else
              mcexitevt(switch,gen)=mcexitevt(switch,gen)+1
            endif               ! switch.eq.0
          else
            weight_0=0d0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
            mweight_0(nset)=0d0
      enddo
#endif
          endif                 ! sgen(gen).ne.0
c     maximal weight
          if(dabs(weighttot).gt.weighttotmax)then
            weighttotmax=dabs(weighttot)
            nmaxtot=evt
          endif

c filling histograms with full event
c                write(*,*) 'integration: settings 3'
c                write(*,*) 'integration: settings weight_0',weight_0
          weight(1,1)=weight_0
c     write(*,*) 'integration: befset ',weight_0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
CAM          weight(1,1)=mweight_0(nset)
CAMc     write(*,*) 'integration: befset ',weight_0
      enddo
#endif
          call settings(pbeam,pborn,weight(1,1),evt,gen,3)

        enddo
c end loop over generators

c calculating results
c total results
        average=average+weighttot
c weighttot = sum_icon weightgen(icon)
        sigma=sigma+weighttot**2
c sigmatot = sum_icon weightgen(icon)**2
        sigmaa=sigmaa+sigmatot
#ifdef multpdf
        do nset=1,pdfmemberto-pdfmemberfrom+1
	  maverage(nset)=maverage(nset)+mweighttot(nset)
          msigma(nset)=msigma(nset)+mweighttot(nset)**2
          msigmaa(nset)=msigmaa(nset)+msigmatot(nset)
	enddo
#endif
c        write(*,*) 'integration: average = ',average,weighttot
#ifdef SUBPROCESSES
        do i1=-4,4
        do i2=-4,4
          averageii(i1,i2)=averageii(i1,i2)+weighttotii(i1,i2)
          sigmaii(i1,i2)=sigmaii(i1,i2)+weighttotii(i1,i2)**2
        enddo
        enddo
#endif
c partial results
        av=av+weighttot
        si=si+weighttot**2
c        write(*,*) 'integration av ',weight(4,1),
c     &      weight(4,2),weight(4,3),weightgen(4),
c     &      weighttot,av

#ifdef mpiuse
        endif

        do gen=0,ngenerator	
	  if(nevt.eq.nopt(opt(gen),gen).AND.iNumProcs.NE.1)then
	     if(iMyRank.NE.0) then
	       call MPI_Send(wi, maxch*(maxg+1),
     &              MPI_DOUBLE_PRECISION, 0, 1, MPI_COMM_WORLD, iErr)
               do i1=1,nchannel(gen)
		  wi(i1,gen)=0D0
	       enddo	
	     endif

	     if (iMyRank.EQ.0) then
	       do i2=1,iNumProcs-1
		 call MPI_Recv(wi_ext, maxch*(maxg+1),
     &                MPI_DOUBLE_PRECISION, i2, 1,
     &                MPI_COMM_WORLD, status, iErr)
        	 do i1=1,nchannel(gen)
		   wi(i1,gen)=wi(i1,gen)+wi_ext(i1,gen)
		 enddo
	       enddo

               do_optimize=1
	       call optimization(0D0,g,0D0,nevt,gen,nchannel(gen))
               do_optimize=0

	       do i2=1,iNumProcs-1
		 call MPI_Send(betaopt, (maxch+1)*(maxg+1),
     &                MPI_DOUBLE_PRECISION, i2, 2, MPI_COMM_WORLD, iErr)
		 call MPI_Send(alphaopt, maxch*maxo*(maxg+1),
     &                MPI_DOUBLE_PRECISION, i2, 3, MPI_COMM_WORLD, iErr)
	       enddo	
	     endif

             if(iMyRank.NE.0) then
	       call MPI_Recv(betaopt, (maxch+1)*(maxg+1),
     &              MPI_DOUBLE_PRECISION, 0, 2, MPI_COMM_WORLD,
     &              status, iErr)
	       call MPI_Recv(alphaopt, maxch*maxo*(maxg+1),
     &              MPI_DOUBLE_PRECISION, 0, 3, MPI_COMM_WORLD,
     &              status, iErr)
               if(nevents.gt.2*nevt) opt(gen)=opt(gen)+1
	     endif	 	 	
	  endif		

	  if(noptran(gen).NE.0) then
            if(mod(nevt,noptran(gen)).eq.0.AND.iNumProcs.NE.1)then
	      if(iMyRank.NE.0) then
	        call MPI_Send(wran, maxgr*maxr*(maxg+1),
     &              MPI_DOUBLE_PRECISION, 0, 1, MPI_COMM_WORLD, iErr)
                do igrv=1,maxr
        	  do igrp=1,maxgr
        	    wran(igrp,igrv,gen)=0d0
        	  enddo
                enddo
	      endif

	      if (iMyRank.EQ.0) then
	        do i2=1,iNumProcs-1
	 	  call MPI_Recv(wran_ext, maxgr*maxr*(maxg+1),
     &                MPI_DOUBLE_PRECISION, i2, 1, MPI_COMM_WORLD,
     &                status, iErr)
        	  do igrv=1,maxr
        	    do igrp=1,maxgr
        	      wran(igrp,igrv,gen)=wran(igrp,igrv,gen)+
     &                                   wran_ext(igrp,igrv,gen)
        	    enddo
        	  enddo
	        enddo

                do_optimize=1
                call ranopt(0.5D0,0.5D0,dummy,dummy,nevt,1,gen,switch)
                call ranopt(0.5D0,0.5D0,dummy,dummy,nevt,2,gen,switch)
                do_optimize=0

	        do i2=1,iNumProcs-1
		  call MPI_Send(beran, (maxgr+1)*maxr*(maxg+1),
     &                MPI_DOUBLE_PRECISION, i2, 2, MPI_COMM_WORLD, iErr)
		  call MPI_Send(alran, maxgr*maxr*(maxg+1),
     &                MPI_DOUBLE_PRECISION, i2, 3, MPI_COMM_WORLD, iErr)
	        enddo	
	      endif

              if(iMyRank.NE.0) then
	        call MPI_Recv(beran, (maxgr+1)*maxr*(maxg+1),
     &              MPI_DOUBLE_PRECISION, 0, 2, MPI_COMM_WORLD,
     &              status, iErr)
	        call MPI_Recv(alran, maxgr*maxr*(maxg+1),
     &              MPI_DOUBLE_PRECISION, 0, 3, MPI_COMM_WORLD,
     &              status, iErr)
	      endif	 	 	
	    endif	
	  endif	
	enddo
#endif

c preliminary results
c also possible for histograms!!!
        if(mod(evt,nsteps).eq.0)then
#ifdef mpiuse
	  if(iMyRank.EQ.0.AND.iNumProcs.NE.1) Then
	    do i2=1,iNumProcs-1
	      call MPI_Recv(keep_av, 1, MPI_DOUBLE_PRECISION,
     &                      i2, 10, MPI_COMM_WORLD, status, iErr)
	      call MPI_Recv(keep_si, 1, MPI_DOUBLE_PRECISION,
     &                      i2, 11, MPI_COMM_WORLD, status, iErr)
              av=av+keep_av
              si=si+keep_si
	      average=average+keep_av
              sigma=sigma+keep_si
            enddo
	  else if(iMyRank.NE.0.AND.iNumProcs.NE.1) Then
	    call MPI_Send(av, 1, MPI_DOUBLE_PRECISION,
     &                    0, 10, MPI_COMM_WORLD, iErr)
	    call MPI_Send(si, 1, MPI_DOUBLE_PRECISION,
     &                    0, 11, MPI_COMM_WORLD, iErr)
            av=0D0
            si=0D0
	  endif
        if(iMyRank.EQ.0) Then
#endif
          if(evt.eq.nsteps) then
            write(nout,*)' '
            write(nout,'(a)')'    events  :     intermediate results'
     &          //'     :     preliminary results '
          endif
          write(nout,'(i10,2("  : ",1f12.5," +-",1f12.5))')evt,
     &        av/dble(nsteps),
     &        sqrt(dabs(si-av**2/dble(nsteps)))/dble(nsteps),
     &        average/dble(evt),
     &        sqrt(dabs(sigma-average**2/dble(evt)))/dble(evt)
          av=0d0
          si=0d0
#ifdef mpiuse
        endif
#endif
        endif

c        write(*,*) 'integration: average = ',average

 999    nevt = evt
      enddo
c end loop over events

      do gen=0,ngenerator
        do icon=1,ncontr(gen)
          do cha=1,nchannel(gen)
            avgen(gen,icon) = avgen(gen,icon) + avgencha(gen,icon,cha)
            sigen(gen,icon) = sigen(gen,icon) + sigencha(gen,icon,cha)
#ifdef multpdf
            do nset=1,pdfmemberto-pdfmemberfrom+1
            mavgen(nset,gen,icon) = mavgen(nset,gen,icon) +
     &                              mavgencha(nset,gen,icon,cha)
            msigen(nset,gen,icon) = msigen(nset,gen,icon) +
     &                              msigencha(nset,gen,icon,cha)
	    enddo
#endif
            nevtgen(gen,icon) = nevtgen(gen,icon)
     &          + nevtgencha(gen,icon,cha)
            rejgen(gen,icon) = rejgen(gen,icon)
     &          + rejcha(gen,icon,cha)
            if (weightmax(gen,icon).lt.weightmaxcha(gen,icon,cha)) then
              weightmax(gen,icon) = weightmaxcha(gen,icon,cha)
              channelmax(gen,icon) = cha
              nmax(gen,icon) = nmaxcha(gen,icon,cha)
            endif
          enddo
        enddo
      enddo

c      write(*,*) 'integration avborn ',avborn
c      write(*,*) 'integration average ',average
c      write(*,*) 'integration average ',sigen(0,1),sigen(0,2),sigen(0,3)

      end
***********************************************************************
      subroutine integrand(random,ln,channel,generator,switch)
***********************************************************************
*     phase space integrand                                           *
*     adapted from COFFERGG by Markus Roth                            *
*                                                                     *
*     LOOPS OVER EMITTER AND SPECTATOR ASSUME UNCHARGED HIGGS!!!!     *
*                                                                     *
*---------------------------------------------------------------------*
*     21.02.06  Ansgar Denner     last changed  24.04.07              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

#ifdef multpdf
#include "multpdf.h"
      integer nset
#endif
c local variables
      real*8 random(3*maxe),rand1,rand2
      real*8 k(maxe,0:3),kbeam(2,0:3),xconv(2)
      real*8 fac(maxps),g(maxch,maxps),gsum(maxps)
c      real*8 wconv
      real*8 woaisr(maxps),woafsr,pf,f,p2,logs,krec(maxe,0:3),fp,fg
      real*8 sk(maxe,maxe),s(maxe,maxe,maxps),y,z,x,v,omz
      real*8 ck(0:3),ckt(0:3),omx,mapx,jacobianx,sfx(2),sfz,weighttot
      real*8 xd,omxd,sfxd(2),woaisrd(maxps),fpd,fgd
      real*8 xisr(maxps),omxisr(maxps),xdisr(maxps),omxdisr(maxps)
      real*8 fac1,fac2,p2b,lam1,lam2,pij,mi2,mj2,kk,kvec
      real*8 s12max,s12min,x1,x2,s12
c      real*8 pdf1,pdf2,pdf_phot1,pdf_phot2
c      real*8 qcdfacscale,qcdfacscale2,qedfacscale,qedfacscale2,
      real*8 tau0,tau,par,g1
      real*8 wfac,wfacg,wfacp
      real*8 dir(maxe)
      real*8 kcms(maxe,0:3),q(0:3),q1(0:3),ecms(maxe,maxps)
      real*8 dli2
      real*8 spp,shat,xminpl
      real*8 omz2,z2,aux,collfact,spk,collfact1
      integer generator,channel,switch,ln,cut(maxps),j
      integer phasespace,em,sp,co,cutall,last
      integer i1,i2,psflag

      data dir /-1d0,-1d0,maxf*1d0/
      data par /1d0/

c      real*8 test

      real*8 m2iendp(-4:4,-4:4)
      real*8 m2ifendp(-4:4,-4:4,-4:4,-4:4)

      real*8 constff,constfi,constif,constii
      common/constsub/constff,constfi,constif,constii

      integer iNumProcs,iMyRank
      common/mpistuff/iNumProcs,iMyRank

c      real*8 m2i(-4:4,-4:4)
c      real*8 m2if(-4:4,-4:4,-4:4,-4:4)
c      character*5 part(-4:4)


c use of random numbers
c random2 choice of proton direction
c random3 optimization of x1
c random4 optimization of x2
c random5 x1  parton energy fraction
c random6 x2  parton energy fraction
c random7 z   energy fraction after collinear photon emission
c (random8 (1-z) for exclusive final states)
c random9- (3*maxe)  (3*maxe-8) phase space variable for maxe-2
c                    final state particles [(3maxe - 10) needed]

c       write(*,*) 'integrand: generator = ',generator
c       write(*,*) 'integrand: random = ',random

 1050           format(1x,a22,": phasespace,weight = ",3i4,g24.10
     &              /4g24.10)
 1051           format(1x,a22,":            weight = ",4x,g24.10
     &              /3g24.10)

c initialize

      do iphsp=1,nphsp(generator)
c        m2pdfpdf_0(iphsp)=0d0
c        m2pdfpdf_vw(iphsp)=0d0
c        m2pdfpdf_vs(iphsp)=0d0

#ifdef SUBPROCESSES
c        do parton1 = -4,4
c        do parton2 = -4,4
c          m2pdfpdfii_0(iphsp,parton1,parton2)=0d0
c          m2pdfpdfii_vw(iphsp,parton1,parton2)=0d0
c          m2pdfpdfii_vs(iphsp,parton1,parton2)=0d0
c          do ext=1,nexternal(generator)
c          do ext1=1,nexternal(generator)
c            m2pdfpdfiiqq_0(iphsp,parton1,parton2,ext,ext1)=0d0
c            m2pdfpdfiitt_0(iphsp,parton1,parton2,ext,ext1)=0d0
c          enddo
c          enddo
c        enddo
c        enddo
#endif

        do ext=1,nextme(generator)
        do ext1=1,nextme(generator)
          m2pdfpdfqq_0(iphsp,ext,ext1)=0d0
          m2pdfpdftt_0(iphsp,ext,ext1)=0d0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
	     iref1=(nset-1)*maxps+iphsp
        mm2pdfpdfqq_0(iref1,ext,ext1)=0d0
        mm2pdfpdftt_0(iref1,ext,ext1)=0d0
      enddo
#endif
        enddo
        enddo
        do icon=1,ncontr(generator)
          weight(icon,iphsp)=0d0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
          mweight(nset,icon,iphsp)=0d0
      enddo
#endif
          rejectedcon(icon,iphsp)=0

#ifdef SUBPROCESSES
          do parton1 = -4,4
          do parton2 = -4,4
              weightii(icon,parton1,parton2,iphsp)=0d0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
              mweightii(nset,icon,parton1,parton2,iphsp)=0d0
      enddo
#endif
            enddo
          enddo
#endif

        enddo
        gsum(iphsp)=0d0
c added AD 18.10.06
        cut(iphsp)=0
        do ext=1,maxe
        do vcomp=0,3
          pevent(ext,vcomp,iphsp)=0d0
        enddo
        enddo
      enddo

      weight_0=0d0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
      mweight_0(nset)=0d0
      enddo
#endif
#ifdef SUBPROCESSES
      do parton1 = -4,4
      do parton2 = -4,4
        weightii_0(parton1,parton2)=0d0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        mweightii_0(nset,parton1,parton2)=0d0
      enddo
#endif
      enddo
      enddo
#endif
      do ext=1,maxe
      do vcomp=0,3
        pborn(ext,vcomp)=0d0
      enddo
      enddo
      do icon=1,ncontr(generator)
        rejected(icon)=0
      enddo
c end initialize

c      write(*,*) 'integration endini',weight(1,1),nphsp(generator),
c     &    ncontr(generator)

 1200 format(a30,5e24.16)
 1202 format(a30,2i3,5e24.16)
 1203 format(a30,3i3,5e24.16)


c      write(*,*) 'generator: switch = ',switch,sconv(generator)
      if(switch.gt.0)return

c convolution with parton distributions
      if(sconv(generator).eq.1)then
        wconv=1d0
c vegas-like mapping for x1 and x2

c        write(*,*) 'integrand generator    ',generator
c        write(*,*) 'integrand use random(2)',random(2)
c        write(*,*) 'integrand use random(3)',random(3)
c        write(*,*) 'integrand use random(4)',random(4)
c        write(*,*) 'integrand use random(5)',random(5)
c        write(*,*) 'integrand use random(6)',random(6)

c        rand1=random(5)
c        rand2=random(6)
        call ranopt(random(3),random(5),rand1,wconv,ln,1,generator,
     &      switch)
        call ranopt(random(4),random(6),rand2,wconv,ln,2,generator,
     &      switch)
c        write(*,*) 'integrand: random   ',(random(i1),i1=2,6)

c        write(*,*) 'integrand redef rand1',rand1
c        write(*,*) 'integrand redef rand2',rand2

c        pdfmap=3
c        pdfmap=1
        if (pdfmap.eq.0) then
c     flat mapping for x1 and tau
          s12max=energy**2
          s12min=xmin(generator)*energy**2
          s12=s12min+(s12max-s12min)*rand2
          wconv=wconv*(s12max-s12min)

          if(random(2).le.0.5d0)then
            x1=s12/s12max+(1d0-s12/s12max)*rand1
            x2=s12/s12max/x1
            wconv=wconv/s12max/x1*(1d0-s12/s12max)
          else
            x2=s12/s12max+(1d0-s12/s12max)*rand1
            x1=s12/s12max/x2
            wconv=wconv/s12max/x2*(1d0-s12/s12max)
          endif

        else if (pdfmap.eq.1) then
c    flat mapping for x1 and tau
          tau0  = xmin(generator)
          tau   = tau0+rand2*(1d0-tau0)
          wconv = wconv*(1d0-tau0)
          if(random(2).le.0.5d0)then
            x1=tau+(1d0-tau)*rand1
            x2=tau/x1
            wconv=wconv/x1*(1d0-tau)
          else
            x2=tau+(1d0-tau)*rand1
            x1=tau/x2
            wconv=wconv/x2*(1d0-tau)
          endif

        else if (pdfmap.eq.2) then
c    preferred  mapping for x1 and tau of std
c    maps x1/tau
         tau0  = xmin(generator)
          tau   = tau0**rand2
          if(random(2).le.0.5d0)then
            x1=tau+(1d0-tau)*rand1
            x2=tau/x1
            wconv = -(wconv * tau*log(tau0)*(1d0-tau)/x1)
          else
            x2=tau+(1d0-tau)*rand1
            x1=tau/x2
            wconv = -(wconv * tau*log(tau0)*(1d0-tau)/x2)
          endif

        else if (pdfmap.eq.3) then
c    alternative mapping for x1 and tau of std
c    maps 1/tau and 1/x1
          tau0  = xmin(generator)
          if(shtr.gt.0)then
             tau0 =  max(xmin(generator)+ (mcm(25)*mcm(25)
     &          + mcm(25)*mcw(25)*tan(random(1)-.5d0))/energy**2,
     &          xmin(generator),1d-5)


          endif
          tau   = tau0**rand2
          if(random(2).le.0.5d0)then
            x1   = tau**rand1
            x2=tau/x1
            wconv = wconv * tau*log(tau0)*log(tau)
          else
            x2   = tau**rand1
            x1=tau/x2
            wconv = wconv * tau*log(tau0)*log(tau)
          endif

        else if (pdfmap.eq.4) then
c    mapping for x1 and tau for off-shell Higgs
          tau0  = xmin(generator)
c          tau   = tau0**rand2

          if(shtr.eq.0)then
            tau   = tau0**rand2
            if(random(2).le.0.5d0)then
              x1   = tau**rand1
              x2=tau/x1
              wconv = wconv * tau*log(tau0)*log(tau)
            else
              x2   = tau**rand1
              x1=tau/x2
              wconv = wconv * tau*log(tau0)*log(tau)
            endif
          elseif(mcw(25).lt.1d-6*mcm(25))then
            tau0 = max(tau0,mcm(25)*(mcm(25)-200*mcw(25))/energy**2)
            tau   = tau0**rand2
            if(random(2).le.0.5d0)then
              x1   = tau**rand1
              x2=tau/x1
              wconv = wconv * tau*log(tau0)*log(tau)
            else
              x2   = tau**rand1
              x1=tau/x2
              wconv = wconv * tau*log(tau0)*log(tau)
            endif
          else
            if(tau0.gt.mcm(25)*mcm(25)-par*mcm(25)*mcw(25))then
              tau=tau0**(1d0-rand2)
              wconv = -wconv * tau * log(tau0)
            else
              g1=mcm(25)*mcw(25)/energy**2*
     &            log((mcm(25)*mcm(25)-tau0*energy**2)/
     &            (par*mcm(25)*mcw(25)))
     &            + (mcm(25)*mcm(25)-par*mcm(25)*mcw(25))
     &            /(par*energy**2)*
     &            log(energy**2/(mcm(25)*mcm(25)-par*mcm(25)*mcw(25)))

              if(g1*rand2.gt.mcm(25)*mcw(25)/energy**2*
     &            log((mcm(25)*mcm(25)-tau0*energy**2)/
     &            (par*mcm(25)*mcw(25))))
     &            then
                tau = (mcm(25)*mcm(25)-par*mcm(25)*mcw(25))/(energy**2)
     &              *exp((g1*rand2-mcm(25)*mcw(25)/energy**2*
     &              log((mcm(25)*mcm(25)-tau0*energy**2)
     &              /(par*mcm(25)*mcw(25))))
     &              *par*energy**2
     &              /(mcm(25)*mcm(25)-par*mcm(25)*mcw(25)))
                wconv = wconv * g1 * par * tau * energy**2/
     &              (mcm(25)*mcm(25)-par*mcm(25)*mcw(25))
              else
                tau=1d0/(energy**2)*(mcm(25)*mcm(25)-
     &              (mcm(25)*mcm(25)-tau0*energy**2)
     &              *exp(-energy**2*g1*rand2/(mcm(25)*mcw(25))))
                wconv = wconv * g1 /( mcm(25)*mcw(25)) *
     &              (mcm(25)*mcm(25)-tau*energy**2)

              endif
            endif
            if(random(2).le.0.5d0)then
              x1   = tau**rand1
              x2=tau/x1
              wconv = -wconv * log(tau)
            else
              x2   = tau**rand1
              x1=tau/x2
              wconv = -wconv * log(tau)
            endif
          endif
        endif

c  adjustment of flux factor included in wconv
        wconv = wconv/tau

c        write(*,*) 'integrand: ran ',rand2,rand1
c        write(*,*) 'integrand: s12 ',s12,s12max,s12min
c        write(*,*) 'integrand: x   ',x1,x2,wconv

c  fixing of factorization scales
        call getqcdfacscale(mcm(24))
        call getqedfacscale(mcm(24))



#ifdef multpdf
        call wpdf(x1,qcdfacscale,pdf1,mpdf1,pdf_phot1,spdf)
        call wpdf(x2,qcdfacscale,pdf2,mpdf2,pdf_phot2,spdf)
#else
        call wpdf(x1,qcdfacscale,pdf1,pdf_phot1,spdf)
        call wpdf(x2,qcdfacscale,pdf2,pdf_phot2,spdf)
#endif
        xconv(1)=x1
        xconv(2)=x2


      else
        do i1=-4,4
          pdf1(i1) = 1d0
          pdf2(i1) = 1d0
        enddo
#ifdef multpdf
        do nset=1,pdfmemberto-pdfmemberfrom+1
          do i1=-4,4
            mpdf1(nset,i1) = 1d0
            mpdf2(nset,i1) = 1d0
          enddo
        enddo
#endif
        pdf_phot1 = 1d0
        pdf_phot2 = 1d0
        xconv(1)=1d0
        xconv(2)=1d0

        wconv=1d0
        call getqedfacscale(mcm(24))
        call getqcdfacscale(mcm(24))
      endif

c      write(*,*) 'integrand: conv',xconv,wconv
c       write(*,*) 'integrand xmin= ',xmin(generator),xconv(1)*xconv(2)

      xminp=xmin(generator)/(xconv(1)*xconv(2))
      xminpl=xminp
      if(ssub(0).gt.0) then
c finite xminpl not yet implemented for subtraction
        xminpl=0
      endif
c      xminpl=0d0
c      if(xmin(generator).ge.xconv(1)*xconv(2))return
c      write(*,*) 'montecarlo 1 ',xminp
      if(xminp.ge.1d0.or.xmin(generator).ge.xconv(1)*xconv(2))then
        cut(1)=1
        weight_0=0d0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        mweight_0(nset)=0d0
      enddo
#endif
        do iphsp=1,nphsp(generator)
          weight(generator,iphsp)=0d0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
          mweight(nset,generator,iphsp)=0d0
      enddo
#endif
        enddo

        write(*,*) 'integrand cut1 ',xminp,xmin(generator),
     &      xconv(1)*xconv(2),1d0-xminp,
     &      xconv(1)*xconv(2)-xmin(generator)

        return
      endif
c      write(*,*) 'montecarlo 2 ',xminp
c      write(*,*) 'integrand: after pdf, '

c end convolution with parton distributions

#ifdef PRINTWEIGHTMAX
         x1print=xconv(1)
         x2print=xconv(2)
#endif

c      write(*,*) 'integrand befcon',generator,sgen(generator)
c        write(*,*) 'integrand befcon',generator,sgen(generator),
c     &      freqgencon(generator,1),mod(nevt,freqgencon(generator,1))
c     &    ,nevt
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     tree-level process, virtual corrections, and                c
c     corresponding subtraction terms                             c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(generator.eq.0.and.sgen(generator).ne.0)then
        if(ssub(generator).ne.0.or.ssli(generator).ne.0.or.
     &      seff(generator).ne.0.or.soaisr(generator).ne.0)then
          if(sexcl.eq.0)then
            nphsp(generator)=5
c           x = 1, x1, x2
          else
            nphsp(generator)=3*(nextgen(generator)-1)
          endif
        else
          nphsp(generator)=1
        endif
        ncontr(generator)=9

c local switches
c switch for lowest-order contributions
        lwborn(0)=swborn(0).eq.1.and.
     &      mod(nevt,iNumProcs*freqgencon(generator,1)).lt.iNumProcs
c switch for lowest-order contributions
        lgencon(0,1)=sgencon(0,1).eq.1.and.
     &      mod(nevt,iNumProcs*freqgencon(generator,1)).lt.iNumProcs
c switch for electroweak virtual corrections
        lgencon(0,2)=sgencon(0,2).eq.1.and.
     &      mod(nevt,iNumProcs*freqgencon(generator,2)).lt.iNumProcs
c switch for qcd virtual corrections
        lgencon(0,5)=sgencon(0,5).eq.1.and.
     &      mod(nevt,iNumProcs*freqgencon(generator,5)).lt.iNumProcs
c switch for photon convolution contributions
        lgencon(0,3)=sgencon(0,3).eq.1.and.
     &      mod(nevt,iNumProcs*freqgencon(generator,3)).lt.iNumProcs
c switch for gluon convolution contributions
        lgencon(0,4)=sgencon(0,4).eq.1.and.
     &        mod(nevt,iNumProcs*freqgencon(generator,4)).lt.iNumProcs
c switch for incoming photon convolution contributions
        lgencon(0,6)=sgencon(0,6).eq.1.and.
     &      mod(nevt,iNumProcs*freqgencon(generator,6)).lt.iNumProcs
        lgencon(0,7)=sgencon(0,7).eq.1.and.
     &      mod(nevt,iNumProcs*freqgencon(generator,7)).lt.iNumProcs
c switch for incoming gluon convolution contributions
        lgencon(0,8)=sgencon(0,8).eq.1.and.
     &      mod(nevt,iNumProcs*freqgencon(generator,8)).lt.iNumProcs
        lgencon(0,9)=sgencon(0,9).eq.1.and.
     &      mod(nevt,iNumProcs*freqgencon(generator,9)).lt.iNumProcs

c        write(*,*) 'integrand sgencon ',sgencon(0,3),lgencon(0,3)

        psflag=0


c generation of momenta and calculation of jacobians and crosssection
        do phasespace=1,nphsp(generator)
        if(phasespace.eq.1.and.(lwborn(0).or.lgencon(0,1)
     &        .or.lgencon(0,2).or.lgencon(0,5))
     &        .or.
     &        (phasespace.eq.2.or.phasespace.eq.3).and.
     &        (lgencon(0,3).or.lgencon(0,4))
     &        .or.
     &        (phasespace.eq.4.or.phasespace.eq.5).and.
     &        (lgencon(0,6).or.lgencon(0,7).or.
     &         lgencon(0,8).or.lgencon(0,9))
     &        )then


c        write(*,*) 'integrand psloop ',lgencon(0,3),phasespace


          do vcomp=0,3
            kbeam(1,vcomp)=xconv(1)*pbeam(1,vcomp)
            kbeam(2,vcomp)=xconv(2)*pbeam(2,vcomp)
          enddo
c          write(nout,*) ' integrand: xc  =',(xconv(vcomp),vcomp=1,2)
c          write(nout,*) ' integrand: pb1 =',(pbeam(1,vcomp),vcomp=0,3)
c          write(nout,*) ' integrand: pb2 =',(pbeam(2,vcomp),vcomp=0,3)
c          write(nout,*) ' integrand: kb1 =',(kbeam(1,vcomp),vcomp=0,3)
c          write(nout,*) ' integrand: kb2 =',(kbeam(2,vcomp),vcomp=0,3)

* part without collinear photon
          if(phasespace.eq.1)then
            em=0
            x=1d0
c            woaisr(phasespace)=1d0
            spp=xconv(1)*xconv(2)*energy*energy
* parts with collinear photons with energy fraction 1-x
          elseif(phasespace.eq.2.or.phasespace.eq.3) then
* maps 1/(1-x)^powersplitf     for incoming fermions
c CHANGED 16.11.06 AD
c            omx=mpinv(random(7),powersplitf,0d0,0d0,1d0,switch)
            em=phasespace-1
            if(1d0-xminpl.gt.omxmaxf)then

c              write(*,*) 'integrand use random(7)',random(7)

              omx=mapx(random(7),powersplitf,0d0,omxmaxf,1d0-xminpl,
     &            switch)
c           write(*,*) 'integrand x= ',omx,random(7),powersplitf
c     &           ,omxmaxf,1d0-xminpl
            else
              cut(phasespace)=1
            endif
            omxd=mapx(random(7),powersplitf,0d0,omxmaxf,1d0,switch)

            if(omx.eq.0d0.or.omxd.eq.0)then
              switch=27
              if(numout.lt.maxout.and.lnoutmc.ge.5)then
 100            format(/" event = ",i9,", gen = ",i4,", channel = ",i4,
     &              ", exit = ",i4,(a))
                write(nout,100) nevt,cogen,cocha,switch,
     &              ', integrand: x = 1'
                numout=numout+1
              endif
              return
            endif
c     CHANGED 16.11.06 AD
c     woaisr(phasespace)=jacobianinv(powersplitf,0d0,omx,0d0,
c     &          1d0,switch)
            if(cut(phasespace).eq.0)then
              woaisr(phasespace)=jacobianx(powersplitf,0d0,omx,omxmaxf,
     &            1d0-xminpl,switch)
c                write(*,*) 'integrand j= ',woaisr(phasespace)
              x=1d0-omx
              sfx(em)=(1d0+x*x)/omx
            endif
            do vcomp=0,3
              kbeam(em,vcomp)=x*kbeam(em,vcomp)
            enddo
            woaisrd(phasespace)=jacobianx(powersplitf,0d0,omxd,omxmaxf,
     &          1d0,switch)
            xd=1d0-omxd
            sfxd(em)=(1d0+xd*xd)/omxd


          elseif(phasespace.eq.4.or.phasespace.eq.5) then
* maps 1/(1-x)^powersplitp   for incoming gauge bosons
c CHANGED 16.11.06 AD
c            omx=mpinv(random(7),powersplitp,0d0,0d0,1d0,switch)
              em=phasespace-3
              if(1d0-xminpl.gt.omxmaxp)then
                omx=mapx(random(7),powersplitp,0d0,omxmaxp,1d0-xminpl,
     &              switch)
c                write(*,*) 'integrand x= ',omx,random(7),powersplitp
c     &           ,omxmaxp,1d0-xminpl
              else
                cut(phasespace)=1
              endif

            if(omx.eq.0d0)then
              switch=27
              if(numout.lt.maxout.and.lnoutmc.ge.5)then
                write(nout,100) nevt,cogen,cocha,switch,
c                write(*,'(" event= ",i9,",",(a))'),nevt,
     &              ' integrand: x = 1'
                numout=numout+1
              endif
c             mcexitevt(1,generator)=mcexitevt(1,generator)+1
              return
            endif
c     CHANGED 16.11.06 AD
c     woaisr(phasespace)=jacobianinv(powersplitp,0d0,omx,0d0,
c     &          1d0,switch)
            if(cut(phasespace).eq.0)then
              woaisr(phasespace)=jacobianx(powersplitp,0d0,omx,omxmaxp,
     &            1d0-xminpl,switch)
c                write(*,*) 'integrand j= ',woaisr(phasespace)
              x=1d0-omx
            endif
            do vcomp=0,3
              kbeam(em,vcomp)=x*kbeam(em,vcomp)
            enddo


          else
            write(*,*) 'integrand: phasespace not defined:',phasespace
            stop
          endif

#ifdef PRINTWEIGHTMAX
       xprint=x
       omxprint=omx
#endif


c          write(*,*) 'integrand afterxgen',phasespace,x,
c     &      x*xconv(1)*xconv(2),xmin(generator),cut(phasespace)

          if(x*xconv(1)*xconv(2).gt.xmin(generator)
     &        .and.cut(phasespace).eq.0
     &        )then
c          if(x.gt.xminp)then

            shat=x*xconv(1)*xconv(2)*energy*energy

            call generate_momenta(random(9),kbeam,k,kcms,
     &          g(1,phasespace),channel,generator,switch)


c            call generation(random(9),kbeam,k,g(1,phasespace),
c     &          channel,generator,switch)
c phase space point for testing
            if (em.eq.0)then
            endif

#ifdef PRINTWEIGHTMAX
            if(lprintweightmax)then
              do vcomp=0,3
                do ext=1,nextgen(generator)
                  pprint(ext,vcomp)=k(ext,vcomp)
                enddo
              enddo
            endif
#endif

c        write(*,*) ' integrand: k =',
c     &      ((k(ext,vcomp),ext=1,nextgen(generator)),vcomp=0,3)

c            write(*,*) 'integrand: g switch= ',switch,
c     &          channel,generator
            do ext=1,nextgen(generator)
               do vcomp=0,3
                  pevent(ext,vcomp,phasespace)=k(ext,vcomp)
                  if(phasespace.eq.1.and.swborn(generator).eq.1)then
                     pborn(ext,vcomp)=k(ext,vcomp)
                  endif
               enddo
            enddo
c            call applycut(pbeam,k,generator,nextgen(generator),
c     &        cut(phasespace),switch)

            call recombination(pbeam,k,krec,generator,
     &          nextgen(generator),cut(phasespace),switch)

c          write(*,*) 'integrand args ',
c     &      generator,nextgen(generator),cut(phasespace),switch
c            write(*,*) 'integrand: c switch= ',
c     &          switch,sexcl,cut(phasespace)

            cutall=cut(phasespace)

c "exclusive" subtraction terms
            if(sexcl.eq.1)then
              write(*,*) 'integrand: sexcl = 1 never used'
              write(*,*) 'integrand: check implementation!'
c not implemented, old code modified, change 'phasespaces'!!!!
              stop
              omz=mapx(random(8),powersplitf,0d0,0d0,1d0,switch)
              z=1d0-omz
              sfz=(1d0+z*z)/omz
              woafsr=jacobianx(powersplitf,0d0,omz,0d0,1d0,switch)
              do ext=3,nextgen(generator)
c                pf=qedfactor(ext,ext,generator)
c     &            +qcdfactor(ext,ext,generator)
                pf= mcalpha*factor(generator)
                if(pf.ne.0d0)then
c CHECK
c                  phasespace=3+em*(nextgen(generator)-2)+ext-2
                  em= (phasespace-3+2-ext)/(nextgen(generator)-2)
                  do ext2=1,nextgen(generator)
                  do vcomp=0,3
                    pevent(ext2,vcomp,phasespace)=k(ext2,vcomp)
                  enddo
                  enddo
                  do vcomp=0,3
                    pevent(ext,vcomp,phasespace)=z*k(ext,vcomp)
                    pevent(nextgen(generator)+1,vcomp,phasespace)=
     &                  (1d0-z)*k(ext,vcomp)
                  enddo
                  call recombination(pbeam,pevent(1,0,phasespace),
     &              krec,generator,nextgen(generator)+1,
     &              cut(phasespace),switch)
c check cutall
                  cutall=min(cutall,cut(phasespace))
                  do ext2=1,nextgen(generator)+1
                  do vcomp=0,3
                    if(cut(phasespace).eq.0)then
                      pevent(ext2,vcomp,phasespace)=krec(ext2,vcomp)
                    else
                      pevent(ext2,vcomp,phasespace)=0d0
                    endif
                  enddo
                  enddo
                endif
              enddo
            endif

c            write(*,*) 'integrand: d cutall= ',cutall,switch

            if(cutall.eq.0.and.switch.eq.0)then
              call density(k,g(1,phasespace),generator,switch)
              if(switch.eq.0)then
                do cha=1,nchannel(generator)
c ADDED for testing AD 6.04.06
                  gcha(cha,phasespace)=g(cha,phasespace)
                  gsum(phasespace)=gsum(phasespace)+g(cha,phasespace)
     &              *alphaopt(cha,opt(generator),generator)
                enddo
c ADDED for testing AD 6.04.06
                gsumm(phasespace)=gsum(phasespace)
                if(gsum(phasespace).eq.0d0)then
                  switch=28
                  if(numout.lt.maxout.and.lnoutmc.ge.5)then
                    write(nout,100) nevt,cogen,cocha,switch,
c                    write(*,'(" event= ",i9,",",(a))'),nevt,
     &                  ' integrand: gsum = 0'
                    numout=numout+1
                  endif
c                  mcexitevt(2,generator)=mcexitevt(2,generator)+1
                  return
                endif


#ifndef SYMEVENTS
                call crosssection(k,xconv(1),xconv(2),
     &              generator,phasespace,0,0,switch)


#endif
#ifdef SYMEVENTS
c SYMEVENTS not correct for distributions!
                if(phasespace.le.2.or.phasespace.eq.4)then
                  call crosssection(k,xconv(1),xconv(2),
     &                generator,phasespace,0,0,switch)
                  psflag=phasespace
                elseif (phasespace.eq.3) then
                  if(psflag.eq.2) then
                    if(lgencon(0,3))then
                      m2pdfpdfqq_0(3,2,1)=m2pdfpdfqq_0(2,2,1)
                      m2pdfpdfqq_0(3,3,1)=m2pdfpdfqq_0(2,3,1)
                      m2pdfpdfqq_0(3,4,1)=m2pdfpdfqq_0(2,4,1)
                      m2pdfpdfqq_0(3,3,2)=m2pdfpdfqq_0(2,3,2)
                      m2pdfpdfqq_0(3,4,2)=m2pdfpdfqq_0(2,4,2)
                      m2pdfpdfqq_0(3,4,3)=m2pdfpdfqq_0(2,4,3)
                      m2pdfpdfqq_0(3,1,2)=m2pdfpdfqq_0(2,1,2)
                      m2pdfpdfqq_0(3,1,3)=m2pdfpdfqq_0(2,1,3)
                      m2pdfpdfqq_0(3,1,4)=m2pdfpdfqq_0(2,1,4)
                      m2pdfpdfqq_0(3,2,3)=m2pdfpdfqq_0(2,2,3)
                      m2pdfpdfqq_0(3,2,4)=m2pdfpdfqq_0(2,2,4)
                      m2pdfpdfqq_0(3,3,4)=m2pdfpdfqq_0(2,3,4)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
	     iref1=(nset-1)*maxps+3
	     iref2=(nset-1)*maxps+2
        mm2pdfpdfqq_0(iref1,2,1)=mm2pdfpdfqq_0(iref2,2,1)
        mm2pdfpdfqq_0(iref1,3,1)=mm2pdfpdfqq_0(iref2,3,1)
        mm2pdfpdfqq_0(iref1,4,1)=mm2pdfpdfqq_0(iref2,4,1)
        mm2pdfpdfqq_0(iref1,3,2)=mm2pdfpdfqq_0(iref2,3,2)
        mm2pdfpdfqq_0(iref1,4,2)=mm2pdfpdfqq_0(iref2,4,2)
        mm2pdfpdfqq_0(iref1,4,3)=mm2pdfpdfqq_0(iref2,4,3)
        mm2pdfpdfqq_0(iref1,1,2)=mm2pdfpdfqq_0(iref2,1,2)
        mm2pdfpdfqq_0(iref1,1,3)=mm2pdfpdfqq_0(iref2,1,3)
        mm2pdfpdfqq_0(iref1,1,4)=mm2pdfpdfqq_0(iref2,1,4)
        mm2pdfpdfqq_0(iref1,2,3)=mm2pdfpdfqq_0(iref2,2,3)
        mm2pdfpdfqq_0(iref1,2,4)=mm2pdfpdfqq_0(iref2,2,4)
        mm2pdfpdfqq_0(iref1,3,4)=mm2pdfpdfqq_0(iref2,3,4)
      enddo
#endif
                      do i1=-4,4
                        do i2=-4,4
                          if(i1*i2.ne.0)then
                            m2pdfpdfiiqq_0(3,i1,i2,1,2)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,1,2)
                            m2pdfpdfiiqq_0(3,i1,i2,1,3)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,1,3)
                            m2pdfpdfiiqq_0(3,i1,i2,1,4)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,1,4)
                            m2pdfpdfiiqq_0(3,i1,i2,2,3)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,2,3)
                            m2pdfpdfiiqq_0(3,i1,i2,2,4)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,2,4)
                            m2pdfpdfiiqq_0(3,i1,i2,3,4)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,3,4)
                            m2pdfpdfiiqq_0(3,i1,i2,2,1)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,2,1)
                            m2pdfpdfiiqq_0(3,i1,i2,3,1)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,3,1)
                            m2pdfpdfiiqq_0(3,i1,i2,4,1)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,4,1)
                            m2pdfpdfiiqq_0(3,i1,i2,3,2)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,3,2)
                            m2pdfpdfiiqq_0(3,i1,i2,4,2)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,4,2)
                            m2pdfpdfiiqq_0(3,i1,i2,4,3)
     &                          =m2pdfpdfiiqq_0(2,i1,i2,4,3)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
	     iref1=(nset-1)*maxps+3
	     iref2=(nset-1)*maxps+2
        mm2pdfpdfiiqq_0(iref1,i1,i2,1,2)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,1,2)
        mm2pdfpdfiiqq_0(iref1,i1,i2,1,3)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,1,3)
        mm2pdfpdfiiqq_0(iref1,i1,i2,1,4)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,1,4)
        mm2pdfpdfiiqq_0(iref1,i1,i2,2,3)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,2,3)
        mm2pdfpdfiiqq_0(iref1,i1,i2,2,4)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,2,4)
        mm2pdfpdfiiqq_0(iref1,i1,i2,3,4)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,3,4)
        mm2pdfpdfiiqq_0(iref1,i1,i2,2,1)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,2,1)
        mm2pdfpdfiiqq_0(iref1,i1,i2,3,1)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,3,1)
        mm2pdfpdfiiqq_0(iref1,i1,i2,4,1)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,4,1)
        mm2pdfpdfiiqq_0(iref1,i1,i2,3,2)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,3,2)
        mm2pdfpdfiiqq_0(iref1,i1,i2,4,2)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,4,2)
        mm2pdfpdfiiqq_0(iref1,i1,i2,4,3)
     &  =mm2pdfpdfiiqq_0(iref2,i1,i2,4,3)
      enddo
#endif
                          endif
                        enddo
                      enddo
                    endif
                    if(lgencon(0,4))then
                      m2pdfpdftt_0(3,2,1)=m2pdfpdftt_0(2,2,1)
                      m2pdfpdftt_0(3,3,1)=m2pdfpdftt_0(2,3,1)
                      m2pdfpdftt_0(3,4,1)=m2pdfpdftt_0(2,4,1)
                      m2pdfpdftt_0(3,3,2)=m2pdfpdftt_0(2,3,2)
                      m2pdfpdftt_0(3,4,2)=m2pdfpdftt_0(2,4,2)
                      m2pdfpdftt_0(3,4,3)=m2pdfpdftt_0(2,4,3)
                      m2pdfpdftt_0(3,1,2)=m2pdfpdftt_0(2,1,2)
                      m2pdfpdftt_0(3,1,3)=m2pdfpdftt_0(2,1,3)
                      m2pdfpdftt_0(3,1,4)=m2pdfpdftt_0(2,1,4)
                      m2pdfpdftt_0(3,2,3)=m2pdfpdftt_0(2,2,3)
                      m2pdfpdftt_0(3,2,4)=m2pdfpdftt_0(2,2,4)
                      m2pdfpdftt_0(3,3,4)=m2pdfpdftt_0(2,3,4)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
	     iref1=(nset-1)*maxps+3
	     iref2=(nset-1)*maxps+2
        mm2pdfpdftt_0(iref1,2,1)=mm2pdfpdftt_0(iref2,2,1)
        mm2pdfpdftt_0(iref1,3,1)=mm2pdfpdftt_0(iref2,3,1)
        mm2pdfpdftt_0(iref1,4,1)=mm2pdfpdftt_0(iref2,4,1)
        mm2pdfpdftt_0(iref1,3,2)=mm2pdfpdftt_0(iref2,3,2)
        mm2pdfpdftt_0(iref1,4,2)=mm2pdfpdftt_0(iref2,4,2)
        mm2pdfpdftt_0(iref1,4,3)=mm2pdfpdftt_0(iref2,4,3)
        mm2pdfpdftt_0(iref1,1,2)=mm2pdfpdftt_0(iref2,1,2)
        mm2pdfpdftt_0(iref1,1,3)=mm2pdfpdftt_0(iref2,1,3)
        mm2pdfpdftt_0(iref1,1,4)=mm2pdfpdftt_0(iref2,1,4)
        mm2pdfpdftt_0(iref1,2,3)=mm2pdfpdftt_0(iref2,2,3)
        mm2pdfpdftt_0(iref1,2,4)=mm2pdfpdftt_0(iref2,2,4)
        mm2pdfpdftt_0(iref1,3,4)=mm2pdfpdftt_0(iref2,3,4)
      enddo
#endif
                      do i1=-4,4
                        do i2=-4,4
                          if(i1*i2.ne.0)then
                            m2pdfpdfiitt_0(3,i1,i2,1,2)
     &                          =m2pdfpdfiitt_0(2,i1,i2,1,2)
                            m2pdfpdfiitt_0(3,i1,i2,1,3)
     &                          =m2pdfpdfiitt_0(2,i1,i2,1,3)
                            m2pdfpdfiitt_0(3,i1,i2,1,4)
     &                          =m2pdfpdfiitt_0(2,i1,i2,1,4)
                            m2pdfpdfiitt_0(3,i1,i2,2,3)
     &                          =m2pdfpdfiitt_0(2,i1,i2,2,3)
                            m2pdfpdfiitt_0(3,i1,i2,2,4)
     &                          =m2pdfpdfiitt_0(2,i1,i2,2,4)
                            m2pdfpdfiitt_0(3,i1,i2,3,4)
     &                          =m2pdfpdfiitt_0(2,i1,i2,3,4)
                            m2pdfpdfiitt_0(3,i1,i2,2,1)
     &                          =m2pdfpdfiitt_0(2,i1,i2,2,1)
                            m2pdfpdfiitt_0(3,i1,i2,3,1)
     &                          =m2pdfpdfiitt_0(2,i1,i2,3,1)
                            m2pdfpdfiitt_0(3,i1,i2,4,1)
     &                          =m2pdfpdfiitt_0(2,i1,i2,4,1)
                            m2pdfpdfiitt_0(3,i1,i2,3,2)
     &                          =m2pdfpdfiitt_0(2,i1,i2,3,2)
                            m2pdfpdfiitt_0(3,i1,i2,4,2)
     &                          =m2pdfpdfiitt_0(2,i1,i2,4,2)
                            m2pdfpdfiitt_0(3,i1,i2,4,3)
     &                          =m2pdfpdfiitt_0(2,i1,i2,4,3)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
	     iref1=(nset-1)*maxps+3
	     iref2=(nset-1)*maxps+2
        mm2pdfpdfiitt_0(iref1,i1,i2,1,2)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,1,2)
        mm2pdfpdfiitt_0(iref1,i1,i2,1,3)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,1,3)
        mm2pdfpdfiitt_0(iref1,i1,i2,1,4)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,1,4)
        mm2pdfpdfiitt_0(iref1,i1,i2,2,3)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,2,3)
        mm2pdfpdfiitt_0(iref1,i1,i2,2,4)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,2,4)
        mm2pdfpdfiitt_0(iref1,i1,i2,3,4)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,3,4)
        mm2pdfpdfiitt_0(iref1,i1,i2,2,1)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,2,1)
        mm2pdfpdfiitt_0(iref1,i1,i2,3,1)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,3,1)
        mm2pdfpdfiitt_0(iref1,i1,i2,4,1)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,4,1)
        mm2pdfpdfiitt_0(iref1,i1,i2,3,2)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,3,2)
        mm2pdfpdfiitt_0(iref1,i1,i2,4,2)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,4,2)
        mm2pdfpdfiitt_0(iref1,i1,i2,4,3)
     &  =mm2pdfpdfiitt_0(iref2,i1,i2,4,3)
      enddo
#endif
                          endif
                        enddo
                      enddo
                    endif
                  else
                    call crosssection(k,xconv(1),xconv(2),
     &                  generator,phasespace,0,0,switch)
                  endif
                else if (phasespace.eq.5) then
                  if(psflag.eq.4) then
                    if(lgencon(0,7))then
                      m2pdfpdfq2_0(5,1)=m2pdfpdfq2_0(4,1)
                      m2pdfpdfq2_0(5,2)=m2pdfpdfq2_0(4,2)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
	     iref1=(nset-1)*maxps+5
	     iref2=(nset-1)*maxps+4
        mm2pdfpdfq2_0(iref1,1)=mm2pdfpdfq2_0(iref2,1)
        mm2pdfpdfq2_0(iref1,2)=mm2pdfpdfq2_0(iref2,2)
      enddo
#endif
                      do i1=-4,4
                        do i2=-4,4
                          if(i1*i2.ne.0)then
                            m2pdfpdfiiq2_0(5,i1,i2,1)
     &                          =m2pdfpdfiiq2_0(4,i1,i2,1)
                            m2pdfpdfiiq2_0(5,i1,i2,2)
     &                          =m2pdfpdfiiq2_0(4,i1,i2,2)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
	     iref1=(nset-1)*maxps+5
	     iref2=(nset-1)*maxps+4
        mm2pdfpdfiiq2_0(iref1,i1,i2,1)
     &  =mm2pdfpdfiiq2_0(iref2,i1,i2,1)
        mm2pdfpdfiiq2_0(iref1,i1,i2,2)
     &  =mm2pdfpdfiiq2_0(iref2,i1,i2,2)
      enddo
#endif
                          endif
                        enddo
                      enddo
                    endif
                    if(lgencon(0,9))then
                      m2pdfpdft2_0(5,1)=m2pdfpdft2_0(4,1)
                      m2pdfpdft2_0(5,2)=m2pdfpdft2_0(4,2)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
	     iref1=(nset-1)*maxps+5
	     iref2=(nset-1)*maxps+4
        mm2pdfpdft2_0(iref1,1)=mm2pdfpdft2_0(iref2,1)
        mm2pdfpdft2_0(iref1,2)=mm2pdfpdft2_0(iref2,2)
      enddo
#endif
                      do i1=-4,4
                        do i2=-4,4
                          if(i1*i2.ne.0)then
                            m2pdfpdfiit2_0(5,i1,i2,1)
     &                          =m2pdfpdfiit2_0(4,i1,i2,1)
                            m2pdfpdfiit2_0(5,i1,i2,2)
     &                          =m2pdfpdfiit2_0(4,i1,i2,2)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
	     iref1=(nset-1)*maxps+5
	     iref2=(nset-1)*maxps+4
        mm2pdfpdfiit2_0(iref1,i1,i2,1)
     &  =mm2pdfpdfiit2_0(iref2,i1,i2,1)
        mm2pdfpdfiit2_0(iref1,i1,i2,2)
     &  =mm2pdfpdfiit2_0(iref2,i1,i2,2)
      enddo
#endif
                          endif
                        enddo
                      enddo
                    endif
                  else
                    call crosssection(k,xconv(1),xconv(2),
     &                  generator,phasespace,0,0,switch)
                  endif
                endif
                endif
#endif

                call dets(k,s(1,1,phasespace),nextgen(generator),
     &            switch)



                fac(phasespace)=0.5d0/pi/gsum(phasespace)*wconv


                if (phasespace.eq.1.and.ssli(generator).ne.0.and.
     &              (sgencon(generator,3).ne.0.or.
     &               sgencon(generator,4).ne.0.or.
     &               sgencon(generator,6).ne.0.or.
     &               sgencon(generator,7).ne.0.or.
     &               sgencon(generator,8).ne.0.or.
     &               sgencon(generator,9).ne.0)
     &              )
     &              then

c     boost momenta to cms
                  do i1=0,3
                    q(i1)=k(1,i1)+k(2,i1)
                  enddo


                  do ext=1,nextgen(generator)


                    do vcomp=0,3
                      q1(vcomp)=k(ext,vcomp)
                    enddo


                    call boost(shat,q,q1,1d0,switch)
                    do vcomp=0,3
                      kcms(ext,vcomp)=q1(vcomp)
                    enddo


                    ecms(ext,phasespace)=kcms(ext,0)
                  enddo
                endif
              endif
            endif
          else
            if(cut(phasespace).eq.0)then
              cut(phasespace)=1
            endif
          endif
          if (switch.gt.0) return


          xisr(phasespace)=x
          omxisr(phasespace)=omx
          xdisr(phasespace)=xd
          omxdisr(phasespace)=omxd
c          write(*,*) 'integrand x= ',phasespace,x
        endif
      enddo
c end generation of momenta and calculation of jacobians and crosssection
c (end loop over phasespaces)
c        write(*,*) 'integrand: b switch= ',switch,cut(1)

c        write(*,*) 'integrand befborn',switch,cut(1),
c     &    cut(2),cut(3),cut(4),cut(5)

c tree-level amplitude and virtual correction
        if(switch.eq.0)then

c          write(*,*) ' integrand: sborn(gen) = ',sborn(generator)
c     &        mod(nevt,freqgencon(generator,1))

          if(cut(1).eq.0)then
            if(sborn(generator).ne.0.and.
     &          mod(nevt,iNumProcs*freqgencon(generator,1)).lt.
     &              iNumProcs)then
              weight(1,1)=weight(1,1)+dble(isign(1,sborn(generator)))
     &            *m2pdfpdf_0(1)*wconv*factor(generator)/gsum(1)
     &            *freqgencon(generator,1)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
              mweight(nset,1,1)=mweight(nset,1,1)+
     &        dble(isign(1,sborn(generator)))
     &            *mm2pdfpdf_0(iref)*wconv*factor(generator)/gsum(1)
     &            *freqgencon(generator,1)
      enddo
#endif

c        write(*,*) 'integrand weightborn', weight(1,1)


#ifdef SUBPROCESSES
              do parton1=-5,5
              do parton2=-5,5
              if (parton1*parton2.ne.0) then
                weightii(1,parton1,parton2,1)
     &              =m2pdfpdfii_0(1,parton1,parton2)
     &              *wconv*factor(generator)/gsum(1)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                mweightii(nset,1,parton1,parton2,1)
     &              =mm2pdfpdfii_0(iref,parton1,parton2)
     &              *wconv*factor(generator)/gsum(1)
      enddo
#endif
 &              *freqgencon(generator,1)
              endif
              enddo
              enddo
c          write(*,*) ' integrand: born(gen) = ',born(0,1)
c          write(*,*) ' integrand: wconv     = ',wconv
c          write(*,*) ' integrand: factor    = ',factor(generator)
c          write(*,*) ' integrand: gsum(1)   = ',gsum(1)
c          write(*,*) ' integrand: born(gen) = ',born
#endif
            endif

            if(svirt(generator).ne.0)then
              if((sweak.ne.0.or.shh2.ne.0)
     &	        .and.mod(nevt,iNumProcs*freqgencon(generator,2)).lt.
     &                   iNumProcs)
     &            then
                weight(2,1)=weight(2,1)
     &              +m2pdfpdf_vw(1)*wconv*factor(generator)/gsum(1)
     &              *freqgencon(generator,2)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                mweight(nset,2,1)=mweight(nset,2,1)
     &              +mm2pdfpdf_vw(iref)*wconv*factor(generator)/gsum(1)
     &              *freqgencon(generator,2)
      enddo
#endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                  if (parton1*parton2.ne.0) then
                    weightii(2,parton1,parton2,1)
     &                  =m2pdfpdfii_vw(1,parton1,parton2)
     &                  *wconv*factor(generator)/gsum(1)
     &                  *freqgencon(generator,2)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweightii(nset,2,parton1,parton2,1)
     &                  =mm2pdfpdfii_vw(iref,parton1,parton2)
     &                  *wconv*factor(generator)/gsum(1)
     &                  *freqgencon(generator,2)
      enddo
#endif
                  endif
                enddo
                enddo
#endif
              endif
              if(sqcd.ne.0.and.
     &           mod(nevt,iNumProcs*freqgencon(generator,5)).lt.
     &               iNumProcs)
     &            then
                weight(5,1)=weight(5,1)
     &              +m2pdfpdf_vs(1)*wconv*factor(generator)/gsum(1)
     &              *freqgencon(generator,5)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                mweight(nset,5,1)=mweight(nset,5,1)
     &              +mm2pdfpdf_vs(iref)*wconv*factor(generator)/gsum(1)
     &              *freqgencon(generator,5)
      enddo
#endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                  if (parton1*parton2.ne.0) then
                    weightii(5,parton1,parton2,1)
     &                  =m2pdfpdfii_vs(1,parton1,parton2)
     &                  *wconv*factor(generator)/gsum(1)
     &                  *freqgencon(generator,5)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweightii(nset,5,parton1,parton2,1)
     &                  =mm2pdfpdfii_vs(iref,parton1,parton2)
     &                  *wconv*factor(generator)/gsum(1)
     &                  *freqgencon(generator,5)
      enddo
#endif
                  endif
                enddo
                enddo
#endif
              endif
            endif

            if(swborn(generator).ne.0.and.
     &          mod(nevt,iNumProcs*freqgencon(generator,1))
     &              .lt.iNumProcs)then
              weight_0=m2pdfpdf_0(1)*wconv*factor(generator)/gsum(1)
     &            *freqgencon(generator,1)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
              mweight_0(nset)=
     &        mm2pdfpdf_0(iref)*wconv*factor(generator)/gsum(1)
     &            *freqgencon(generator,1)
      enddo
#endif


#ifdef SUBPROCESSES
              do parton1=-5,5
                do parton2=-5,5
                  if (parton1*parton2.ne.0) then
                    weightii_0(parton1,parton2)
     &                  =m2pdfpdfii_0(1,parton1,parton2)
     &                  *wconv*factor(generator)/gsum(1)
     &                  *freqgencon(generator,1)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweightii_0(nset,parton1,parton2)
     &                  =mm2pdfpdfii_0(iref,parton1,parton2)
     &                  *wconv*factor(generator)/gsum(1)
     &                  *freqgencon(generator,1)
      enddo
#endif
                  endif
                enddo
              enddo
#endif
            endif

c          write(*,*) ' integrand: weight = ',weight(1,1),weight_0
c     &        ,wconv
c          write(*,*) ' integrand: weight(1) = ',born(0,2),wconv,
c     &        factor(generator),gsum(1)
          endif

c        write(*,*) 'integrand cut',cut(1)

          if(sborn(generator).ne.0)then
            rejectedcon(1,1)=cut(1)
            rejected(1)=cut(1)
          endif

c          write(*,*) 'integrand befvirtrej',svirt(generator)

          if(svirt(generator).ne.0)then
            rejectedcon(2,1)=cut(1)
            rejected(2)=cut(1)
            rejectedcon(5,1)=cut(1)
            rejected(5)=cut(1)

c            write(*,*) 'integrand rejvirt',rejected(2)

          endif
          if(swborn(generator).ne.0)then
            rejected_0=cut(1)

c            write(*,*) 'integrand cutb=',rejected_0

          endif
        endif

c subtraction terms

c       write(*,*) 'integrand: ssub ',ssub(generator),generator

        if(ssub(generator).ne.0.and.
     &      (lgencon(generator,3).or.lgencon(generator,4))
     &      .and.switch.eq.0) then
          if(cut(1).eq.0.or.cut(2).eq.0.or.cut(3).eq.0)then
c final-state emitter with final-state spectator
c          do em=3,nextgen(generator)
c          do sp=3,nextgen(generator)
c ADAPTED FOR UNCHARGED HIGGS
          do em=3,nextsub(generator)
          do sp=3,nextsub(generator)
          if(em.ne.sp)then
c            pf=qedfactor(em,sp,generator)+qcdfactor(em,sp,generator)
            pf= factor(generator)
            if(pf.ne.0d0)then
              if(ssub(generator).eq.1)then
                if(cut(1).eq.0)then
                  wfac=-pf*(1d0-pi2/3d0-const(em,sp))*fac(1)
c     &              -pf*0.5d0*fac(1)*(born(em,1)-born(0,1))

                  if(lgencon(0,3))then
                    wfacp=mcalpha*wfac*freqgencon(generator,3)
                    weight(3,1)=weight(3,1)+wfacp*m2pdfpdfqq_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweight(nset,3,1)=mweight(nset,3,1)+
     &              wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                  endif
                  if(lgencon(0,4))then
                    wfacg=mcalphas*wfac*freqgencon(generator,4)
                    weight(4,1)=weight(4,1)+wfacg*m2pdfpdftt_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweight(nset,4,1)=mweight(nset,4,1)+
     &              wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                  endif
#ifdef SUBPROCESSES
                  do parton1=-4,4
                  do parton2=-4,4
                  if (parton1*parton2.ne.0) then
                    if(lgencon(0,3))then
                      weightii(3,parton1,parton2,1) =
     &                    weightii(3,parton1,parton2,1) +
     &                    wfacp*m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweightii(nset,3,parton1,parton2,1) =
     &                    weightii(3,parton1,parton2,1) +
     &                    wfacp*mm2pdfpdfiiqq_0(iref,parton1,parton2,em,
     &                    sp)
      enddo
#endif
                    endif
                    if(lgencon(0,4))then
                      weightii(4,parton1,parton2,1) =
     &                    weightii(4,parton1,parton2,1) +
     &                    wfacg*m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweightii(nset,4,parton1,parton2,1) =
     &                    weightii(4,parton1,parton2,1) +
     &                    wfacg*mm2pdfpdfiitt_0(iref,parton1,parton2,em,
     &                    sp)
      enddo
#endif
                    endif
                  endif
                  enddo
                  enddo
#endif
               endif
c "exclusive" subtraction terms
              elseif(ssub(generator).eq.2)then
                if(cut(1).eq.0)then
                  wfac = -pf*(2.5d0-2d0/3d0*pi2-const(em,sp))*fac(1)
c     &              -pf*0.5d0*fac(1)*(born(em,1)-born(0,1))
                  if(lgencon(0,3))then
                    wfacp=mcalpha*wfac*freqgencon(generator,3)
                    weight(3,1)=weight(3,1)+wfacp*m2pdfpdfqq_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweight(nset,3,1)=mweight(nset,3,1)+
     &              wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                  endif
                  if(lgencon(0,4))then
                    wfacg=mcalphas*wfac*freqgencon(generator,4)
                    weight(4,1)=weight(4,1)+wfacg*m2pdfpdftt_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweight(nset,4,1)=mweight(nset,4,1)+
     &              wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                  endif
#ifdef SUBPROCESSES
                  do parton1=-4,4
                  do parton2=-4,4
                  if (parton1*parton2.ne.0) then
                    if(lgencon(0,3))then
                      weightii(3,parton1,parton2,1) =
     &                    weightii(3,parton1,parton2,1) +
     &                  wfacp*m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweightii(nset,3,parton1,parton2,1) =
     &                    weightii(3,parton1,parton2,1) +
     &                  wfacp*mm2pdfpdfiiqq_0(iref,parton1,parton2,em,
     &                  sp)
      enddo
#endif
                    endif
                    if(lgencon(0,4))then
                      weightii(4,parton1,parton2,1) =
     &                    weightii(4,parton1,parton2,1) +
     &                    wfacg*m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweightii(nset,4,parton1,parton2,1) =
     &                    weightii(4,parton1,parton2,1) +
     &                    wfacg*mm2pdfpdfiitt_0(iref,parton1,parton2,em,
     &                    sp)
      enddo
#endif
                    endif
                  endif
                  enddo
                  enddo
#endif
                endif
c "exclusive" subtraction terms
              endif
            endif
          endif
          enddo
          enddo

c final-state emitter with initial-state spectator
c          do em=3,nextgen(generator)
c ADAPTED FOR UNCHARGED HIGGS
          do em=3,nextsub(generator)
          do sp=1,2
          phasespace=sp+1
          x=xisr(phasespace)
          omx=omxisr(phasespace)
          xd=xdisr(phasespace)
          omxd=omxdisr(phasespace)
c          pf=(qedfactor(em,sp,generator)+qedfactor(em,sp,generator))
c     &        *woaisr(phasespace)
c minus sign from incoming*outgoing particles
          pf= -factor(generator) * woaisr(phasespace)
          if(pf.ne.0d0)then
            if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
              f=(2d0*dlog((2d0-x)/omx)-1.5d0)/omx
              if(cut(phasespace).eq.0)then
                wfac=-pf*f*fac(phasespace)/x
                if(lgencon(0,3))then
                  wfacp=mcalpha*wfac*freqgencon(generator,3)
                  weight(3,phasespace)=weight(3,phasespace)
     &              +wfacp*m2pdfpdfqq_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweight(nset,3,phasespace)=mweight(nset,3,phasespace)
     &              +wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                endif
                if(lgencon(0,4))then
                  wfacg=mcalphas*wfac*freqgencon(generator,4)
                  weight(4,phasespace)=weight(4,phasespace)
     &              +wfacg*m2pdfpdftt_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweight(nset,4,phasespace)=mweight(nset,4,phasespace)
     &              +wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                endif
c                test=0d0
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  if(lgencon(0,3))then
                    weightii(3,parton1,parton2,phasespace) =
     &                  weightii(3,parton1,parton2,phasespace) +
     &                  wfacp*
     &                  m2pdfpdfiiqq_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweightii(nset,3,parton1,parton2,phasespace) =
     &                  weightii(3,parton1,parton2,phasespace) +
     &                  wfacp*
     &                  mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                  endif
                  if(lgencon(0,4))then
                    weightii(4,parton1,parton2,phasespace) =
     &                  weightii(4,parton1,parton2,phasespace) +
     &                  wfacg*
     &                  m2pdfpdfiitt_0(phasespace,parton1,parton2,em,sp)
c                  test = test +
c     &                m2pdfpdfiiqq_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweightii(nset,4,parton1,parton2,phasespace) =
     &                  weightii(4,parton1,parton2,phasespace) +
     &                  wfacg*
     &                  mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
c                  test = test +
c     &                mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                  endif
                endif
                enddo
                enddo
#endif
              endif
              if(cut(1).eq.0)then
                wfac=+pf*f*fac(1)
     &            -pf*fac(1)*(1d0-0.5d0*pi2-const(em,sp))
                if(lgencon(0,3))then
                  wfacp=mcalpha*wfac*freqgencon(generator,3)
                  weight(3,1)=weight(3,1)
     &                +wfacp*m2pdfpdfqq_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                  mweight(nset,3,1)=mweight(nset,3,1)
     &                +wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                endif
                if(lgencon(0,4))then
                  wfacg=mcalphas*wfac*freqgencon(generator,4)
                  weight(4,1)=weight(4,1)
     &                +wfacg*m2pdfpdftt_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                  mweight(nset,4,1)=mweight(nset,4,1)
     &                +wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                  if (parton1*parton2.ne.0) then
                    if(lgencon(0,3))then
                      weightii(3,parton1,parton2,1) =
     &                    weightii(3,parton1,parton2,1) +
     &                    wfacp*m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweightii(nset,3,parton1,parton2,1) =
     &                    weightii(3,parton1,parton2,1) +
     &                    wfacp*mm2pdfpdfiiqq_0(iref,parton1,parton2,em,
     &                    sp)
      enddo
#endif
                    endif
                    if(lgencon(0,4))then
                      weightii(4,parton1,parton2,1) =
     &                    weightii(4,parton1,parton2,1) +
     &                    wfacg*m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweightii(nset,4,parton1,parton2,1) =
     &                    weightii(4,parton1,parton2,1) +
     &                    wfacg*mm2pdfpdfiitt_0(iref,parton1,parton2,em,
     &                    sp)
      enddo
#endif
                    endif
                  endif
                enddo
                enddo
#endif
              endif
            endif
          endif
          enddo
          enddo

c initial-state emitter with final-state spectator
          do em=1,2
c          do sp=3,nextgen(generator)
c ADAPTED FOR UNCHARGED HIGGS
          do sp=3,nextsub(generator)
            phasespace=em+1
            x=xisr(phasespace)
            omx=omxisr(phasespace)
            xd=xdisr(phasespace)
            omxd=omxdisr(phasespace)
c          pf=(qedfactor(em,sp,generator)+qedfactor(em,sp,generator))
c     &        *woaisr(phasespace)
c minus sign from incoming*outgoing particles
            pf= -factor(generator) * woaisr(phasespace)
            if(pf.ne.0d0)then
              if(s(em,sp,phasespace).le.0d0.or.s(em,sp,1).le.0d0)then
                switch=29
                if(numout.lt.maxout.and.lnoutmc.ge.5)then
                  write(nout,100) nevt,cogen,cocha,switch,
c                write(*,'(" event= ",i9,",",(a))'),nevt,
     &                ' integrand: s(em,sp,...) < 0'
                 write(nout,*) 's(em,sp,ps) = ',em,sp,phasespace,
     &                s(em,sp,phasespace),s(em,sp,1)
                  numout=numout+1
                endif
                return
              else
                if(ssub(generator).eq.1.and.sfactqed.eq.0
     &              .and.sfactqcd.eq.0)then
                  f=(-2d0)/omx*dlog(2d0-x)+(1d0+x)*dlog(omx)
                  do parton1=-4,4
                  do parton2=-4,4
                  if (parton1*parton2.ne.0) then
                    if(em.eq.1)then
                      mass2(em,generator) = m2parton(parton1)
c                      write(*,*) 'integrand: mass1 ',
c     &                    parton1,mass2(em,generator)
                    else
                      mass2(em,generator) = m2parton(parton2)
c                      write(*,*) 'integrand: mass1 ',
c     &                    parton1,mass2(em,generator)
                    endif
c                    write(*,*) 'integrand: mass  ',mass2(em,generator)
c                    mass2(em,generator)=qedfacscale2/omx**2/exp(1d0)
                    if(cut(phasespace).eq.0)then
                      wfac=
     &                    -pf*(sfx(em)
     &                    *(dlog(s(em,sp,phasespace)/mass2(em,generator)
     &                                     /x)
     &                    -1d0)+f)*fac(phasespace)/x
     &                    -pf*omx*fac(phasespace)/x
c     &              -pf*omx*fac(phasespace)*born(em,phasespace)/x
c     &             *(born(em,phasespace)-born(0,phasespace))
                      if(lgencon(0,3))then
                        wfacp=mcalpha*wfac*freqgencon(generator,3)
#ifdef SUBPROCESSES
                        weightii(3,parton1,parton2,phasespace) =
     &                      weightii(3,parton1,parton2,phasespace) +
     &                      wfacp*m2pdfpdfiiqq_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweightii(nset,3,parton1,parton2,phasespace) =
     &                      weightii(3,parton1,parton2,phasespace) +
     &                      wfacp*mm2pdfpdfiiqq_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
#endif
                        weight(3,phasespace)=weight(3,phasespace) +
     &                      wfacp*m2pdfpdfiiqq_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweight(nset,3,phasespace)=
     &                  mweight(nset,3,phasespace) +
     &                      wfacp*mm2pdfpdfiiqq_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
                      endif
                      if(lgencon(0,4))then
                        wfacg=mcalphas*wfac*freqgencon(generator,4)
#ifdef SUBPROCESSES
                        weightii(4,parton1,parton2,phasespace) =
     &                      weightii(4,parton1,parton2,phasespace) +
     &                      wfacg*m2pdfpdfiitt_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweightii(nset,4,parton1,parton2,phasespace) =
     &                      weightii(4,parton1,parton2,phasespace) +
     &                      wfacg*mm2pdfpdfiitt_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
#endif
                        weight(4,phasespace)=weight(4,phasespace) +
     &                      wfacg*m2pdfpdfiitt_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweight(nset,4,phasespace)=
     &                  mweight(nset,4,phasespace) +
     &                      wfacg*mm2pdfpdfiitt_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
                      endif


                    endif
                    if(cut(1).eq.0)then
                      wfac= pf*
     &                  (sfx(em)*(dlog(s(em,sp,1)/mass2(em,generator)/x)
     &                  -1d0)+f)*fac(1)
     &                  +pf*omx*fac(1)
     &                  -pf*(-(3d0/2d0)+pi2/6d0-const(em,sp))
     &                  *fac(1)
c     &              +pf*omx*fac(1)*(born(em,1)-born(0,1))
c     &              -pf*(0.5d0)*fac(1)*(born(em,1)-born(0,1))
                      if(lgencon(0,3))then
                        wfacp=mcalpha*wfac*freqgencon(generator,3)
#ifdef SUBPROCESSES
                        weightii(3,parton1,parton2,1) =
     &                      weightii(3,parton1,parton2,1) +
     &                      wfacp
     &                      *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                        mweightii(nset,3,parton1,parton2,1) =
     &                      weightii(3,parton1,parton2,1) +
     &                      wfacp
     &                      *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
#endif
                        weight(3,1)=weight(3,1)+
     &                      wfacp
     &                      *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                        mweight(nset,3,1)=mweight(nset,3,1)+
     &                      wfacp
     &                      *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                      endif
                      if(lgencon(0,4))then
                        wfacg=mcalphas*wfac*freqgencon(generator,4)
#ifdef SUBPROCESSES
                        weightii(4,parton1,parton2,1) =
     &                      weightii(4,parton1,parton2,1) +
     &                      wfacg
     &                      *m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                        mweightii(nset,4,parton1,parton2,1) =
     &                      weightii(4,parton1,parton2,1) +
     &                      wfacg
     &                      *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
#endif
                        weight(4,1)=weight(4,1)+
     &                      wfacg
     &                      *m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                        mweight(nset,4,1)=mweight(nset,4,1)+
     &                      wfacg
     &                      *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                      endif
                    endif
                  endif
                  enddo
                  enddo
#ifdef SUBPROCESSES
#endif

                elseif(ssub(generator).eq.2.and.sfactqed.eq.0
     &                .and.sfactqcd.eq.0)then
                  f=2d0/omx*dlog(omx/(2d0-x))-sfx(em)*(dlog(omx)+1d0)
                  do parton1=-4,4
                  do parton2=-4,4
                  if (parton1*parton2.ne.0) then
                    if(em.eq.1)then
                      mass2(em,generator) = m2parton(parton1)
                    else
                      mass2(em,generator) = m2parton(parton2)
                    endif
                    if(cut(phasespace).eq.0)then
                      wfac=
     &                    -pf*(f-sfx(em)
     &                    *dlog(mass2(em,generator)*x/
     &                    s(em,sp,phasespace)))
     &                    *fac(phasespace)/x
     &                    -pf*omx*fac(phasespace)/x
c     &              -pf*omx*fac(phasespace)/x
c     &                 *(born(em,phasespace)-born(0,phasespace))
                      if(lgencon(0,3))then
                        wfacp=mcalpha*wfac*freqgencon(generator,3)
#ifdef SUBPROCESSES
                        weightii(3,parton1,parton2,phasespace) =
     &                      weightii(3,parton1,parton2,phasespace) +
     &                      wfacp*m2pdfpdfiiqq_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweightii(nset,3,parton1,parton2,phasespace) =
     &                      weightii(3,parton1,parton2,phasespace) +
     &                      wfacp*mm2pdfpdfiiqq_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
#endif
                        weight(3,phasespace)=weight(3,phasespace) +
     &                      wfacp*m2pdfpdfiiqq_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweight(nset,3,phasespace)=
     &                  mweight(nset,3,phasespace) +
     &                      wfacp*mm2pdfpdfiiqq_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
                      endif
                      if(lgencon(0,4))then
                        wfacg=mcalphas*wfac*freqgencon(generator,4)
#ifdef SUBPROCESSES
                        weightii(4,parton1,parton2,phasespace) =
     &                      weightii(4,parton1,parton2,phasespace) +
     &                      wfacg*m2pdfpdfiitt_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweightii(nset,4,parton1,parton2,phasespace) =
     &                      weightii(4,parton1,parton2,phasespace) +
     &                      wfacg*mm2pdfpdfiitt_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
#endif
                        weight(4,phasespace)=weight(4,phasespace) +
     &                      wfacg*m2pdfpdfiitt_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweight(nset,4,phasespace)=
     &                  mweight(nset,4,phasespace) +
     &                      wfacg*mm2pdfpdfiitt_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
                      endif
                    endif
                    if(cut(1).eq.0)then
                      wfac=pf*
     &                 (f-sfx(em)*dlog(mass2(em,generator)/s(em,sp,1)))
     &                    *fac(1)
     &                    +pf*omx*fac(1)
     &                    -pf*(-0.25d0-pi2/6d0-const(em,sp))
     &                    *fac(1)
c     &              +pf*omx*fac(1)*(born(em,1)-born(0,1))
c     &              -pf*0.5d0*fac(1)*(born(em,1)-born(0,1))
                      if(lgencon(0,3))then
                        wfacp=mcalpha*wfac*freqgencon(generator,3)
#ifdef SUBPROCESSES
                        weightii(3,parton1,parton2,1) =
     &                      weightii(3,parton1,parton2,1) +
     &                      wfacp
     &                      *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                        mweightii(nset,3,parton1,parton2,1) =
     &                      weightii(3,parton1,parton2,1) +
     &                      wfacp
     &                      *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
#endif
                        weight(3,1)=weight(3,1) +
     &                      wfacp
     &                      *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                        mweight(nset,3,1)=mweight(nset,3,1) +
     &                      wfacp
     &                      *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                      endif
                      if(lgencon(0,4))then
                        wfacg=mcalphas*wfac*freqgencon(generator,4)
#ifdef SUBPROCESSES
                        weightii(4,parton1,parton2,1) =
     &                      weightii(4,parton1,parton2,1) +
     &                      wfacg
     &                      *m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                        mweightii(nset,4,parton1,parton2,1) =
     &                      weightii(4,parton1,parton2,1) +
     &                      wfacg
     &                      *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
#endif
                        weight(4,1)=weight(4,1) +
     &                      wfacg
     &                      *m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                        mweight(nset,4,1)=mweight(nset,4,1) +
     &                      wfacg
     &                      *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                      endif
                    endif
                  endif
                  enddo
                  enddo


                else if(ssub(generator).eq.1.and.sfactqed.gt.0
     &                .and.sfactqcd.gt.0)then
                  f=(-2d0)/omx*dlog(2d0-x)+(1d0+x)*dlog(omx)
                  if(cut(phasespace).eq.0)then
                    if(lgencon(0,3))then
                      if(sfactqed.eq.1)then
                        wfac=
     &                      -pf*fac(phasespace)*(f+omx+sfx(em)
     &                      *log(s(em,sp,phasespace)*omx*omx/
     &                      (qedfacscale2*x))
     &                      )/x
                      elseif(sfactqed.eq.2)then
                        wfac=
     &                      -pf*fac(phasespace)*(f+omx+sfx(em)
     &                      *(log(s(em,sp,phasespace)*omx/
     &                      (qedfacscale2))+.75d0)
     &                      -2.25d0-1.25d0*x
     &                      )/x
                      endif
                      wfacp=mcalpha*wfac*freqgencon(generator,3)
                      weight(3,phasespace)=weight(3,phasespace)
     &                    +wfacp*m2pdfpdfqq_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweight(nset,3,phasespace)=
     &                mweight(nset,3,phasespace)
     &                    +wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                    endif
                    if(lgencon(0,4))then
                      if(sfactqcd.eq.1)then
                        wfac=
     &                      -pf*fac(phasespace)*(f+omx+sfx(em)
     &                      *log(s(em,sp,phasespace)*omx*omx/
     &                      (qcdfacscale2*x))
     &                      )/x
                      elseif(sfactqcd.eq.2)then
                        wfac=
     &                      -pf*fac(phasespace)*(f+omx+sfx(em)
     &                      *(log(s(em,sp,phasespace)*omx/
     &                      (qcdfacscale2))+.75d0)
     &                      -2.25d0-1.25d0*x
     &                      )/x
                      endif
                      wfacg=mcalphas*wfac*freqgencon(generator,4)
                      weight(4,phasespace)=weight(4,phasespace)
     &                    +wfacg*m2pdfpdftt_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweight(nset,4,phasespace)=
     &                mweight(nset,4,phasespace)
     &                    +wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                    endif
#ifdef SUBPROCESSES
                    do parton1=-4,4
                    do parton2=-4,4
                    if (parton1*parton2.ne.0) then
                      if(lgencon(0,3))then
                        weightii(3,parton1,parton2,phasespace) =
     &                      weightii(3,parton1,parton2,phasespace) +
     &                      wfacp*m2pdfpdfiiqq_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweightii(nset,3,parton1,parton2,phasespace) =
     &                      weightii(3,parton1,parton2,phasespace) +
     &                      wfacp*mm2pdfpdfiiqq_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
                      endif
                      if(lgencon(0,4))then
                        weightii(4,parton1,parton2,phasespace) =
     &                      weightii(4,parton1,parton2,phasespace) +
     &                      wfacg*m2pdfpdfiitt_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweightii(nset,4,parton1,parton2,phasespace) =
     &                      weightii(4,parton1,parton2,phasespace) +
     &                      wfacg*mm2pdfpdfiitt_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
                      endif
                    endif
                    enddo
                    enddo

#endif
                 endif
                  if(cut(1).eq.0)then
                    if(lgencon(0,3))then
                      if(sfactqed.eq.1)then
                        wfac= pf*fac(1)*
     &                      (f+omx+sfx(em)*
     &                      dlog(s(em,sp,1)*omx*omx/(qedfacscale2*x)))
     &                      -pf*fac(1)*(-(3d0/2d0)+pi2/6d0-const(em,sp))
                      elseif(sfactqed.eq.2)then
                        wfac= pf*fac(1)*
     &                      (f+omx+sfx(em)*
     &                      (dlog(s(em,sp,1)*omx/(qedfacscale2))+.75d0)
     &                      -2.25d0-1.25d0*x
     &                      )
     &                      -pf*fac(1)*(-(3d0/2d0)+pi2/6d0-const(em,sp))
                      endif
                      wfacp=mcalpha*wfac*freqgencon(generator,3)
                      weight(3,1)=weight(3,1)+
     &                    wfacp*m2pdfpdfqq_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweight(nset,3,1)=mweight(nset,3,1)+
     &                    wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                    endif
                    if(lgencon(0,4))then
                      if(sfactqcd.eq.1)then
                        wfac= pf*fac(1)*
     &                      (f+omx+sfx(em)*
     &                      dlog(s(em,sp,1)*omx*omx/(qcdfacscale2*x)))
     &                      -pf*fac(1)*(-(3d0/2d0)+pi2/6d0-const(em,sp))
                      elseif(sfactqcd.eq.2)then
                        wfac= pf*fac(1)*
     &                      (f+omx+sfx(em)*
     &                      (dlog(s(em,sp,1)*omx/(qcdfacscale2))+.75d0)
     &                      -2.25d0-1.25d0*x
     &                      )
     &                      -pf*fac(1)*(-(3d0/2d0)+pi2/6d0-const(em,sp))
                      endif
                      wfacg=mcalphas*wfac*freqgencon(generator,4)
                      weight(4,1)=weight(4,1)+
     &                    wfacg*m2pdfpdftt_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweight(nset,4,1)=mweight(nset,4,1)+
     &                    wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                    endif
#ifdef SUBPROCESSES
                    do parton1=-4,4
                    do parton2=-4,4
                      if (parton1*parton2.ne.0) then
                        if(lgencon(0,3))then
                          weightii(3,parton1,parton2,1) =
     &                        weightii(3,parton1,parton2,1) +
     &                        wfacp
     &                        *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                          mweightii(nset,3,parton1,parton2,1) =
     &                        weightii(3,parton1,parton2,1) +
     &                        wfacp
     &                        *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,
     &                        sp)
      enddo
#endif
                        endif
                        if(lgencon(0,4))then
                          weightii(4,parton1,parton2,1) =
     &                        weightii(4,parton1,parton2,1) +
     &                        wfacg
     &                        *m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                          mweightii(nset,4,parton1,parton2,1) =
     &                        weightii(4,parton1,parton2,1) +
     &                        wfacg
     &                        *mm2pdfpdfiitt_0(iref,parton1,parton2,em,
     &                        sp)
      enddo
#endif
                        endif
                      endif
                    enddo
                    enddo
#endif
                  endif

                elseif(ssub(generator).eq.2.and.sfactqed.gt.0
     &                .and.sfactqcd.gt.0)then
                  write(*,*) 'integrand: ssub = 2,sfactqed>0',
     &                ' not yet tested'
                  stop
                  if(lgencon(0,3))then
                    if(sfactqed.eq.1)then
                      fp=2d0/omx*dlog(omx/(2d0-x))+sfx(em)*dlog(omx)
                    elseif(sfactqed.eq.2)then
                      fp=2d0/omx*dlog(omx/(2d0-x))
     &                    +sfx(em)*(log(x)+.75d0)
     &                    -2.25d0-1.25d0*x
                    endif
                  endif
                  if(lgencon(0,4))then
                    if(sfactqcd.eq.1)then
                      fg=2d0/omx*dlog(omx/(2d0-x))+sfx(em)*dlog(omx)
                    elseif(sfactqcd.eq.2)then
                      fg=2d0/omx*dlog(omx/(2d0-x))
     &                    +sfx(em)*(log(x)+.75d0)
     &                    -2.25d0-1.25d0*x
                    endif
                  endif
                  if(cut(phasespace).eq.0)then
                    if(lgencon(0,3))then
                      wfac=
     &                  -pf*(fp-sfx(em)
     &                  *dlog(qedfacscale2*x/s(em,sp,phasespace)))
     &                  *fac(phasespace)/x
     &                  -pf*omx*fac(phasespace)/x
                      wfacp=mcalpha*wfac*freqgencon(generator,3)
                      weight(3,phasespace)=weight(3,phasespace) +
     &                    wfacp*m2pdfpdfqq_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweight(nset,3,phasespace)=
     &                mweight(nset,3,phasespace) +
     &                    wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                    endif
                    if(lgencon(0,4))then
                      wfac=
     &                  -pf*(fg-sfx(em)
     &                  *dlog(qcdfacscale2*x/s(em,sp,phasespace)))
     &                  *fac(phasespace)/x
     &                  -pf*omx*fac(phasespace)/x
                      wfacg=mcalphas*wfac*freqgencon(generator,4)
                      weight(4,phasespace)=weight(4,phasespace) +
     &                    wfacg*m2pdfpdftt_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweight(nset,4,phasespace)=
     &                mweight(nset,4,phasespace) +
     &                    wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                    endif
#ifdef SUBPROCESSES
                    do parton1=-4,4
                    do parton2=-4,4
                    if (parton1*parton2.ne.0) then
                      if(lgencon(0,3))then
                        weightii(3,parton1,parton2,phasespace) =
     &                      weightii(3,parton1,parton2,phasespace) +
     &                      wfacp*m2pdfpdfiiqq_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweightii(nset,3,parton1,parton2,phasespace) =
     &                      weightii(3,parton1,parton2,phasespace) +
     &                      wfacp*mm2pdfpdfiiqq_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
                      endif
                      if(lgencon(0,4))then
                        weightii(4,parton1,parton2,phasespace) =
     &                      weightii(4,parton1,parton2,phasespace) +
     &                      wfacg*m2pdfpdfiitt_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweightii(nset,4,parton1,parton2,phasespace) =
     &                      weightii(4,parton1,parton2,phasespace) +
     &                      wfacg*mm2pdfpdfiitt_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
                      endif
                    endif
                    enddo
                    enddo
#endif
                  endif
                  if(cut(1).eq.0)then
                    if(lgencon(0,3))then
                      wfac=pf*
     &                    (fp-sfx(em)*dlog(qedfacscale2/s(em,sp,1)))
     &                    *fac(1)
     &                    +pf*omx*fac(1)
     &                    -pf*(-0.25d0-pi2/6d0-const(em,sp))
     &                    *fac(1)
                      wfacp=mcalpha*wfac*freqgencon(generator,3)
                      weight(3,1)=weight(3,1) +
     &                    wfacp*m2pdfpdfqq_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweight(nset,3,1)=mweight(nset,3,1) +
     &                    wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                    endif
                    if(lgencon(0,4))then
                      wfac=pf*
     &                  (fg-sfx(em)*dlog(qcdfacscale2/s(em,sp,1)))
     &                  *fac(1)
     &                  +pf*omx*fac(1)
     &                  -pf*(-0.25d0-pi2/6d0-const(em,sp))
     &                  *fac(1)
c     &              +pf*omx*fac(1)*(born(em,1)-born(0,1))
c     &              -pf*0.5d0*fac(1)*(born(em,1)-born(0,1))
                      wfacg=mcalphas*wfac*freqgencon(generator,4)
                      weight(4,1)=weight(4,1) +
     &                    wfacg*m2pdfpdftt_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweight(nset,4,1)=mweight(nset,4,1) +
     &                    wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                    endif
#ifdef SUBPROCESSES
                    do parton1=-4,4
                    do parton2=-4,4
                      if (parton1*parton2.ne.0) then
                        if(lgencon(0,3))then
                          weightii(3,parton1,parton2,1) =
     &                        weightii(3,parton1,parton2,1) +
     &                        wfacp
     &                        *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                          mweightii(nset,3,parton1,parton2,1) =
     &                        weightii(3,parton1,parton2,1) +
     &                        wfacp
     &                        *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,
     &                        sp)
      enddo
#endif
                        endif
                        if(lgencon(0,4))then
                          weightii(4,parton1,parton2,1) =
     &                        weightii(4,parton1,parton2,1) +
     &                        wfacg
     &                        *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                          mweightii(nset,4,parton1,parton2,1) =
     &                        weightii(4,parton1,parton2,1) +
     &                        wfacg
     &                        *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,
     &                        sp)
      enddo
#endif
                        endif
                      endif
                    enddo
                    enddo
#endif
                  endif
                endif
              endif
            endif
          enddo
          enddo

c initial-state emitter with initial-state spectator
          do em=1,2
          do sp=1,2
          phasespace=em+1
          x=xisr(phasespace)
          omx=omxisr(phasespace)
          xd=xdisr(phasespace)
          omxd=omxdisr(phasespace)
c          pf=(qedfactor(em,sp,generator)+qedfactor(em,sp,generator))
c     &        *woaisr(phasespace)
          pf=factor(generator) * woaisr(phasespace)
          if(pf.ne.0d0.and.em.ne.sp)then
            if((ssub(generator).eq.1.or.ssub(generator).eq.2)
     &          .and.sfactqed.eq.0.and.sfactqcd.eq.0)then
              do parton1=-4,4
              do parton2=-4,4
              if (parton1*parton2.ne.0) then
                if(em.eq.1)then
                  mass2(em,generator) = m2parton(parton1)
c                  write(*,*) 'integrand: mass1 ',
c     &                parton1,mass2(em,generator)
                else
                  mass2(em,generator) = m2parton(parton2)
c                write(*,*) 'integrand: mass2 ',
c     &                parton2,mass2(em,generator)
                endif
c                write(*,*) 'integrand: mass ',mass2(em,generator)
c                mass2(em,generator)=qedfacscale2/omx**2/exp(1d0)
                if(s(1,2,1).gt.0d0)then
                  f=sfx(em)*(dlog(s(1,2,1)/mass2(em,generator))-1d0)
                else
                  switch=47
                  if(numout.lt.maxout.and.lnoutmc.ge.5)then
                    write(nout,100) nevt,cogen,cocha,switch,
     &                  ' integrand: s(1,2,1) < 0'
                    write(nout,*) 's(1,2,1) = ',s(1,2,1)
                    numout=numout+1
                  endif
                  return
                  f=0d0
                endif
                if(cut(phasespace).eq.0)then
                  wfac=
     &                -pf*f*fac(phasespace)/x
     &                -pf*omx*fac(phasespace)/x
c     &            -pf*omx*fac(phasespace)/x
c     &            *(born(em,phasespace)-born(em,phasespace))
                  if(lgencon(0,3))then
                    wfacp=mcalpha*wfac*freqgencon(generator,3)
#ifdef SUBPROCESSES
                    weightii(3,parton1,parton2,phasespace) =
     &                  weightii(3,parton1,parton2,phasespace) +
     &                  wfacp*
     &                  m2pdfpdfiiqq_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweightii(nset,3,parton1,parton2,phasespace) =
     &                  weightii(3,parton1,parton2,phasespace) +
     &                  wfacp*
     &                  mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
#endif
                    weight(3,phasespace)=weight(3,phasespace) +
     &                  wfacp*
     &                  m2pdfpdfiiqq_0(phasespace,parton1,parton2,em,sp)
c                  write(*,*) 'integrand: weight(3,ps) ',em,sp,
c     &                parton1,parton2,
c     &                weightii(3,parton1,parton2,phasespace),wfacp,
c     &                m2pdfpdfiiqq_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,3,phasespace)=
     &              mweight(nset,3,phasespace) +
     &                  wfacp*
     &                  mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
c                  write(*,*) 'integrand: mweight(nset,3,ps) ',em,sp,
c     &                parton1,parton2,
c     &                weightii(3,parton1,parton2,phasespace),wfacp,
c     &                mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                  endif
                  if(lgencon(0,4))then
                    wfacg=mcalphas*wfac*freqgencon(generator,4)
#ifdef SUBPROCESSES
                    weightii(4,parton1,parton2,phasespace) =
     &                  weightii(4,parton1,parton2,phasespace) +
     &                  wfacg*
     &                  m2pdfpdfiitt_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweightii(nset,4,parton1,parton2,phasespace) =
     &                  weightii(4,parton1,parton2,phasespace) +
     &                  wfacg*
     &                  mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
#endif
                    weight(4,phasespace)=weight(4,phasespace) +
     &                  wfacg*
     &                  m2pdfpdfiitt_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,4,phasespace)=
     &              mweight(nset,4,phasespace) +
     &                  wfacg*
     &                  mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                  endif
                endif
                if(cut(1).eq.0)then
                  wfac=
     &                +pf*f*fac(1)
     &                +pf*omx*fac(1)
     &                -pf*(1.5d0-pi2/3d0-const(em,sp))*fac(1)
c     &            +pf*omx*fac(1)*(born(em,1)-born(0,1))
c     &            -pf*0.5d0*fac(1)*(born(em,1)-born(0,1))
                  if(lgencon(0,3))then
                    wfacp=mcalpha*wfac*freqgencon(generator,3)
#ifdef SUBPROCESSES
                    weightii(3,parton1,parton2,1) =
     &                  weightii(3,parton1,parton2,1) +
     &                  wfacp
     &                  *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweightii(nset,3,parton1,parton2,1) =
     &                  weightii(3,parton1,parton2,1) +
     &                  wfacp
     &                  *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
#endif
                    weight(3,1)=weight(3,1) +
     &                  wfacp
     &                  *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
c                  write(*,*) 'integrand: weight(3,1) ',em,sp,
c     &                parton1,parton2,
c     &                weightii(3,parton1,parton2,1),wfacp,
c     &                m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweight(nset,3,1)=mweight(nset,3,1) +
     &                  wfacp
     &                  *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
c                  write(*,*) 'integrand: mweight(nset,3,1) ',em,sp,
c     &                parton1,parton2,
c     &                weightii(3,parton1,parton2,1),wfacp,
c     &                mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                  endif
                  if(lgencon(0,4))then
                    wfacg=mcalphas*wfac*freqgencon(generator,4)
#ifdef SUBPROCESSES
                    weightii(4,parton1,parton2,1) =
     &                  weightii(4,parton1,parton2,1) +
     &                  wfacg
     &                  *m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweightii(nset,4,parton1,parton2,1) =
     &                  weightii(4,parton1,parton2,1) +
     &                  wfacg
     &                  *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
#endif
                    weight(4,1)=weight(4,1) +
     &                  wfacg
     &                  *m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweight(nset,4,1)=mweight(nset,4,1) +
     &                  wfacg
     &                  *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                  endif
                endif
              endif
              enddo
              enddo
#ifdef SUBPROCESSES
#endif
            elseif((ssub(generator).eq.1.or.ssub(generator).eq.2)
     &            .and.sfactqed.gt.0.and.sfactqcd.gt.0)then
              if(cut(phasespace).eq.0.or.cut(1).eq.0)then

c                write(*,*) 'integrand spp',phasespace,
c     &              s(1,2,phasespace),shat,spp,s(1,2,1)

                if(lgencon(0,3))then
                  if(sfactqed.eq.1)then
                    fp=sfx(em)*(log(spp/qedfacscale2)+2d0*log(omx))
                  elseif(sfactqed.eq.2)then
                    fp=sfx(em)*(log(spp/qedfacscale2)
     &                  +log(omx*x)+.75d0)-2.25d0-1.25d0*x
                  endif
                endif
                if(lgencon(0,4))then
                  if(sfactqcd.eq.1)then
                    fg=sfx(em)*(log(spp/qcdfacscale2)+2d0*log(omx))
                  elseif(sfactqcd.eq.2)then
                    fg=sfx(em)*(log(spp/qcdfacscale2)
     &                  +log(omx*x)+.75d0)-2.25d0-1.25d0*x
                  endif
                endif
              endif
              if(cut(phasespace).eq.0)then
                if(lgencon(0,3))then
                  wfac=
     &              -pf*fac(phasespace)*(fp+omx)/x
                  wfacp=mcalpha*wfac*freqgencon(generator,3)
                  weight(3,phasespace)=weight(3,phasespace) +
     &                wfacp*m2pdfpdfqq_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweight(nset,3,phasespace)=
     &            mweight(nset,3,phasespace) +
     &                wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                endif
                if(lgencon(0,4))then
                  wfac=
     &              -pf*fac(phasespace)*(fg+omx)/x
                  wfacg=mcalphas*wfac*freqgencon(generator,4)
                  weight(4,phasespace)=weight(4,phasespace) +
     &                wfacg*m2pdfpdftt_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweight(nset,4,phasespace)=
     &            mweight(nset,4,phasespace) +
     &                wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  if(lgencon(0,3))then
                    weightii(3,parton1,parton2,phasespace) =
     &                  weightii(3,parton1,parton2,phasespace) +
     &                  wfacp*
     &                  m2pdfpdfiiqq_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweightii(nset,3,parton1,parton2,phasespace) =
     &                  weightii(3,parton1,parton2,phasespace) +
     &                  wfacp*
     &                  mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                  endif
                  if(lgencon(0,4))then
                    weightii(4,parton1,parton2,phasespace) =
     &                  weightii(4,parton1,parton2,phasespace) +
     &                  wfacg*
     &                  m2pdfpdfiitt_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweightii(nset,4,parton1,parton2,phasespace) =
     &                  weightii(4,parton1,parton2,phasespace) +
     &                  wfacg*
     &                  mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                  endif
                endif
                enddo
                enddo
#endif
              endif
              if(cut(1).eq.0)then
                if(lgencon(0,3))then
                  wfac=
     &              +pf*fac(1)*(fp+omx)
     &              -pf*fac(1)*(1.5d0-pi2/3d0-const(em,sp))
                  wfacp=mcalpha*wfac*freqgencon(generator,3)
                  weight(3,1)=weight(3,1) +
     &                wfacp*m2pdfpdfqq_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                  mweight(nset,3,1)=mweight(nset,3,1) +
     &                wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                endif
                if(lgencon(0,4))then
                  wfac=
     &              +pf*fac(1)*(fg+omx)
     &              -pf*fac(1)*(1.5d0-pi2/3d0-const(em,sp))
                  wfacg=mcalphas*wfac*freqgencon(generator,4)
                  weight(4,1)=weight(4,1) +
     &                wfacg*m2pdfpdftt_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                  mweight(nset,4,1)=mweight(nset,4,1) +
     &                wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  if(lgencon(0,3))then
                    weightii(3,parton1,parton2,1) =
     &                  weightii(3,parton1,parton2,1) +
     &                  wfacp
     &                  *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweightii(nset,3,parton1,parton2,1) =
     &                  weightii(3,parton1,parton2,1) +
     &                  wfacp
     &                  *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                  endif
                  if(lgencon(0,4))then
                    weightii(4,parton1,parton2,1) =
     &                  weightii(4,parton1,parton2,1) +
     &                  wfacg
     &                  *m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweightii(nset,4,parton1,parton2,1) =
     &                  weightii(4,parton1,parton2,1) +
     &                  wfacg
     &                  *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                  endif
                endif
                enddo
                enddo
#endif
              endif
#ifdef SUBPROCESSES
#endif
            endif
          endif
          enddo
          enddo
          endif
          if(lgencon(0,3))then
            rejectedcon(3,1)=cut(1)
            rejectedcon(3,2)=cut(2)
            rejectedcon(3,3)=cut(3)
            if(cut(1).eq.0.or.cut(2).eq.0.or.cut(3).eq.0) then
              rejected(3)=0
            else
              rejected(3)=max(cut(1),cut(2),cut(3))
            endif
c            rejected(3)=min(cut(1),cut(2),cut(3))
          endif
          if(lgencon(0,4))then
            rejectedcon(4,1)=cut(1)
            rejectedcon(4,2)=cut(2)
            rejectedcon(4,3)=cut(3)
            if(cut(1).eq.0.or.cut(2).eq.0.or.cut(3).eq.0) then
              rejected(4)=0
            else
              rejected(4)=max(cut(1),cut(2),cut(3))
            endif
c            rejected(4)=min(cut(1),cut(2),cut(3))

c            write(*,*) 'integrand cut04=',rejected(4)

          endif
        endif
c end subtraction terms

c        write(*,*) 'integrand weights',weight(1,1),weight(2,1),
c     &  weight(3,1),weight(4,1),weight(5,1)

c slicing terms
c        write(*,*) 'integrand: ssli ',ssli(generator),generator

        if(ssli(generator).ne.0.and.
     &      (lgencon(0,3).or.lgencon(0,4))
     &      .and.switch.eq.0) then
          if(cut(1).eq.0.or.cut(2).eq.0.or.cut(3).eq.0)then

c final-state emitter
c          do em=3,nextgen(generator)
c          do sp=1,nextgen(generator)
c ADAPTED FOR UNCHARGED HIGGS
          do em=3,nextsub(generator)
          do sp=1,nextsub(generator)
c minus sign from incoming*outgoing particles
          pf=factor(generator)*dir(em)*dir(sp)
          if(pf.ne.0d0.and.em.ne.sp)then
            if(ssli(generator).eq.1.and.sfactqed.gt.0
     &          .and.sfactqcd.gt.0)then
              if(s(em,sp,1).lt.0d0)then
                switch=48
                if(numout.lt.maxout.and.lnoutmc.ge.5)then
                  write(nout,100) nevt,cogen,cocha,switch,
     &                ' integrand: s(em,sp,1) < 0'
                 write(nout,*) 's(em,sp,1) = ',em,sp,s(em,sp,1)
                  numout=numout+1
                endif
                return
              endif
               if(cut(1).eq.0)then
c const=Cij-1/2
                f=pi*pi-4d0+const(em,sp)
     &              + dli2(1d0-4d0*ecms(em,1)*ecms(sp,1)/s(em,sp,1),
     &                           switch)
     &              + log(2d0*ecms(em,1)**2*mcslideltac/s(em,sp,1))
     &              * (1.5d0+log(mcslideltas**2*s(1,2,1)/
     &                           (4d0*ecms(em,1)**2)))
     &              + .5d0*(log(4d0*ecms(em,1)**2/s(em,sp,1)))**2
                wfac=pf*f*fac(1)
                if(lgencon(0,3))then
                  wfacp=mcalpha*wfac*freqgencon(generator,3)
                  weight(3,1)=weight(3,1)
     &                +wfacp*m2pdfpdfqq_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                  mweight(nset,3,1)=mweight(nset,3,1)
     &                +wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                endif
                if(lgencon(0,4))then
                    wfacg=mcalphas*wfac*freqgencon(generator,4)
                  weight(4,1)=weight(4,1)
     &                +wfacg*m2pdfpdftt_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                  mweight(nset,4,1)=mweight(nset,4,1)
     &                +wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  if(lgencon(0,3))then
                    weightii(3,parton1,parton2,1) =
     &                  weightii(3,parton1,parton2,1) +
     &                  wfacp*m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweightii(nset,3,parton1,parton2,1) =
     &                  weightii(3,parton1,parton2,1) +
     &                  wfacp*mm2pdfpdfiiqq_0(iref,parton1,parton2,em,
     &                  sp)
      enddo
#endif
                  endif
                  if(lgencon(0,4))then
                    weightii(4,parton1,parton2,1) =
     &                  weightii(4,parton1,parton2,1) +
     &                  wfacg*m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                    mweightii(nset,4,parton1,parton2,1) =
     &                  weightii(4,parton1,parton2,1) +
     &                  wfacg*mm2pdfpdfiitt_0(iref,parton1,parton2,em,
     &                  sp)
      enddo
#endif
                  endif
                endif
                enddo
                enddo
#endif
             endif
            endif
          endif
          enddo
          enddo

c initial-state emitter
          do em=1,2
c          do sp=3,nextgen(generator)
c ADAPTED FOR UNCHARGED HIGGS
          do sp=1,nextsub(generator)
            phasespace=em+1
            x=xisr(phasespace)
            omx=omxisr(phasespace)
            xd=xdisr(phasespace)
            omxd=omxdisr(phasespace)
c          pf=(qedfactor(em,sp,generator)+qedfactor(em,sp,generator))
c     &        *woaisr(phasespace)
            pf= factor(generator)*dir(em)*dir(sp)
            if(pf.ne.0d0.and.em.ne.sp)then
              if(s(em,sp,phasespace).lt.0d0.or.s(em,sp,1).lt.0d0)then
                switch=30
                if(numout.lt.maxout.and.lnoutmc.ge.5)then
                  write(nout,100) nevt,cogen,cocha,switch,
c                  write(*,'(" event= ",i9,",",(a))'),nevt,
     &                ' integrand: s(em,sp,...) < 0'
                  numout=numout+1
                endif
c                mcexitevt(4,generator)=mcexitevt(4,generator)+1
                return
              else
                if(ssli(generator).eq.1
     &              .and.sfactqed.gt.0.and.sfactqcd.gt.0)then
                  if(cut(phasespace).eq.0)then
                    if(lgencon(0,3))then
                      if(sfactqed.eq.1)then
                        fp=omx+sfx(em)*
     &                      (log(spp*omx*omx*mcslideltac/
     &                      (2d0*qedfacscale2)))
                      elseif(sfactqed.eq.2)then
                        fp=omx+sfx(em)*
     &                      (log(spp*omx*x*mcslideltac/
     &                  (2d0*qedfacscale2))+.75d0)
     &                      -2.25d0-1.25d0*x
                      endif
                      wfac=
     &                  -pf*fac(phasespace)*fp/x*woaisr(phasespace)
                      wfacp=mcalpha*wfac*freqgencon(generator,3)
                      weight(3,phasespace)=weight(3,phasespace)
     &                    +wfacp*m2pdfpdfqq_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweight(nset,3,phasespace)=
     &                mweight(nset,3,phasespace)
     &                    +wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                    endif
                    if(lgencon(0,4))then

c                write(*,*) 'integrand spp',phasespace,
c     &              s(1,2,phasespace),shat,spp,s(1,2,1)

                      if(sfactqcd.eq.1)then
                        fg=omx+sfx(em)*
     &                      (log(spp*omx*omx*mcslideltac/
     &                      (2d0*qcdfacscale2)))
                      elseif(sfactqcd.eq.2)then
                        fg=omx+sfx(em)*
     &                      (log(spp*omx*x*mcslideltac/
     &                      (2d0*qcdfacscale2))+.75d0)
     &                      -2.25d0-1.25d0*x
                      endif
                      wfac=
     &                  -pf*fac(phasespace)*fg/x*woaisr(phasespace)
                      wfacg=mcalphas*wfac*freqgencon(generator,4)
                      weight(4,phasespace)=weight(4,phasespace)
     &                    +wfacg*m2pdfpdftt_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweight(nset,4,phasespace)=
     &                mweight(nset,4,phasespace)
     &                    +wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
                    endif
#ifdef SUBPROCESSES
                    do parton1=-4,4
                    do parton2=-4,4
                    if (parton1*parton2.ne.0) then
                      if(lgencon(0,3))then
                        weightii(3,parton1,parton2,phasespace) =
     &                      weightii(3,parton1,parton2,phasespace) +
     &                      wfacp*m2pdfpdfiiqq_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweightii(nset,3,parton1,parton2,phasespace) =
     &                      weightii(3,parton1,parton2,phasespace) +
     &                      wfacp*mm2pdfpdfiiqq_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
                      endif
                      if(lgencon(0,4))then
                        weightii(4,parton1,parton2,phasespace) =
     &                      weightii(4,parton1,parton2,phasespace) +
     &                      wfacg*m2pdfpdfiitt_0(phasespace,
     &                      parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                        mweightii(nset,4,parton1,parton2,phasespace) =
     &                      weightii(4,parton1,parton2,phasespace) +
     &                      wfacg*mm2pdfpdfiitt_0(iref,
     &                      parton1,parton2,em,sp)
      enddo
#endif
                      endif
                    endif
                    enddo
                    enddo

#endif
                  endif
                  if(cut(1).eq.0)then
                    wfac= pf*fac(1)*(
c const=Cij-1/2
     &                  pi*pi/3d0-1.5d0+const(em,sp)
     &                  + dli2(1d0-4d0*ecms(em,1)*ecms(sp,1)/s(em,sp,1),
     &                                  switch)
     &                  + log(s(1,2,1)*mcslideltac/(2d0*s(em,sp,1)))
     &                  * (1.5d0+2d0*log(mcslideltas))
     &                  + .5d0*(log(s(1,2,1)/s(em,sp,1)))**2
     &                  )
                    if(lgencon(0,3))then
                      if(sfactqed.eq.1)then
                        fpd=omxd+sfxd(em)*
     &                      (log(s(1,2,1)*omxd*omxd*mcslideltac/
     &                      (2d0*qedfacscale2)))
                      elseif(sfactqed.eq.2)then
                        fpd=omxd+sfxd(em)*
     &                      (log(s(1,2,1)*omxd*xd*mcslideltac/
     &                      (2d0*qedfacscale2))+.75d0)
     &                      -2.25d0-1.25d0*xd
                      endif
                      wfacp=wfac+ pf*fac(1)*fpd*woaisrd(phasespace)
                      wfacp=mcalpha*wfacp*freqgencon(generator,3)
                      weight(3,1)=weight(3,1)+
     &                    wfacp*m2pdfpdfqq_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweight(nset,3,1)=mweight(nset,3,1)+
     &                    wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
                    endif
                    if(lgencon(0,4))then
                      if(sfactqcd.eq.1)then
                        fgd=omxd+sfxd(em)*
     &                      (log(s(1,2,1)*omxd*omxd*mcslideltac/
     &                      (2d0*qcdfacscale2)))
                      elseif(sfactqcd.eq.2)then
                        fgd=omxd+sfxd(em)*
     &                      (log(s(1,2,1)*omxd*xd*mcslideltac/
     &                      (2d0*qcdfacscale2))+.75d0)
     &                      -2.25d0-1.25d0*xd
                      endif
                      wfacg=wfac+ pf*fac(1)*fgd*woaisrd(phasespace)
                      wfacg=mcalphas*wfacg*freqgencon(generator,4)
                      weight(4,1)=weight(4,1)+
     &                    wfacg*m2pdfpdftt_0(1,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                      mweight(nset,4,1)=mweight(nset,4,1)+
     &                    wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif

                    endif
#ifdef SUBPROCESSES
                    do parton1=-4,4
                    do parton2=-4,4
                    if (parton1*parton2.ne.0) then
                      if(lgencon(0,3))then
                        weightii(3,parton1,parton2,1) =
     &                      weightii(3,parton1,parton2,1) +
     &                      wfacp
     &                      *m2pdfpdfiiqq_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                        mweightii(nset,3,parton1,parton2,1) =
     &                      weightii(3,parton1,parton2,1) +
     &                      wfacp
     &                      *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                      endif
                      if(lgencon(0,4))then
                        weightii(4,parton1,parton2,1) =
     &                      weightii(4,parton1,parton2,1) +
     &                      wfacg
     &                      *m2pdfpdfiitt_0(1,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                        mweightii(nset,4,parton1,parton2,1) =
     &                      weightii(4,parton1,parton2,1) +
     &                      wfacg
     &                      *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                      endif
                    endif
                    enddo
                    enddo
#endif

                  endif
                endif
              endif
            endif
          enddo
          enddo
          endif
          if(lgencon(0,3))then
            rejectedcon(3,1)=cut(1)
            rejectedcon(3,2)=cut(2)
            rejectedcon(3,3)=cut(3)
            if(cut(1).eq.0.or.cut(2).eq.0.or.cut(3).eq.0) then
              rejected(3)=0
            else
              rejected(3)=max(cut(1),cut(2),cut(3))
            endif
c            rejected(3)=max(cut(1),cut(2),cut(3))
          endif
          if(lgencon(0,4))then
            rejectedcon(4,1)=cut(1)
            rejectedcon(4,2)=cut(2)
            rejectedcon(4,3)=cut(3)
            if(cut(1).eq.0.or.cut(2).eq.0.or.cut(3).eq.0) then
              rejected(4)=0
            else
              rejected(4)=max(cut(1),cut(2),cut(3))
            endif
c            rejected(4)=max(cut(1),cut(2),cut(3))
          endif
        endif
* end slicing contributions


c end-point part of subtraction functions
        if(sendp(generator).eq.1.and.switch.eq.0)then
          if(cut(1).eq.0)then
c          do em=1,nextgen(generator)
c          do sp=em+1,nextgen(generator)
c ADAPTED FOR UNCHARGED HIGGS
          do em=1,nextsub(generator)-1
          do sp=em+1,nextsub(generator)
c          pf=qedfactor(em,sp,generator)+qcdfactor(em,sp,generator)
          pf= factor(generator)*dir(em)*dir(sp)
          if(pf.ne.0d0)then
            p2=s(em,sp,1)
            if(p2.le.0d0.and.cut(1).eq.0)then
              switch=31
              if(numout.lt.maxout.and.lnoutmc.ge.5)then
                write(nout,100) nevt,cogen,cocha,switch,
c                write(*,'(" event= ",i9,",",(a))'),nevt,
     &              ' integrand: s(em,sp,1) <= 0'
                numout=numout+1
              endif
c              mcexitevt(5,generator)=mcexitevt(5,generator)+1
              return
            elseif(cut(1).eq.0)then
              do parton1=-4,4
              do parton2=-4,4
              do parton3=-4,4
              do parton4=-4,4
              if (parton1*parton2*parton3*parton4.ne.0) then
                if(em.eq.1)then
                  mass2(em,generator) = m2parton(parton1)
                elseif(em.eq.2)then
                  mass2(em,generator) = m2parton(parton2)
                elseif(em.eq.3)then
                  mass2(em,generator) = m2parton(parton3)
                elseif(em.eq.4)then
                  mass2(em,generator) = m2parton(parton4)
                endif
                if(sp.eq.4)then
                  mass2(sp,generator) = m2parton(parton3)
                else if(sp.eq.3)then
                  mass2(sp,generator) = m2parton(parton4)
                else if(sp.eq.2)then
                  mass2(sp,generator) = m2parton(parton4)
                else if(sp.eq.1)then
                  mass2(sp,generator) = m2parton(parton4)
                else
                  write(*,*) ' integrand: mass2 for spectator > 4 ',
     &                'not defined'
                  write(*,*) nextsub(generator),em,sp
                  write(*,*) parton1,parton2,parton3,parton4
                  stop
                endif
                logs=(dlog(mass2(em,generator)/p2)
     &              +dlog(mass2(sp,generator)/p2)+2d0)
     &              *dlog(lambda2(generator)/p2)
     &              -0.5d0*dlog(mass2(em,generator)/p2)**2
     &              +0.5d0*dlog(mass2(em,generator)/p2)
     &              -0.5d0*dlog(mass2(sp,generator)/p2)**2
     &              +0.5d0*dlog(mass2(sp,generator)/p2)
                wfac=
     &              -pf*(logs+const(em,sp)+const(sp,em)+1d0)*fac(1)
                wfacp=mcalpha*wfac*freqgencon(generator,2)
                wfacg=mcalphas*wfac*freqgencon(generator,5)


                if(lgencon(0,2))then
#ifdef SUBPROCESSES
                  weightii(2,parton1,parton2,1) =
     &                weightii(2,parton1,parton2,1) +
     &                wfacp
     &                *m2pdfpdfiiiiqq_0(1,parton1,parton2,
     &                parton3,parton4,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        Write(*,*) "unexpected branch for multpdf"
        Write(*,*) "Program stops here"
        Stop
CAM        iref=(nset-1)*maxps+1
CAM                  mweightii(nset,2,parton1,parton2,1) =
CAM     &                weightii(2,parton1,parton2,1) +
CAM     &                wfacp
CAM     &                *mm2pdfpdfiiiiqq_0(iref,parton1,parton2,
CAM     &                parton3,parton4,em,sp)
      enddo
#endif
#endif
                  weight(2,1)=weight(2,1) +
     &                wfacp
     &                *m2pdfpdfiiiiqq_0(1,parton1,parton2,
     &                parton3,parton4,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        Write(*,*) "unexpected branch for multpdf"
        Write(*,*) "Program stops here"
        Stop
CAM        iref=(nset-1)*maxps+1
CAM                  mweight(nset,2,1)=mweight(nset,2,1) +
CAM     &                wfacp
CAM     &                *mm2pdfpdfiiiiqq_0(iref,parton1,parton2,
CAM     &                parton3,parton4,em,sp)
CAMc>         if(parton1.eq.-4.and.parton2.eq.-4.
CAMc>     &      .and.parton3.eq.-4.and.parton4.eq.-4) then
CAMc>          write(*,*) 'endp ',wfacp,parton3,parton4,em,sp,const(em,sp),
CAMc>     &         mm2pdfpdfiiiiqq_0(iref,parton1,parton2,
CAMc>     &              parton3,parton4,em,sp),sendp(generator)
CAMc>          write(*,*) 'endp c ',parton3,parton4,em,sp,
CAMc>     &         mm2pdfpdfiiiiqq_0(iref,parton1,parton2,
CAMc>     &              parton3,parton4,em,sp)
CAMc>     &        *wfacp/wconv*gsum(phasespace)/factor(generator)
CAMc>          write(*,*) 'endp f ',parton3,parton4,em,sp,
CAMc>     &         mm2pdfpdfiiiiqq_0(iref,parton1,parton2,
CAMc>     &              parton3,parton4,em,sp),
CAMc>     &        wfacp/wconv*gsum(phasespace)/factor(generator)/
CAMc>     &        (logs+const(em,sp)+const(sp,em)+1d0),
CAMc>     &        (logs+const(em,sp)+const(sp,em)+1d0),
CAMc>     &        logs+1d0,const(em,sp)+const(sp,em)
CAMc>         endif
      enddo
#endif
                endif
                if(lgencon(0,5))then
#ifdef SUBPROCESSES
                  weightii(5,parton1,parton2,1) =
     &                weightii(5,parton1,parton2,1) +
     &                wfacg
     &                *m2pdfpdfiiiitt_0(1,parton1,parton2,
     &                parton3,parton4,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        Write(*,*) "unexpected branch for multpdf"
        Write(*,*) "Program stops here"
        Stop
CAM        iref=(nset-1)*maxps+1
CAM                  mweightii(nset,5,parton1,parton2,1) =
CAM     &                weightii(5,parton1,parton2,1) +
CAM     &                wfacg
CAM     &                *mm2pdfpdfiiiitt_0(iref,parton1,parton2,
CAM     &                parton3,parton4,em,sp)
      enddo
#endif
#endif
                  weight(5,1)=weight(5,1) +
     &                wfacg
     &                *m2pdfpdfiiiitt_0(1,parton1,parton2,
     &                parton3,parton4,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        Write(*,*) "unexpected branch for multpdf"
        Write(*,*) "Program stops here"
        Stop
CAM        iref=(nset-1)*maxps+1
CAM                  mweight(nset,5,1)=mweight(nset,5,1) +
CAM     &                wfacg
CAM     &                *mm2pdfpdfiiiitt_0(iref,parton1,parton2,
CAM     &                parton3,parton4,em,sp)
      enddo
#endif
                endif
              endif
              enddo
              enddo
              enddo
              enddo
            endif
          endif
          enddo
          enddo
#ifdef SUBPROCESSES
#endif

           endif
         endif

c incoming photon and gluon subtraction terms

c       write(*,*) 'integrand: ssub ',ssub(generator),generator

        if(ssub(generator).ne.0.and.
     &       (lgencon(generator,6).or.lgencon(generator,7).or.
     &       lgencon(generator,8).or.lgencon(generator,9))
     &       .and.switch.eq.0) then
          if (cut(4).eq.0.or.cut(5).eq.0)then

c initial-state emitter with initial-state spectator
          do em=1,2
            phasespace=em+3
            x=xisr(phasespace)
            omx=omxisr(phasespace)
c          pf=(qedfactor(em,sp,generator)+qedfactor(em,sp,generator))
c     &        *woaisr(phasespace)
            pf=factor(generator) * woaisr(phasespace)
            if(pf.ne.0d0)then

            if((ssub(generator).eq.1.or.ssub(generator).eq.2)
     &            .and.sfactqed.gt.0.and.sfactqcd.gt.0)then
              if(cut(phasespace).eq.0)then
                if(lgencon(0,6).or.lgencon(0,7))then
                  if(sfactqed.eq.1)then
                    fp=(omx*omx+x*x)*
     &                  log(spp*omx*omx/qedfacscale2)
     &                  +2d0*x*omx
                  elseif(sfactqed.eq.2)then
                    fp=(omx*omx+x*x)*
     &                  log(spp*omx*x/qedfacscale2)
     &                  +1d0-6d0*x*omx
                  endif
                  wfac=
     &              pf*fac(phasespace)*fp/x
                  wfacp=mcalpha*wfac*freqgencon(generator,6)
                  if(em.eq.1.and.lgencon(0,6))then
                    weight(6,phasespace)=weight(6,phasespace) +
     &                  wfacp*m2pdfpdfq2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,6,phasespace)=
     &              mweight(nset,6,phasespace) +
     &                  wfacp*mm2pdfpdfq2_0(iref,em)
      enddo
#endif
                  elseif(em.eq.2.and.lgencon(0,7))then
                    weight(7,phasespace)=weight(7,phasespace) +
     &                  wfacp*m2pdfpdfq2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,7,phasespace)=
     &              mweight(nset,7,phasespace) +
     &                  wfacp*mm2pdfpdfq2_0(iref,em)
      enddo
#endif
                  endif
                endif
                if(lgencon(0,8).or.lgencon(0,9))then
                  if(sfactqcd.eq.1)then
                    fg=(omx*omx+x*x)*
     &                  log(spp*omx*omx/qcdfacscale2)
     &                  +2d0*x*omx
                  elseif(sfactqcd.eq.2)then
                    fg=(omx*omx+x*x)*
     &                  log(spp*omx*x/qcdfacscale2)
     &                  +1d0-6d0*x*omx
                  endif
                  wfac=
     &              pf*fac(phasespace)*fg/x
                  wfacg=mcalphas*wfac*freqgencon(generator,8)
                  if(em.eq.1.and.lgencon(0,8))then
                    weight(8,phasespace)=weight(8,phasespace) +
     &                  wfacg*m2pdfpdft2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,8,phasespace)=
     &              mweight(nset,8,phasespace) +
     &                  wfacg*mm2pdfpdft2_0(iref,em)
      enddo
#endif
                  elseif(em.eq.2.and.lgencon(0,9))then
                    weight(9,phasespace)=weight(9,phasespace) +
     &                  wfacg*m2pdfpdft2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,9,phasespace)=
     &              mweight(nset,9,phasespace) +
     &                  wfacg*mm2pdfpdft2_0(iref,em)
      enddo
#endif
                  endif
                endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                if (parton1.ne.0) then
                    if(em.eq.1.and.lgencon(0,6))then
                      weightii(6,0,parton1,phasespace) =
     &                    weightii(6,0,parton1,phasespace) +
     &                    wfacp*
     &                    m2pdfpdfiiq2_0(phasespace,0,parton1,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,6,0,parton1,phasespace) =
     &                    weightii(6,0,parton1,phasespace) +
     &                    wfacp*
     &                    mm2pdfpdfiiq2_0(iref,0,parton1,em)
      enddo
#endif
                    elseif(em.eq.2.and.lgencon(0,7))then
                      weightii(7,parton1,0,phasespace) =
     &                    weightii(7,parton1,0,phasespace) +
     &                    wfacp*
     &                    m2pdfpdfiiq2_0(phasespace,parton1,0,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,7,parton1,0,phasespace) =
     &                    weightii(7,parton1,0,phasespace) +
     &                    wfacp*
     &                    mm2pdfpdfiiq2_0(iref,parton1,0,em)
      enddo
#endif
                    endif
                    if(em.eq.1.and.lgencon(0,8))then
                      weightii(8,0,parton1,phasespace) =
     &                    weightii(8,0,parton1,phasespace) +
     &                    wfacg*
     &                    m2pdfpdfiit2_0(phasespace,0,parton1,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,8,0,parton1,phasespace) =
     &                    weightii(8,0,parton1,phasespace) +
     &                    wfacg*
     &                    mm2pdfpdfiit2_0(iref,0,parton1,em)
      enddo
#endif
                    elseif(em.eq.2.and.lgencon(0,9))then
                      weightii(9,parton1,0,phasespace) =
     &                    weightii(9,parton1,0,phasespace) +
     &                    wfacg*
     &                    m2pdfpdfiit2_0(phasespace,parton1,0,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,9,parton1,0,phasespace) =
     &                    weightii(9,parton1,0,phasespace) +
     &                    wfacg*
     &                    mm2pdfpdfiit2_0(iref,parton1,0,em)
      enddo
#endif
                    endif
                endif
                enddo
#endif
              endif
            endif
          endif
          enddo
          endif
          if(lgencon(0,6))then
            rejectedcon(6,4)=cut(4)
            rejected(6)=cut(4)
          endif
          if(lgencon(0,7))then
            rejectedcon(7,5)=cut(5)
            rejected(7)=cut(5)
          endif
          if(lgencon(0,8))then
            rejectedcon(8,4)=cut(4)
            rejected(8)=cut(4)
          endif
          if(lgencon(0,9))then
            rejectedcon(9,5)=cut(5)
            rejected(9)=cut(5)
          endif
        endif

c        write(*,*) 'integrand ingasli',generator,ssli(generator)
c     &      ,sgencon(generator,6),sgencon(generator,7)

c incoming photon and gluon slicing terms

c       write(*,*) 'integrand: ssub ',ssub(generator),generator

        if(ssli(generator).ne.0.and.
     &      (lgencon(generator,6).or.lgencon(generator,7).or.
     &       lgencon(generator,8).or.lgencon(generator,9))
     &      .and.switch.eq.0)then
          if(cut(4).eq.0.or.cut(5).eq.0)then

c initial-state emitter with initial-state spectator
          do em=1,2
          phasespace=em+3
          x=xisr(phasespace)
          omx=omxisr(phasespace)
c          pf=(qedfactor(em,sp,generator)+qedfactor(em,sp,generator))
c     &        *woaisr(phasespace)
          pf=factor(generator) * woaisr(phasespace)
          if(pf.ne.0d0)then

            if(ssli(generator).eq.1
     &            .and.sfactqed.gt.0.and.sfactqcd.gt.0)then
              if(cut(phasespace).eq.0)then
                if(lgencon(0,6).or.lgencon(0,7))then
                  if(sfactqed.eq.1)then
                    fp=(omx*omx+x*x)*
     &                  log(spp*omx*omx*mcslideltac/
     &                  (2d0*qedfacscale2))
     &                  +2d0*x*omx
                  elseif(sfactqed.eq.2)then
                    fp=(omx*omx+x*x)*
     &                  log(spp*omx*x*mcslideltac/(2d0*qedfacscale2))
     &                  +1d0-6d0*x*omx
                  endif
                  wfac=
     &              pf*fac(phasespace)*fp/x
                  wfacp=mcalpha*wfac*freqgencon(generator,6)
                  if(em.eq.1.and.lgencon(0,6))then
                    weight(6,phasespace)=weight(6,phasespace) +
     &                  wfacp*m2pdfpdfq2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,6,phasespace)=
     &              mweight(nset,6,phasespace) +
     &                  wfacp*mm2pdfpdfq2_0(iref,em)
      enddo
#endif
                  elseif(em.eq.2.and.lgencon(0,7))then
                    weight(7,phasespace)=weight(7,phasespace) +
     &                  wfacp*m2pdfpdfq2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,7,phasespace)=
     &              mweight(nset,7,phasespace) +
     &                  wfacp*mm2pdfpdfq2_0(iref,em)
      enddo
#endif
                  endif
                endif
                if(lgencon(0,8).or.lgencon(0,9))then
                  if(sfactqcd.eq.1)then
                    fg=(omx*omx+x*x)*
     &                  log(spp*omx*omx*mcslideltac/
     &                  (2d0*qcdfacscale2))
     &                  +2d0*x*omx
                  elseif(sfactqcd.eq.2)then
                    fg=(omx*omx+x*x)*
     &                  log(spp*omx*x*mcslideltac/(2d0*qcdfacscale2))
     &                  +1d0-6d0*x*omx
                  endif
                  wfac=
     &              pf*fac(phasespace)*fg/x
                  wfacg=mcalphas*wfac*freqgencon(generator,8)
                  if(em.eq.1.and.lgencon(0,8))then
                    weight(8,phasespace)=weight(8,phasespace) +
     &                  wfacg*m2pdfpdft2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,8,phasespace)=
     &              mweight(nset,8,phasespace) +
     &                  wfacg*mm2pdfpdft2_0(iref,em)
      enddo
#endif
                  elseif(em.eq.2.and.lgencon(0,9))then
                    weight(9,phasespace)=weight(9,phasespace) +
     &                  wfacg*m2pdfpdft2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,9,phasespace)=
     &              mweight(nset,9,phasespace) +
     &                  wfacg*mm2pdfpdft2_0(iref,em)
      enddo
#endif
                  endif
                endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                if (parton1.ne.0) then
                    if(em.eq.1.and.lgencon(0,6))then
                      weightii(6,0,parton1,phasespace) =
     &                    weightii(6,0,parton1,phasespace) +
     &                    wfacp*
     &                    m2pdfpdfiiq2_0(phasespace,0,parton1,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,6,0,parton1,phasespace) =
     &                    weightii(6,0,parton1,phasespace) +
     &                    wfacp*
     &                    mm2pdfpdfiiq2_0(iref,0,parton1,em)
      enddo
#endif
                    elseif(em.eq.2.and.lgencon(0,7))then
                      weightii(7,parton1,0,phasespace) =
     &                    weightii(7,parton1,0,phasespace) +
     &                    wfacp*
     &                    m2pdfpdfiiq2_0(phasespace,parton1,0,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,7,parton1,0,phasespace) =
     &                    weightii(7,parton1,0,phasespace) +
     &                    wfacp*
     &                    mm2pdfpdfiiq2_0(iref,parton1,0,em)
      enddo
#endif
                    endif
                    if(em.eq.1.and.lgencon(0,8))then
                      weightii(8,0,parton1,phasespace) =
     &                    weightii(8,0,parton1,phasespace) +
     &                    wfacg*
     &                    m2pdfpdfiit2_0(phasespace,0,parton1,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,8,0,parton1,phasespace) =
     &                    weightii(8,0,parton1,phasespace) +
     &                    wfacg*
     &                    mm2pdfpdfiit2_0(iref,0,parton1,em)
      enddo
#endif
                    elseif(em.eq.2.and.lgencon(0,9))then
                      weightii(9,parton1,0,phasespace) =
     &                    weightii(9,parton1,0,phasespace) +
     &                    wfacg*
     &                    m2pdfpdfiit2_0(phasespace,parton1,0,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,9,parton1,0,phasespace) =
     &                    weightii(9,parton1,0,phasespace) +
     &                    wfacg*
     &                    mm2pdfpdfiit2_0(iref,parton1,0,em)
      enddo
#endif
                    endif
                endif
                enddo
#endif
              endif
            endif
          endif
          enddo
          endif
          if(lgencon(0,6))then
            rejectedcon(6,4)=cut(4)
            rejected(6)=cut(4)
          endif
          if(lgencon(0,7))then
            rejectedcon(7,5)=cut(5)
            rejected(7)=cut(5)
          endif
          if(lgencon(0,8))then
            rejectedcon(8,4)=cut(4)
            rejected(8)=cut(4)
          endif
          if(lgencon(0,9))then
            rejectedcon(9,5)=cut(5)
            rejected(9)=cut(5)
          endif
        endif

c incoming photon and gluon convolution terms for eff. coll. factors

c       write(*,*) 'integrand: seff ',seff(generator),generator

        if(seff(generator).ne.0.and.
     &      (lgencon(generator,6).or.lgencon(generator,7).or.
     &       lgencon(generator,8).or.lgencon(generator,9))
     &      .and.switch.eq.0)then
          if(cut(4).eq.0.or.cut(5).eq.0)then

c initial-state emitter with initial-state spectator
          do em=1,2
          phasespace=em+3
          x=xisr(phasespace)
          omx=omxisr(phasespace)
c          pf=(qedfactor(em,sp,generator)+qedfactor(em,sp,generator))
c     &        *woaisr(phasespace)
          pf=factor(generator) * woaisr(phasespace)
          if(pf.ne.0d0)then

            if(seff(generator).eq.1
     &            .and.sfactqed.gt.0.and.sfactqcd.gt.0)then
              if(lgencon(0,6).or.lgencon(0,7))then
                if(sfactqed.eq.1)then
                  fp=(omx*omx+x*x)*
     &                log(mcollreg2/qedfacscale2)
                elseif(sfactqed.eq.2)then
                  fp=(omx*omx+x*x)*
     &                log(mcollreg2*x/(omx*qedfacscale2))
     &                +1d0-8d0*x*omx
                endif
              endif
              if(lgencon(0,8).or.lgencon(0,9))then
                if(sfactqcd.eq.1)then
                  fg=(omx*omx+x*x)*
     &                log(mcollreg2/qcdfacscale2)
                elseif(sfactqcd.eq.2)then
                  fg=(omx*omx+x*x)*
     &                log(mcollreg2*x/(omx*qcdfacscale2))
     &                +1d0-8d0*x*omx
                endif
              endif
              if(cut(phasespace).eq.0)then
                if(lgencon(0,6).or.lgencon(0,7))then
                  wfac=fp*pf*fac(phasespace)/x
                  wfacp=mcalpha*wfac*freqgencon(generator,6)
                  if(em.eq.1.and.lgencon(0,6))then
                    weight(6,phasespace)=weight(6,phasespace) +
     &                  wfacp*m2pdfpdfq2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,6,phasespace)=
     &              mweight(nset,6,phasespace) +
     &                  wfacp*mm2pdfpdfq2_0(iref,em)
      enddo
#endif
                  elseif(em.eq.2.and.lgencon(0,7))then
                    weight(7,phasespace)=weight(7,phasespace) +
     &                  wfacp*m2pdfpdfq2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,7,phasespace)=
     &              mweight(nset,7,phasespace) +
     &                  wfacp*mm2pdfpdfq2_0(iref,em)
      enddo
#endif
                  endif
                endif
                if(lgencon(0,8).or.lgencon(0,9))then
                  wfac=fg*pf*fac(phasespace)/x
                  wfacg=mcalphas*wfac*freqgencon(generator,8)
                  if(em.eq.1.and.lgencon(0,8))then
                    weight(8,phasespace)=weight(8,phasespace) +
     &                  wfacg*m2pdfpdft2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,8,phasespace)=
     &              mweight(nset,8,phasespace) +
     &                  wfacg*mm2pdfpdft2_0(iref,em)
      enddo
#endif
                  elseif(em.eq.2.and.lgencon(0,9))then
                    weight(9,phasespace)=weight(9,phasespace) +
     &                  wfacg*m2pdfpdft2_0(phasespace,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                    mweight(nset,9,phasespace)=
     &              mweight(nset,9,phasespace) +
     &                  wfacg*mm2pdfpdft2_0(iref,em)
      enddo
#endif
                  endif
                endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                if (parton1.ne.0) then
                    if(em.eq.1.and.lgencon(0,6))then
                      weightii(6,0,parton1,phasespace) =
     &                    weightii(6,0,parton1,phasespace) +
     &                    wfacp*
     &                    m2pdfpdfiiq2_0(phasespace,0,parton1,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,6,0,parton1,phasespace) =
     &                    weightii(6,0,parton1,phasespace) +
     &                    wfacp*
     &                    mm2pdfpdfiiq2_0(iref,0,parton1,em)
      enddo
#endif
                    elseif(em.eq.2.and.lgencon(0,7))then
                      weightii(7,parton1,0,phasespace) =
     &                    weightii(7,parton1,0,phasespace) +
     &                    wfacp*
     &                    m2pdfpdfiiq2_0(phasespace,parton1,0,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,7,parton1,0,phasespace) =
     &                    weightii(7,parton1,0,phasespace) +
     &                    wfacp*
     &                    mm2pdfpdfiiq2_0(iref,parton1,0,em)
      enddo
#endif
                    endif
                    if(em.eq.1.and.lgencon(0,8))then
                      weightii(8,0,parton1,phasespace) =
     &                    weightii(8,0,parton1,phasespace) +
     &                    wfacg*
     &                    m2pdfpdfiit2_0(phasespace,0,parton1,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,8,0,parton1,phasespace) =
     &                    weightii(8,0,parton1,phasespace) +
     &                    wfacg*
     &                    mm2pdfpdfiit2_0(iref,0,parton1,em)
      enddo
#endif
                    elseif(em.eq.2.and.lgencon(0,9))then
                      weightii(9,parton1,0,phasespace) =
     &                    weightii(9,parton1,0,phasespace) +
     &                    wfacg*
     &                    m2pdfpdfiit2_0(phasespace,parton1,0,em)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                      mweightii(nset,9,parton1,0,phasespace) =
     &                    weightii(9,parton1,0,phasespace) +
     &                    wfacg*
     &                    mm2pdfpdfiit2_0(iref,parton1,0,em)
      enddo
#endif
                    endif
                endif
                enddo
#endif
              endif
            endif
          endif
          enddo
          endif
          if(lgencon(0,6))then
            rejectedcon(6,4)=cut(4)
            rejected(6)=cut(4)
          endif
          if(lgencon(0,7))then
            rejectedcon(7,5)=cut(5)
            rejected(7)=cut(5)
          endif
          if(lgencon(0,8))then
            rejectedcon(8,4)=cut(4)
            rejected(8)=cut(4)
          endif
          if(lgencon(0,9))then
            rejectedcon(9,5)=cut(5)
            rejected(9)=cut(5)
          endif
        endif

c        write(*,*) 'integrand weights',weight(1,1),weight(2,1),
c     &  weight(3,1),weight(4,1),weight(5,1)

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     tree-level process and                                      c
c     subtraction terms for real photonic corrections             c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      write(*,*) 'integrand ',generator,sgen(generator)
      else if(generator.eq.1.and.sgen(generator).ne.0
     &      .and.mod(nevt,iNumProcs*freqgencon(generator,1)).lt.
     &               iNumProcs)then

c        write(*,*) 'integrand branch for gen=1'

        ncontr(generator)=1
        phasespace=1
        do vcomp=0,3
          kbeam(1,vcomp)=xconv(1)*pbeam(1,vcomp)
          kbeam(2,vcomp)=xconv(2)*pbeam(2,vcomp)
        enddo
        shat=xconv(1)*xconv(2)*energy*energy
c        write(*,*) ' integrand: xc  =',(xconv(vcomp),vcomp=1,2)
c        write(*,*) ' integrand: pb1 =',(pbeam(1,vcomp),vcomp=0,3)
c        write(*,*) ' integrand: pb2 =',(pbeam(2,vcomp),vcomp=0,3)
c        write(*,*) ' integrand: kb1 =',(kbeam(1,vcomp),vcomp=0,3)
c        write(*,*) ' integrand: kb2 =',(kbeam(2,vcomp),vcomp=0,3)
c generate momenta
c        write(*,*) 'integrand'

c phase space point for testing
           call generate_momenta(random(9),kbeam,k,kcms,
     &         g(1,phasespace),channel,generator,switch)
c           call generation(random(9),kbeam,k,g(1,phasespace),
c     &         channel,generator,switch)

#ifdef PRINTWEIGHTMAX
           if(lprintweightmax)then
             do vcomp=0,3
               do ext=1,nextgen(generator)
                 pprint(ext,vcomp)=k(ext,vcomp)
               enddo
             enddo
           endif
#endif

c        write(*,*) 'integrand switch',switch
c                do ext=1,nextgen(generator)
c                  do vcomp=0,3
c                    write(*,100) ext,vcomp,k(ext,vcomp)
c                  enddo
c                enddo
c 100        !    format('integrand:',' k(',i1,',',i1,')    = ',d20.14)

c        write(*,*) 'integrand: gen=1 bef subtraction cuts'

c technical cuts for subtraction branch
        if(ssub(generator).ne.0)then
          call dets(k,sk,nextgen(generator),switch)


          do ext=1,nextsub(generator)
             if (mccharge(ext,generator).ne.0) then

c              write(*,*) 'integrand: gen=1  techcuts 1',
c     &             ext,sk(ext,nextgen(generator)),tcdeltas

              if(sk(ext,nextgen(generator)).lt.sk(1,2)*tcdeltas)then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
              kk=k(ext,1)*k(nextgen(generator),1)
     &          +k(ext,2)*k(nextgen(generator),2)
     &          +k(ext,3)*k(nextgen(generator),3)
              kvec=sqrt(k(ext,1)**2+k(ext,2)**2+k(ext,3)**2)

c              write(*,*) 'integrand techcuts 2',
c     &             ext,kk,tcdeltac*kvec*k(nextgen(generator),0)

              if(kk.gt.tcdeltac*kvec*k(nextgen(generator),0))then
                rejectedcon(1,phasespace)=-ext-nextgen(generator)
                rejected(1)=-ext-nextgen(generator)
                return
              endif
            endif
          enddo
          if(k(nextgen(generator),0).le.tcdeltae*sqrt(sk(1,2)))then
            rejectedcon(1,phasespace)=-nextgen(generator)
            rejected(1)=-nextgen(generator)
            return
          endif

c         write(*,*) 'integrand techcuts 3',
c     &             k(nextgen(generator),0),tcdeltae

        endif

c        write(*,*) 'integrand bef slicing cuts'

c slicing cuts
        if (ssli(generator).ne.0) then

c     boost momenta to cms
          do i1=0,3
            q(i1)=k(1,i1)+k(2,i1)
          enddo
c     write(*,*) 'integrand: q ',(q(vcomp),vcomp=0,3)
          do ext=1,nextgen(generator)
c     write(*,*) 'integrand: ext,k ',ext,(k(ext,vcomp),vcomp=0,3)
            do vcomp=0,3
              q1(vcomp)=k(ext,vcomp)
            enddo
c     write(*,*) 'integrand: q1 ',(q1(vcomp),vcomp=0,3)
            call boost(shat,q,q1,1d0,switch)
            do vcomp=0,3
              kcms(ext,vcomp)=q1(vcomp)
            enddo
c     write(*,*)'integrand: kcms ',ext,(kcms(ext,vcomp),vcomp=0,3)
            ecms(ext,phasespace)=kcms(ext,0)
          enddo
          do ext=1,nextsub(generator)
            if (mccharge(ext,generator).ne.0) then
              kk=kcms(ext,1)*kcms(nextgen(generator),1)
     &            +kcms(ext,2)*kcms(nextgen(generator),2)
     &            +kcms(ext,3)*kcms(nextgen(generator),3)
              kvec=sqrt(kcms(ext,1)**2+kcms(ext,2)**2
     &            +kcms(ext,3)**2)
              if(kk.gt.(1d0-mcslideltac)*kvec*
     &            kcms(nextgen(generator),0))then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
            endif
          enddo
          if(kcms(nextgen(generator),0).le.
     &        mcslideltas*kcms(1,0))then
            rejectedcon(1,phasespace)=-nextgen(generator)
            rejected(1)=-nextgen(generator)
            return
          endif
        endif

c        write(*,*) 'integrand after slicing cuts'

c recombination
        call recombination(pbeam,k,krec,generator,nextgen(generator),
     &    cut(phasespace),switch)

        if(switch.eq.0)then
          rejectedcon(1,phasespace)=cut(phasespace)
          rejected(1)=cut(phasespace)
        endif
c        write(*,*) 'integrand: afterrec',cut(1),switch

c if event not cutted ...
c        if((sincludecuts(generator).eq.0.or.cut(phasespace).eq.0).and.
c     &    switch.eq.0)then
c changed AD 10.11.06: density needed if subtraction terms not cutted!
        if(switch.eq.0)then
c calculate densities
          call density(k,g,generator,switch)

c          write(*,*) 'integrand gsum',gsum(phasespace)

c        write(*,*) 'integrand: afterrdens',switch
          if(switch.eq.0)then
            do cha=1,nchannel(generator)
c ADDED for testing AD 6.04.06
              gsum(phasespace)=gsum(phasespace)+g(cha,phasespace)
     &            *alphaopt(cha,opt(generator),generator)
            enddo

c            write(*,*) 'integrand gsum',gsum(phasespace)

            gsum(phasespace)=gsum(phasespace)/freqgencon(generator,1)
c ADDED for testing AD 6.04.06
            gsumm(phasespace)=gsum(phasespace)
            if(cut(phasespace).eq.0.and.switch.eq.0)then
              call crosssection(k,xconv(1),xconv(2),
     &            generator,phasespace,0,0,switch)
              weight(1,phasespace)=m2pdfpdf_0(phasespace)*wconv
     &          *factor(generator)/gsum(phasespace)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
              mweight(nset,1,phasespace)=mm2pdfpdf_0(iref)*wconv
     &          *factor(generator)/gsum(phasespace)
      enddo
#endif

c              write(*,*) 'integrand weight(1,1)',
c     &            weight(1,phasespace),m2pdfpdf_0(phasespace),wconv,
c     &          factor(generator),gsum(phasespace)

#ifdef SUBPROCESSES
              do parton1=-4,4
              do parton2=-4,4
              if (parton1*parton2.ne.0) then
                weightii(1,parton1,parton2,phasespace)
     &              =m2pdfpdfii_0(1,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                mweightii(nset,1,parton1,parton2,phasespace)
     &              =mm2pdfpdfii_0(iref,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
      enddo
#endif
              endif
              enddo
              enddo
#endif


              do ext=1,nextgen(generator)
              do vcomp=0,3
                pevent(ext,vcomp,phasespace)=krec(ext,vcomp)
              enddo
              enddo
            endif
          endif
        endif

c        write(*,*) 'integrand: ssub= ',ssub(generator),switch

c subtraction terms
        if(ssub(generator).ne.0.and.switch.eq.0)then
          co=nextgen(generator)
c final-state emitter with final-state spectator
c          do em=3,nextgen(generator)-1
c          do sp=3,nextgen(generator)-1
c ADAPTED FOR UNCHARGED HIGGS
          do em=3,nextsub(generator)
          do sp=3,nextsub(generator)
            pf= mcalpha*factor(generator)

c            write(*,*) 'integrand: pf =',pf,em,sp
c            write(*,*) 'integrand: gen =',generator,
c     &          nextgen(generator),em,sp

          if(pf.ne.0d0.and.em.ne.sp)then
            phasespace=phasespace+1
            z=sk(em,sp)/(sk(em,sp)+sk(sp,co))
            y=sk(em,co)/(sk(em,sp)+sk(em,co)+sk(sp,co))
            do vcomp=0,3
              do ext=1,nextgen(generator)
                pevent(ext,vcomp,phasespace)=k(ext,vcomp)
              enddo
              pevent(co,vcomp,phasespace)=0d0
            enddo
c massless case
            if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
              do vcomp=0,3
                pevent(em,vcomp,phasespace)=k(em,vcomp)+k(co,vcomp)
     &              -y/(1d0-y)*k(sp,vcomp)
                pevent(sp,vcomp,phasespace)=1d0/(1d0-y)*k(sp,vcomp)
              enddo
c subtraction for massive case
            endif

            if(sexcl.eq.0)then
c "inclusive" subtraction terms
c              call applycut(pbeam,pevent(1,0,phasespace),generator,
c     &          nextgen(generator)-1,cut(phasespace),switch)
              call recombination(pbeam,pevent(1,0,phasespace),krec,
     &            generator,nextgen(generator)-1,cut(phasespace),switch)
            else
c "exclusive" subtraction terms
              write(*,*) 'integrand: sexcl = 1 never used'
              write(*,*) 'integrand: check implementation!'
              stop
              do vcomp=0,3
                pevent(co,vcomp,phasespace)=
     &              (1d0-z)*pevent(em,vcomp,phasespace)
                pevent(em,vcomp,phasespace)=
     &              z*pevent(em,vcomp,phasespace)
              enddo
              call recombination(pbeam,pevent(1,0,phasespace),krec,
     &          generator,nextgen(generator),cut(phasespace),switch)
              do ext=1,nextgen(generator)
              do vcomp=0,3
                pevent(ext,vcomp,phasespace)=krec(ext,vcomp)
              enddo
              enddo
            endif
            if(switch.eq.0)then
              if(cut(phasespace).eq.0) then
                rejected(1)=0
              elseif(rejected(1).gt.0
     &              .and.cut(phasespace).gt.rejected(1))then
                rejected(1)=cut(phasespace)
              endif
c              rejected(1)=min(rejected(1),cut(phasespace))
              rejectedcon(1,phasespace)=cut(phasespace)
            endif

c determine subtraction terms
            if(cut(phasespace).eq.0)then
              call crosssection(pevent(1,0,phasespace),
     &            xconv(1),xconv(2),
     &            generator,phasespace,em,sp,switch)
              if(ssub(generator).eq.1)then
                wfacp=8d0*pi*wconv*pf/gsum(1)
     &            *(2d0/(1d0-z*(1d0-y))-(1d0+z))/(1d0-y)/sk(em,co)
                weight(1,phasespace)=
     &              wfacp*m2pdfpdfqq_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacp
     &                *m2pdfpdfiiqq_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacp
     &                *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                endif
                enddo
                enddo
#endif

              elseif(ssub(generator).eq.2)then
                wfacp=8d0*pi*wconv*pf /gsum(1)
     &            *(2d0/(1d0-z*(1d0-y))-(1d0+z))/sk(em,co)
                weight(1,phasespace)=
     &              wfacp*m2pdfpdfqq_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacp
     &                *m2pdfpdfiiqq_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacp
     &                *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                endif
                enddo
                enddo
#endif
              endif
            endif
          endif
c            write(*,*) 'integrand: weight =',
c     &      weight(phasespace),phasespace,generator,em,sp
          enddo
          enddo

c final-state emitter with initial-state spectator
c          do em=3,nextgen(generator)-1
c ADAPTED FOR UNCHARGED HIGGS
          do em=3,nextsub(generator)
          do sp=1,2
c minus sign from incoming*outgoing particles
            pf= -mcalpha*factor(generator)
          if(pf.ne.0d0)then
            phasespace=phasespace+1
            x=(sk(em,sp)+sk(sp,co)-sk(em,co))/(sk(em,sp)+sk(sp,co))
            z=sk(em,sp)/(sk(em,sp)+sk(sp,co))
            do ext=1,nextgen(generator)
            do vcomp=0,3
              pevent(ext,vcomp,phasespace)=k(ext,vcomp)
            enddo
            enddo
            do vcomp=0,3
              pevent(em,vcomp,phasespace)=k(em,vcomp)+k(co,vcomp)
     &            -(1d0-x)*k(sp,vcomp)
              pevent(sp,vcomp,phasespace)=x*k(sp,vcomp)
              pevent(co,vcomp,phasespace)=0d0
            enddo

c            call applycut(pbeam,pevent(1,0,phasespace),generator,
c     &        nextgen(generator)-1,cut(phasespace),switch)
            call recombination(pbeam,pevent(1,0,phasespace),krec,
     &          generator,nextgen(generator)-1,cut(phasespace),switch)
            if(switch.eq.0)then
              rejectedcon(1,phasespace)=cut(phasespace)
              if(cut(phasespace).eq.0) then
                rejected(1)=0
              elseif(rejected(1).gt.0
     &              .and.cut(phasespace).gt.rejected(1))then
                rejected(1)=cut(phasespace)
              endif
c              rejected(1)=min(rejected(1),cut(phasespace))
            endif
            if(cut(phasespace).eq.0)then
              call crosssection(pevent(1,0,phasespace),
     &            xconv(1),xconv(2),
     &            generator,phasespace,em,sp,switch)
              if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
                wfacp=8d0*pi*wconv*pf
     &            *( (2d0/(2d0-x-z)-1d0-z)/x/sk(em,co)
c initial-state emitter with final-state spectator (em <-> sp)
     &              +(2d0/(2d0-x-z)-1d0-x)/x/sk(sp,co))
     &            /gsum(1)
                weight(1,phasespace)=
     &              wfacp*m2pdfpdfqq_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacp
     &                *m2pdfpdfiiqq_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacp
     &                *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                endif
                enddo
                enddo
#endif

              endif
            endif
          endif
c            write(*,*) 'integrand: weight =',
c     &      weight(phasespace),phasespace,generator,em,sp
          enddo
          enddo

c initial-state emitter with initial-state spectator
          do em=1,2
          do sp=1,2
            pf= mcalpha*factor(generator)
          if(pf.ne.0d0.and.em.ne.sp)then
            phasespace=phasespace+1
            x=(sk(em,sp)-sk(em,co)-sk(sp,co))/sk(em,sp)
            v=sk(em,co)/sk(em,sp)
            do vcomp=0,3
              ck(vcomp)=k(em,vcomp)+k(sp,vcomp)-k(co,vcomp)
              ckt(vcomp)=x*k(em,vcomp)+k(sp,vcomp)
            enddo
            do vcomp=0,3
              do j=3,nextgen(generator)-1
                pevent(j,vcomp,phasespace)=k(j,vcomp)
     &            -((1d0+x)*sk(em,j)+2d0*sk(sp,j)-sk(j,co))
     &              /(4d0*x+v-x*v)/sk(em,sp)*(ck(vcomp)+ckt(vcomp))
     &            +(sk(em,j)+sk(sp,j)-sk(j,co))/x/sk(em,sp)*ckt(vcomp)
              enddo
              pevent(em,vcomp,phasespace)=x*k(em,vcomp)
              pevent(sp,vcomp,phasespace)=k(sp,vcomp)
              pevent(co,vcomp,phasespace)=0d0
            enddo
c            call applycut(pbeam,pevent(1,0,phasespace),generator,
c     &        nextgen(generator)-1,cut(phasespace),switch)
            call recombination(pbeam,pevent(1,0,phasespace),krec,
     &          generator,nextgen(generator)-1,cut(phasespace),switch)
            if(switch.eq.0)then
              if(cut(phasespace).eq.0) then
                rejected(1)=0
              elseif(rejected(1).gt.0
     &              .and.cut(phasespace).gt.rejected(1))then
                rejected(1)=cut(phasespace)
              endif
c              rejected(1)=min(rejected(1),cut(phasespace))
              rejectedcon(1,phasespace)=cut(phasespace)
            endif
            if(cut(phasespace).eq.0)then
              call crosssection(pevent(1,0,phasespace),
     &            xconv(1),xconv(2),
     &            generator,phasespace,em,sp,switch)
              if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
                wfacp=8d0*pi*wconv*pf
     &            *(2d0/(1d0-x)-1d0-x)/x/sk(em,co)
     &            /gsum(1)

                weight(1,phasespace)=
     &              wfacp*m2pdfpdfqq_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacp*mm2pdfpdfqq_0(iref,em,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacp
     &                *m2pdfpdfiiqq_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacp
     &                *mm2pdfpdfiiqq_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                endif
                enddo
                enddo
#endif


              endif
            endif
          endif
c            write(*,*) 'integrand: weight =',
c     &      weight(phasespace),phasespace,generator,em,sp
          enddo
          enddo
        endif

c            write(*,*) 'integrand: phasespace =',phasespace

c        nphsp(generator)=phasespace
        nphsp(generator)=9

c            write(*,*) 'integrand: ouga weights =',generator,
c     &      (weight(generator,phasespace),phasespace=1,nphsp(generator))

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     tree-level process and                                      c
c     subtraction terms for real gluonic corrections              c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if(generator.eq.2.and.sgen(generator).ne.0
     &      .and.mod(nevt,iNumProcs*freqgencon(generator,1)).lt.
     &               iNumProcs)then
        ncontr(generator) = 1

c        write(*,*) 'integrand  gen=2 branch',nevt

        phasespace=1
        do vcomp=0,3
          kbeam(1,vcomp)=xconv(1)*pbeam(1,vcomp)
          kbeam(2,vcomp)=xconv(2)*pbeam(2,vcomp)
        enddo
        shat=xconv(1)*xconv(2)*energy*energy
c        write(*,*) ' integrand: xc  =',(xconv(vcomp),vcomp=1,2)
c        write(*,*) ' integrand: pb1 =',(pbeam(1,vcomp),vcomp=0,3)
c        write(*,*) ' integrand: pb2 =',(pbeam(2,vcomp),vcomp=0,3)
c        write(*,*) ' integrand: kb1 =',(kbeam(1,vcomp),vcomp=0,3)
c        write(*,*) ' integrand: kb2 =',(kbeam(2,vcomp),vcomp=0,3)
c generate momenta
c        write(*,*) 'integrand'

c phase space point for testing
           call generate_momenta(random(9),kbeam,k,kcms,
     &         g(1,phasespace),channel,generator,switch)
c           call generation(random(9),kbeam,k,g(1,phasespace),
c     &         channel,generator,switch)


#ifdef PRINTWEIGHTMAX
           if(lprintweightmax)then
             do vcomp=0,3
               do ext=1,nextgen(generator)
                 pprint(ext,vcomp)=k(ext,vcomp)
               enddo
             enddo
           endif
#endif

c        write(*,*) 'integrand switch',switch
c                do ext=1,nextgen(generator)
c                  do vcomp=0,3
c                    write(*,100) ext,vcomp,k(ext,vcomp)
c                  enddo
c                enddo
c 100            format('integrand:',' k(',i1,',',i1,')    = ',d20.14)


c        write(*,*) 'integrand: gen=2 bef subtraction cuts'

c technical cuts for subtraction branch
        if(ssub(generator).ne.0)then
          call dets(k,sk,nextgen(generator),switch)

          do ext=1,nextsub(generator)
             if (mccharge(ext,generator).ne.0) then

              if(sk(ext,nextgen(generator)).lt.sk(1,2)*tcdeltas)then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
              kk=k(ext,1)*k(nextgen(generator),1)
     &          +k(ext,2)*k(nextgen(generator),2)
     &          +k(ext,3)*k(nextgen(generator),3)
              kvec=sqrt(k(ext,1)**2+k(ext,2)**2+k(ext,3)**2)

c              write(*,*) 'integrand techcuts 2',
c     &             ext,kk,tcdeltac*kvec*k(nextgen(generator),0)

              if(kk.gt.tcdeltac*kvec*k(nextgen(generator),0))then
                rejectedcon(1,phasespace)=-ext-nextgen(generator)
                rejected(1)=-ext-nextgen(generator)
                return
              endif
            endif
          enddo
          if(k(nextgen(generator),0).le.tcdeltae*sqrt(sk(1,2)))then
            rejectedcon(1,phasespace)=-nextgen(generator)
            rejected(1)=-nextgen(generator)
            return
          endif

c         write(*,*) 'integrand techcuts 3',
c     &             k(nextgen(generator),0),tcdeltae

        endif

c slicing cuts
        if (ssli(generator).ne.0) then

c     boost momenta to cms
          do i1=0,3
            q(i1)=k(1,i1)+k(2,i1)
          enddo
          do ext=1,nextgen(generator)
            do vcomp=0,3
              q1(vcomp)=k(ext,vcomp)
            enddo
            call boost(shat,q,q1,1d0,switch)
            do vcomp=0,3
              kcms(ext,vcomp)=q1(vcomp)
            enddo
            ecms(ext,phasespace)=kcms(ext,0)
          enddo
          do ext=1,nextsub(generator)
            if (mccharge(ext,generator).ne.0) then
              kk=kcms(ext,1)*kcms(nextgen(generator),1)
     &            +kcms(ext,2)*kcms(nextgen(generator),2)
     &            +kcms(ext,3)*kcms(nextgen(generator),3)
              kvec=sqrt(kcms(ext,1)**2+kcms(ext,2)**2
     &            +kcms(ext,3)**2)
              if(kk.gt.(1d0-mcslideltac)*kvec*
     &            kcms(nextgen(generator),0))then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
            endif
          enddo
          if(kcms(nextgen(generator),0).le.
     &        mcslideltas*kcms(1,0))then
            rejectedcon(1,phasespace)=-nextgen(generator)
            rejected(1)=-nextgen(generator)
            return
          endif
        endif

c recombination

c        write(*,*) 'integrand befrec ',generator,nextgen(generator)
c        write(*,*) 'integrand befrec ',nevt

        call recombination(pbeam,k,krec,generator,nextgen(generator),
     &    cut(phasespace),switch)


        if(switch.eq.0)then
          rejectedcon(1,phasespace)=cut(phasespace)
          rejected(1)=cut(phasespace)
c              if(nevt.eq.73) write(*,*) 'integrand rej = ',
c     &            cut(phasespace),rejected(1)
        endif
c        write(*,*) 'integrand real',phasespace,
c     &      rejected(1),rejectedcon(phasespace,1)


c if event not cutted ...
c        if((sincludecuts(generator).eq.0.or.cut(phasespace).eq.0).and.
c     &    switch.eq.0)then
c changed AD 10.11.06: density needed if subtraction terms not cutted!
        if(switch.eq.0)then
c calculate densities
          call density(k,g,generator,switch)
c        write(*,*) 'integrand: afterrdens',switch
          if(switch.eq.0)then
            do cha=1,nchannel(generator)
c ADDED for testing AD 6.04.06
              gcha(cha,phasespace)=g(cha,phasespace)
              gsum(phasespace)=gsum(phasespace)+g(cha,phasespace)
     &            *alphaopt(cha,opt(generator),generator)


            enddo

c            write(*,*) 'integrand: gsum ',gsum(phasespace),
c     &          freqgencon(generator,1)

            gsum(phasespace)=gsum(phasespace)/freqgencon(generator,1)
c ADDED for testing AD 6.04.06
            gsumm(phasespace)=gsum(phasespace)
            if(cut(phasespace).eq.0.and.switch.eq.0)then
              call crosssection(k,xconv(1),xconv(2),
     &            generator,phasespace,0,0,switch)
              weight(1,phasespace)=m2pdfpdf_0(phasespace)*wconv
     &          *factor(generator)/gsum(phasespace)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
              mweight(nset,1,phasespace)=mm2pdfpdf_0(iref)*wconv
     &          *factor(generator)/gsum(phasespace)
      enddo
#endif
#ifdef SUBPROCESSES
              do parton1=-4,4
              do parton2=-4,4
              if (parton1*parton2.ne.0) then
                weightii(1,parton1,parton2,phasespace)
     &              =m2pdfpdfii_0(1,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                mweightii(nset,1,parton1,parton2,phasespace)
     &              =mm2pdfpdfii_0(iref,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
      enddo
#endif
              endif
              enddo
              enddo
#endif


              do ext=1,nextgen(generator)
              do vcomp=0,3
                pevent(ext,vcomp,phasespace)=krec(ext,vcomp)
              enddo
              enddo
            endif
          endif
        endif
c        write(*,*) 'integrand: qcd ',weight(1,phasespace),
c     &      cut(phasespace)

c        write(*,*) 'integrand: ssub= ',ssub(generator),switch

c subtraction terms
        if(ssub(generator).ne.0.and.switch.eq.0)then
          co=nextgen(generator)
c final-state emitter with final-state spectator
c          do em=3,nextgen(generator)-1
c          do sp=3,nextgen(generator)-1
c ADAPTED FOR UNCHARGED HIGGS
          do em=3,nextsub(generator)
          do sp=3,nextsub(generator)
            pf= mcalphas*factor(generator)

c            write(*,*) 'integrand: pf ff =',pf,em,sp
c            write(*,*) 'integrand: gen =',generator,
c     &          nextgen(generator),em,sp

          if(pf.ne.0d0.and.em.ne.sp)then
            phasespace=phasespace+1
            z=sk(em,sp)/(sk(em,sp)+sk(sp,co))
            y=sk(em,co)/(sk(em,sp)+sk(em,co)+sk(sp,co))
            do vcomp=0,3
              do ext=1,nextgen(generator)
                pevent(ext,vcomp,phasespace)=k(ext,vcomp)
              enddo
              pevent(co,vcomp,phasespace)=0d0
            enddo
c massless case
            if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
              do vcomp=0,3
                pevent(em,vcomp,phasespace)=k(em,vcomp)+k(co,vcomp)
     &              -y/(1d0-y)*k(sp,vcomp)
                pevent(sp,vcomp,phasespace)=1d0/(1d0-y)*k(sp,vcomp)
              enddo
            endif

            if(sexcl.eq.0)then
c "inclusive" subtraction terms
              call recombination(pbeam,pevent(1,0,phasespace),krec,
     &            generator,nextgen(generator)-1,cut(phasespace),switch)
            else
c "exclusive" subtraction terms
              write(*,*) 'integrand: sexcl = 1 never used'
              write(*,*) 'integrand: check implementation!'
              stop
              do vcomp=0,3
                pevent(co,vcomp,phasespace)=
     &              (1d0-z)*pevent(em,vcomp,phasespace)
                pevent(em,vcomp,phasespace)=
     &              z*pevent(em,vcomp,phasespace)
              enddo
              call recombination(pbeam,pevent(1,0,phasespace),krec,
     &          generator,nextgen(generator),cut(phasespace),switch)
              do ext=1,nextgen(generator)
              do vcomp=0,3
                pevent(ext,vcomp,phasespace)=krec(ext,vcomp)
              enddo
              enddo
            endif
            if(switch.eq.0)then
              rejectedcon(1,phasespace)=cut(phasespace)
              if(cut(phasespace).eq.0) then
                rejected(1)=0
              elseif(rejected(1).gt.0
     &              .and.cut(phasespace).gt.rejected(1))then
                rejected(1)=cut(phasespace)
              endif
c              rejected(1)=min(rejected(1),cut(phasespace))

c              if(nevt.eq.73) write(*,*) 'integrand rej = ',
c     &            cut(phasespace),rejected(1)

            endif

c            write(*,*) 'integrand ff',em,sp,phasespace,
c     &          rejected(1),rejectedcon(1,phasespace)

c determine subtraction terms
            if(cut(phasespace).eq.0)then


              call crosssection(pevent(1,0,phasespace),
     &            xconv(1),xconv(2),
     &            generator,phasespace,em,sp,switch)
              if(ssub(generator).eq.1)then
                wfacg=8d0*pi*wconv*pf/gsum(1)
     &            *(2d0/(1d0-z*(1d0-y))-(1d0+z))/(1d0-y)/sk(em,co)
                weight(1,phasespace)=
     &              wfacg*m2pdfpdftt_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                  if (parton1*parton2.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacg
     &                *m2pdfpdfiitt_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacg
     &                *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                endif
                enddo
                enddo
#endif

              elseif(ssub(generator).eq.2)then
                wfacg=8d0*pi*wconv*pf /gsum(1)
     &            *(2d0/(1d0-z*(1d0-y))-(1d0+z))/sk(em,co)
                weight(1,phasespace)=
     &              wfacg*m2pdfpdftt_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacg
     &                *m2pdfpdfiitt_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacg
     &                *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                endif
                enddo
                enddo
#endif
              endif
            endif


          endif
          enddo
          enddo

c final-state emitter with initial-state spectator
c          do em=3,nextgen(generator)-1
c ADAPTED FOR UNCHARGED HIGGS
          do em=3,nextsub(generator)
          do sp=1,2

c minus sign from incoming*outgoing particles

            pf= -mcalphas*factor(generator)
          if(pf.ne.0d0)then
            phasespace=phasespace+1
            x=(sk(em,sp)+sk(sp,co)-sk(em,co))/(sk(em,sp)+sk(sp,co))
            z=sk(em,sp)/(sk(em,sp)+sk(sp,co))
            do ext=1,nextgen(generator)
            do vcomp=0,3
              pevent(ext,vcomp,phasespace)=k(ext,vcomp)
            enddo
            enddo
            do vcomp=0,3
              pevent(em,vcomp,phasespace)=k(em,vcomp)+k(co,vcomp)
     &            -(1d0-x)*k(sp,vcomp)
              pevent(sp,vcomp,phasespace)=x*k(sp,vcomp)
              pevent(co,vcomp,phasespace)=0d0
            enddo
            call recombination(pbeam,pevent(1,0,phasespace),krec,
     &          generator,nextgen(generator)-1,cut(phasespace),switch)

c              write(*,*) 'integrand if',em,sp,phasespace,
c     &            cut(phasespace)

            if(switch.eq.0)then
              rejectedcon(1,phasespace)=cut(phasespace)
              if(cut(phasespace).eq.0) then
                rejected(1)=0
              elseif(rejected(1).gt.0
     &              .and.cut(phasespace).gt.rejected(1))then
                rejected(1)=cut(phasespace)
              endif
c              rejected(1)=min(rejected(1),cut(phasespace))

c              if(nevt.eq.73) write(*,*) 'integrand rej = ',
c     &            cut(phasespace),rejected(1)

            endif
c            write(*,*) 'integrand if',em,sp,phasespace,
c     &          rejected(1),rejectedcon(1,phasespace)
            if(cut(phasespace).eq.0)then
              call crosssection(pevent(1,0,phasespace),
     &            xconv(1),xconv(2),
     &            generator,phasespace,em,sp,switch)
              if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
                wfacg=8d0*pi*wconv*pf
     &            *( (2d0/(2d0-x-z)-1d0-z)/x/sk(em,co)
c initial-state emitter with final-state spectator (em <-> sp)
     &              +(2d0/(2d0-x-z)-1d0-x)/x/sk(sp,co))
     &            /gsum(1)
                weight(1,phasespace)=
     &              wfacg*m2pdfpdftt_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacg
     &                *m2pdfpdfiitt_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacg
     &                *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                endif
                enddo
                enddo
#endif

              endif
            endif


         endif

          enddo
          enddo


c initial-state emitter with initial-state spectator
          do em=1,2
          do sp=1,2
            pf= mcalphas*factor(generator)

c            write(*,*) 'integrand: pf ii =',pf,em,sp

          if(pf.ne.0d0.and.em.ne.sp)then
            phasespace=phasespace+1
            x=(sk(em,sp)-sk(em,co)-sk(sp,co))/sk(em,sp)
            v=sk(em,co)/sk(em,sp)
            do vcomp=0,3
              ck(vcomp)=k(em,vcomp)+k(sp,vcomp)-k(co,vcomp)
              ckt(vcomp)=x*k(em,vcomp)+k(sp,vcomp)
            enddo
            do vcomp=0,3
              do j=3,nextgen(generator)-1
                pevent(j,vcomp,phasespace)=k(j,vcomp)
     &            -((1d0+x)*sk(em,j)+2d0*sk(sp,j)-sk(j,co))
     &              /(4d0*x+v-x*v)/sk(em,sp)*(ck(vcomp)+ckt(vcomp))
     &            +(sk(em,j)+sk(sp,j)-sk(j,co))/x/sk(em,sp)*ckt(vcomp)
              enddo
              pevent(em,vcomp,phasespace)=x*k(em,vcomp)
              pevent(sp,vcomp,phasespace)=k(sp,vcomp)
              pevent(co,vcomp,phasespace)=0d0
            enddo
            call recombination(pbeam,pevent(1,0,phasespace),krec,
     &          generator,nextgen(generator)-1,cut(phasespace),switch)

c              write(*,*) 'integrand ii',em,sp,phasespace,
c     &            cut(phasespace)

            if(switch.eq.0)then
              rejectedcon(1,phasespace)=cut(phasespace)
              if(cut(phasespace).eq.0) then
                rejected(1)=0
              elseif(rejected(1).gt.0
     &              .and.cut(phasespace).gt.rejected(1))then
                rejected(1)=cut(phasespace)
              endif
c              rejected(1)=min(rejected(1),cut(phasespace))
            endif

c            write(*,*) 'integrand ii',em,sp,phasespace,
c     &          rejected(1),rejectedcon(1,phasespace)

            if(cut(phasespace).eq.0)then
              call crosssection(pevent(1,0,phasespace),
     &            xconv(1),xconv(2),
     &            generator,phasespace,em,sp,switch)
              if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
                wfacg=8d0*pi*wconv*pf
     &            *(2d0/(1d0-x)-1d0-x)/x/sk(em,co)
     &            /gsum(1)

                weight(1,phasespace)=
     &              wfacg*m2pdfpdftt_0(phasespace,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacg*mm2pdfpdftt_0(iref,em,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                do parton1=-4,4
                do parton2=-4,4
                if (parton1*parton2.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacg
     &                *m2pdfpdfiitt_0(phasespace,parton1,parton2,em,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacg
     &                *mm2pdfpdfiitt_0(iref,parton1,parton2,em,sp)
      enddo
#endif
                endif
                enddo
                enddo
#endif


              endif
            endif


          endif

          enddo
          enddo

c          write(*,*) 'integrand qcdcut =',rejected(1)

        endif


c            write(*,*) 'integrand: phasespace =',phasespace

c        nphsp(generator)=phasespace
        nphsp(generator)=9



***********************************************************************
*                                                                     *
*     tree-level process and                                          *
*     subtraction terms for initial photon contributions 1            *
*                                                                     *
***********************************************************************
      else if(generator.eq.3.and.sgen(generator).ne.0
     &      .and.mod(nevt,iNumProcs*freqgencon(generator,1)).lt.
     &               iNumProcs)then
c        write(*,*) 'integrand sgen',generator,sgen(generator)
        ncontr(generator) = 1
c no real corrections for energy < energyibadpa
        phasespace=1
        do vcomp=0,3
          kbeam(1,vcomp)=xconv(1)*pbeam(1,vcomp)
          kbeam(2,vcomp)=xconv(2)*pbeam(2,vcomp)
        enddo
        shat=xconv(1)*xconv(2)*energy*energy

c generate momenta

        call generate_momenta(random(9),kbeam,k,kcms,
     &      g(1,phasespace),channel,generator,switch)
c        call generation(random(9),kbeam,k,g(1,phasespace),
c     &    channel,generator,switch)
c phase space point for testing

#ifdef PRINTWEIGHTMAX
        if(lprintweightmax)then
          do vcomp=0,3
            do ext=1,nextgen(generator)
              pprint(ext,vcomp)=k(ext,vcomp)
            enddo
          enddo
        endif
#endif

        collfact=1d0

c technical cuts for subtraction branch
        if(ssub(generator).ne.0)then
          call dets(k,sk,nextgen(generator),switch)
          do ext=3,nextgen(generator)
            if (mccharge(ext,generator).ne.0) then

c              write(*,*) 'integrand techcuts 1',
c     &             ext,sk(1,ext),sk(1,2)*tcdeltas,tcdeltas

              if(sk(1,ext).lt.sk(1,2)*tcdeltas)then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
              kk=k(ext,1)*k(1,1)
     &          +k(ext,2)*k(1,2)
     &          +k(ext,3)*k(1,3)
              kvec=sqrt(k(ext,1)**2+k(ext,2)**2+k(ext,3)**2)

c              write(*,*) 'integrand techcuts 2',
c     &             ext,kk,tcdeltac*kvec*k(1,0)

              if(kk.gt.tcdeltac*kvec*k(1,0))then
                rejectedcon(1,phasespace)=-ext-nextgen(generator)
                rejected(1)=-ext-nextgen(generator)
                return
              endif
            endif
          enddo
          if(k(1,0).le.tcdeltae*sqrt(sk(1,2)))then
            rejectedcon(1,phasespace)=-1
            rejected(1)=-1
            return
          endif

c         write(*,*) 'integrand techcuts 3',
c     &             k(nextgen(generator),0),tcdeltae

        endif
c        write(*,*) 'integrand: aftersubcuts',cut(1),switch


c slicing cuts
        if (ssli(generator).ne.0) then

c     boost momenta to cms
          do i1=0,3
            q(i1)=k(1,i1)+k(2,i1)
          enddo
          do ext=1,nextgen(generator)
            do vcomp=0,3
              q1(vcomp)=k(ext,vcomp)
            enddo
            call boost(shat,q,q1,1d0,switch)
            do vcomp=0,3
              kcms(ext,vcomp)=q1(vcomp)
            enddo
            ecms(ext,phasespace)=kcms(ext,0)
          enddo
          do ext=3,nextgen(generator)
            if (mccharge(ext,generator).ne.0) then
              kk=kcms(ext,1)*kcms(1,1)
     &            +kcms(ext,2)*kcms(1,2)
     &            +kcms(ext,3)*kcms(1,3)
              kvec=sqrt(kcms(ext,1)**2+kcms(ext,2)**2
     &            +kcms(ext,3)**2)
              if(kk.gt.(1d0-mcslideltac)*kvec*
     &            kcms(1,0))then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
c                write(*,*) 'integrand slicuts',ext,kk,kvec,kcms(1,0),
c     &         mcslideltac,(1d0-mcslideltac)*kvec*kcms(1,0)
                return
              endif
            endif
          enddo
c          if(kcms(1,0).le.mcslideltas*kcms(1,0))then
c            rejectedcon(1,phasespace)=-1
c            rejected(1)=-1
c            return
c          endif
        endif
c        write(*,*) 'integrand: afterslicuts',cut(1),switch

c input for effective collinear factors
        if (seff(generator).ne.0) then

c     boost momenta to cms
          do i1=0,3
            q(i1)=k(1,i1)+k(2,i1)
          enddo
          do ext=1,nextgen(generator)
            do vcomp=0,3
              q1(vcomp)=k(ext,vcomp)
            enddo
            call boost(shat,q,q1,1d0,switch)
            do vcomp=0,3
              kcms(ext,vcomp)=q1(vcomp)
            enddo
            ecms(ext,phasespace)=kcms(ext,0)
          enddo

          collfact=1d0
c          collfact1=1d0
          do ext=3,nextgen(generator)
            if (mccharge(ext,generator).ne.0) then
              spk=k(ext,0)*k(1,0)-k(ext,1)*k(1,1)
     &            -k(ext,2)*k(1,2)-k(ext,3)*k(1,3)

c              write(*,*) 'integrand cut',spk,mceffcut*shat

              if(spk.lt.mceffcut*shat) then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
              omz=ecms(ext,phasespace)/ecms(1,phasespace)
              omz2=omz*omz
              z2=(1d0-omz)**2
c              s2th2=(1d0-kk/(kvec*kcms(1,0)))*.5d0
              aux=2d0*omz*spk
              collfact=collfact/(omz2+z2)
     &            *aux*(mcollreg2+aux*(omz2+z2))/(mcollreg2+aux)**2

            endif
          enddo
        endif


c recombination
        call recombination(pbeam,k,krec,generator,nextgen(generator),
     &    cut(phasespace),switch)
        if(switch.eq.0)then
          rejectedcon(1,phasespace)=cut(phasespace)
          rejected(1)=cut(phasespace)
        endif
c        write(*,*) 'integrand: afterrec',cut(1),switch

c if event not cutted ...
        if(switch.eq.0)then
c calculate densities
          call density(k,g,generator,switch)

          if(switch.eq.0)then
            do cha=1,nchannel(generator)
c ADDED for testing AD 6.04.06
              gcha(cha,phasespace)=g(cha,phasespace)
              gsum(phasespace)=gsum(phasespace)+g(cha,phasespace)
     &            *alphaopt(cha,opt(generator),generator)
            enddo
            gsum(phasespace)=gsum(phasespace)/freqgencon(generator,1)
c ADDED for testing AD 6.04.06
            gsumm(phasespace)=gsum(phasespace)
            if(cut(phasespace).eq.0.and.switch.eq.0)then
              call crosssection(k,xconv(1),xconv(2),
     &            generator,phasespace,0,0,switch)
              weight(1,phasespace)=m2pdfpdf_0(phasespace)*wconv
     &          *factor(generator)/gsum(phasespace)
     &            *collfact
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
              mweight(nset,1,phasespace)=mm2pdfpdf_0(iref)*wconv
     &          *factor(generator)/gsum(phasespace)
     &            *collfact
      enddo
#endif
c              write(*,*) 'integrand ingaweight',
c     &             phasespace,weight(1,phasespace)
#ifdef SUBPROCESSES
              parton1=0
              do parton2=-4,4
              if (parton2.ne.0) then
                weightii(1,parton1,parton2,phasespace)
     &              =m2pdfpdfii_0(1,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
     &            *collfact
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                mweightii(nset,1,parton1,parton2,phasespace)
     &              =mm2pdfpdfii_0(iref,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
     &            *collfact
      enddo
#endif
              endif
              enddo

#endif

              do ext=1,nextgen(generator)
              do vcomp=0,3
                pevent(ext,vcomp,phasespace)=krec(ext,vcomp)
              enddo
              enddo
            endif
          endif
        endif

c subtraction terms

c        write(*,*) 'integrand ssub',ssub(generator),switch

        if(ssub(generator).ne.0.and.switch.eq.0)then
          last=nextgen(generator)

c initial-state emitter (=photon) with initial-state spectator
c sum over collinear final state particles
          do em=1,1
          do sp=2,2
          do co=3,nextgen(generator)
            pf= mcalpha*factor(generator)
          if(mccharge(co,generator).ne.0d0.and.em.ne.sp)then
            phasespace=phasespace+1
            x=(sk(em,sp)-sk(em,co)-sk(sp,co))/sk(em,sp)
            v=sk(em,co)/sk(em,sp)
            do vcomp=0,3
              ck(vcomp)=k(em,vcomp)+k(sp,vcomp)-k(co,vcomp)
              ckt(vcomp)=x*k(em,vcomp)+k(sp,vcomp)
            enddo
            do vcomp=0,3
              do j=3,nextgen(generator)
                pevent(j,vcomp,phasespace)=k(j,vcomp)
     &            -((1d0+x)*sk(em,j)+2d0*sk(sp,j)-sk(j,co))
     &              /(4d0*x+v-x*v)/sk(em,sp)*(ck(vcomp)+ckt(vcomp))
     &            +(sk(em,j)+sk(sp,j)-sk(j,co))/x/sk(em,sp)*ckt(vcomp)
              enddo
              pevent(em,vcomp,phasespace)=x*k(em,vcomp)
              pevent(sp,vcomp,phasespace)=k(sp,vcomp)
              if(co.eq.3)then
                pevent(3,vcomp,phasespace)=pevent(last,vcomp,phasespace)
              elseif(co.eq.4)then
                pevent(4,vcomp,phasespace)=pevent(3,vcomp,phasespace)
                pevent(3,vcomp,phasespace)=pevent(last,vcomp,phasespace)
              endif
              pevent(nextgen(generator),vcomp,phasespace)=0d0
            enddo
c            call applycut(pbeam,pevent(1,0,phasespace),generator,
c     &        nextgen(generator)-1,cut(phasespace),switch)
            call recombination(pbeam,pevent(1,0,phasespace),krec,
     &          generator,nextgen(generator)-1,cut(phasespace),switch)
            if(switch.eq.0)then
              if(cut(phasespace).eq.0) then
                rejected(1)=0
              elseif(rejected(1).gt.0
     &              .and.cut(phasespace).gt.rejected(1))then
                rejected(1)=cut(phasespace)
              endif
c              rejected(1)=min(rejected(1),cut(phasespace))
              rejectedcon(1,phasespace)=cut(phasespace)
            endif
            if(cut(phasespace).eq.0)then
              call crosssection(pevent(1,0,phasespace),
     &            xconv(1),xconv(2),
     &            generator,phasespace,co,sp,switch)
              if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
                wfacp=-8d0*pi*wconv*pf
     &            *(x*x+(1d0-x)**2)/(x*sk(em,co))
     &            /gsum(1)

                weight(1,phasespace)=
     &              wfacp*m2pdfpdfqq_0(phasespace,co,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacp*mm2pdfpdfqq_0(iref,co,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                parton1=0
                do parton2=-4,4
                if (parton2.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacp
     &                *m2pdfpdfiiqq_0(phasespace,parton1,parton2,co,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacp
     &                *mm2pdfpdfiiqq_0(iref,parton1,parton2,co,sp)
      enddo
#endif
                endif
                enddo
#endif
c                write(*,*) 'integrand ingaweightsub',
c     &              phasespace,weight(1,phasespace)

              else
                write(*,*) ' generator 3: not implemented ssub = ',
     &              ssub(3)
                stop
              endif
            endif
          endif
          enddo
          enddo
          enddo
        endif

c        nphsp(generator)=phasespace
        nphsp(generator)=4

c            write(*,*) 'integrand: weights =',
c     &      (weight(phasespace),phasespace=1,nphsp(generator)),generator

***********************************************************************
*                                                                     *
*     tree-level process and                                          *
*     subtraction terms for initial photon contributions 2            *
*                                                                     *
***********************************************************************
c       write(*,*) 'integrand sgen',generator,sgen(generator)
      else if(generator.eq.4.and.sgen(generator).ne.0
     &      .and.mod(nevt,iNumProcs*freqgencon(generator,1))
     &               .lt.iNumProcs)then
        ncontr(generator) = 1
        phasespace=1
        do vcomp=0,3
          kbeam(1,vcomp)=xconv(1)*pbeam(1,vcomp)
          kbeam(2,vcomp)=xconv(2)*pbeam(2,vcomp)
        enddo
        shat=xconv(1)*xconv(2)*energy*energy

c generate momenta

        call generate_momenta(random(9),kbeam,k,kcms,
     &      g(1,phasespace),channel,generator,switch)
c        call generation(random(9),kbeam,k,g(1,phasespace),
c     &    channel,generator,switch)
c phase space point for testing

#ifdef PRINTWEIGHTMAX
        if(lprintweightmax)then
          do vcomp=0,3
            do ext=1,nextgen(generator)
              pprint(ext,vcomp)=k(ext,vcomp)
            enddo
          enddo
        endif
#endif

        collfact=1d0
c        write(*,*) 'collfact = ',collfact

c technical cuts for subtraction branch
        if(ssub(generator).ne.0)then
          call dets(k,sk,nextgen(generator),switch)
          do ext=3,nextgen(generator)
             if (mccharge(ext,generator).ne.0) then

c              write(*,*) 'integrand techcuts 1',
c     &             ext,sk(2,ext),tcdeltas*sk(1,2),
c     &             tcdeltas

              if(sk(2,ext).lt.sk(1,2)*tcdeltas)then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
              kk=k(ext,1)*k(2,1)
     &          +k(ext,2)*k(2,2)
     &          +k(ext,3)*k(2,3)
              kvec=sqrt(k(ext,1)**2+k(ext,2)**2+k(ext,3)**2)

c              write(*,*) 'integrand techcuts 2',
c     &             ext,kk,tcdeltac*kvec*k(nextgen(generator),0)

              if(kk.gt.tcdeltac*kvec*k(2,0))then
                rejectedcon(1,phasespace)=-ext-nextgen(generator)
                rejected(1)=-ext-nextgen(generator)
                return
              endif
            endif
          enddo
          if(k(2,0).le.tcdeltae*sqrt(sk(1,2)))then
            rejectedcon(1,phasespace)=-2
            rejected(1)=-2
            return
          endif

c         write(*,*) 'integrand techcuts 3',
c     &             k(2,0),tcdeltae

        endif

c slicing cuts
        if (ssli(generator).ne.0) then

c     boost momenta to cms
          do i1=0,3
            q(i1)=k(1,i1)+k(2,i1)
          enddo
c          write(*,*) 'integrand shat',shat,
c     &        q(0)**2-q(1)**2-q(2)**2-q(3)**2
          do ext=1,nextgen(generator)
            do vcomp=0,3
              q1(vcomp)=k(ext,vcomp)
            enddo
            call boost(shat,q,q1,1d0,switch)
            do vcomp=0,3
              kcms(ext,vcomp)=q1(vcomp)
            enddo
            ecms(ext,phasespace)=kcms(ext,0)
          enddo
          do ext=3,nextgen(generator)
            if (mccharge(ext,generator).ne.0) then
              kk=kcms(ext,1)*kcms(2,1)
     &            +kcms(ext,2)*kcms(2,2)
     &            +kcms(ext,3)*kcms(2,3)
              kvec=sqrt(kcms(ext,1)**2+kcms(ext,2)**2
     &            +kcms(ext,3)**2)
              if(kk.gt.(1d0-mcslideltac)*kvec*
     &            kcms(2,0))then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
            endif
          enddo
c          if(kcms(2,0).le.mcslideltas*kcms(1,0))then
c            rejectedcon(1,phasespace)=-2
c            rejected(1)=-2
c            return
c          endif
        endif

        if (seff(generator).ne.0) then

c     boost momenta to cms
          do i1=0,3
            q(i1)=k(1,i1)+k(2,i1)
          enddo
          do ext=1,nextgen(generator)
            do vcomp=0,3
              q1(vcomp)=k(ext,vcomp)
            enddo
            call boost(shat,q,q1,1d0,switch)
            do vcomp=0,3
              kcms(ext,vcomp)=q1(vcomp)
            enddo
            ecms(ext,phasespace)=kcms(ext,0)
          enddo

          collfact=1d0
c          collfact1=1d0
          do ext=3,nextgen(generator)
            if (mccharge(ext,generator).ne.0) then
              spk=k(ext,0)*k(2,0)-k(ext,1)*k(2,1)
     &            -k(ext,2)*k(2,2)-k(ext,3)*k(2,3)
c              write(*,*) 'integrand spk',spk,mceffcut*shat,
c     &            spk.lt.mceffcut*shat
              if(spk.lt.mceffcut*shat)then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
              omz=ecms(ext,phasespace)/ecms(2,phasespace)
              omz2=omz*omz
              z2=(1d0-omz)**2
              aux=2d0*omz*spk
              collfact=collfact/(omz2+z2)
     &            *aux*(mcollreg2+aux*(omz2+z2))/(mcollreg2+aux)**2

            endif
          enddo
        endif


c recombination
        call recombination(pbeam,k,krec,generator,nextgen(generator),
     &    cut(phasespace),switch)
        if(switch.eq.0)then
          rejectedcon(1,phasespace)=cut(phasespace)
          rejected(1)=cut(phasespace)
        endif
c        write(*,*) 'integrand: afterrec',cut(1),switch

c if event not cutted ...
        if(switch.eq.0)then
c calculate densities
          call density(k,g,generator,switch)

          if(switch.eq.0)then
            do cha=1,nchannel(generator)
c ADDED for testing AD 6.04.06
              gcha(cha,phasespace)=g(cha,phasespace)
              gsum(phasespace)=gsum(phasespace)+g(cha,phasespace)
     &            *alphaopt(cha,opt(generator),generator)
            enddo
            gsum(phasespace)=gsum(phasespace)/freqgencon(generator,1)
c ADDED for testing AD 6.04.06
            gsumm(phasespace)=gsum(phasespace)
            if(cut(phasespace).eq.0.and.switch.eq.0)then
              call crosssection(k,xconv(1),xconv(2),
     &            generator,phasespace,0,0,switch)
              weight(1,phasespace)=m2pdfpdf_0(phasespace)*wconv
     &          *factor(generator)/gsum(phasespace)
     &                *collfact
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
              mweight(nset,1,phasespace)=mm2pdfpdf_0(iref)*wconv
     &          *factor(generator)/gsum(phasespace)
     &                *collfact
      enddo
#endif
#ifdef SUBPROCESSES
              parton2=0
              do parton1=-4,4
              if (parton1.ne.0) then
                weightii(1,parton1,parton2,phasespace)
     &              =m2pdfpdfii_0(1,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
     &                *collfact
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                mweightii(nset,1,parton1,parton2,phasespace)
     &              =mm2pdfpdfii_0(iref,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
     &                *collfact
      enddo
#endif
              endif
              enddo

#endif

              do ext=1,nextgen(generator)
              do vcomp=0,3
                pevent(ext,vcomp,phasespace)=krec(ext,vcomp)
              enddo
              enddo
            endif
          endif
        endif

c subtraction terms
        if(ssub(generator).ne.0.and.switch.eq.0)then
          last=nextgen(generator)

c initial-state emitter (=photon) with initial-state spectator
c sum over collinear final state particles
          do em=2,2
          do sp=1,1
          do co=3,nextgen(generator)
            pf= mcalpha*factor(generator)
          if(mccharge(co,generator).ne.0d0.and.em.ne.sp)then
            phasespace=phasespace+1
            x=(sk(em,sp)-sk(em,co)-sk(sp,co))/sk(em,sp)
            v=sk(em,co)/sk(em,sp)
            do vcomp=0,3
              ck(vcomp)=k(em,vcomp)+k(sp,vcomp)-k(co,vcomp)
              ckt(vcomp)=x*k(em,vcomp)+k(sp,vcomp)
            enddo
            do vcomp=0,3
              do j=3,nextgen(generator)
                pevent(j,vcomp,phasespace)=k(j,vcomp)
     &            -((1d0+x)*sk(em,j)+2d0*sk(sp,j)-sk(j,co))
     &              /(4d0*x+v-x*v)/sk(em,sp)*(ck(vcomp)+ckt(vcomp))
     &            +(sk(em,j)+sk(sp,j)-sk(j,co))/x/sk(em,sp)*ckt(vcomp)
              enddo
              pevent(em,vcomp,phasespace)=x*k(em,vcomp)
              pevent(sp,vcomp,phasespace)=k(sp,vcomp)
              if(co.eq.3)then
                pevent(3,vcomp,phasespace)=pevent(4,vcomp,phasespace)
                pevent(4,vcomp,phasespace)=pevent(last,vcomp,phasespace)
              elseif(co.eq.4)then
                pevent(4,vcomp,phasespace)=pevent(last,vcomp,phasespace)
              endif
              pevent(nextgen(generator),vcomp,phasespace)=0d0
            enddo
c            call applycut(pbeam,pevent(1,0,phasespace),generator,
c     &        nextgen(generator)-1,cut(phasespace),switch)
            call recombination(pbeam,pevent(1,0,phasespace),krec,
     &          generator,nextgen(generator)-1,cut(phasespace),switch)
            if(switch.eq.0)then
              if(cut(phasespace).eq.0) then
                rejected(1)=0
              elseif(rejected(1).gt.0
     &              .and.cut(phasespace).gt.rejected(1))then
                rejected(1)=cut(phasespace)
              endif
c              rejected(1)=min(rejected(1),cut(phasespace))
              rejectedcon(1,phasespace)=cut(phasespace)
            endif
            if(cut(phasespace).eq.0)then
              call crosssection(pevent(1,0,phasespace),
     &            xconv(1),xconv(2),
     &            generator,phasespace,co,sp,switch)
              if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
                wfacp=-8d0*pi*wconv*pf
     &            *(x*x+(1d0-x)**2)/(x*sk(em,co))
     &            /gsum(1)

                weight(1,phasespace)=
     &              wfacp*m2pdfpdfqq_0(phasespace,co,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacp*mm2pdfpdfqq_0(iref,co,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                parton2=0
                do parton1=-4,4
                if (parton1.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacp
     &                *m2pdfpdfiiqq_0(phasespace,parton1,parton2,co,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacp
     &                *mm2pdfpdfiiqq_0(iref,parton1,parton2,co,sp)
      enddo
#endif
                endif
                enddo
#endif

              else
                write(*,*) ' generator 4: not implemented ssub = ',
     &              ssub(4)
                stop
              endif
            endif
          endif
          enddo
          enddo
          enddo
        endif

c        nphsp(generator)=phasespace
        nphsp(generator)=4

c            write(*,*) 'integrand: weights =',
c     &      (weight(phasespace),phasespace=1,nphsp(generator)),generator

***********************************************************************
*                                                                     *
*     tree-level process and                                          *
*     subtraction terms for initial gluon contributions 1             *
*                                                                     *
***********************************************************************
c       write(*,*) 'integrand sgen',generator,sgen(generator)
      else if(generator.eq.5.and.sgen(generator).ne.0
     &      .and.mod(nevt,iNumProcs*freqgencon(generator,1))
     &               .lt.iNumProcs)then
        ncontr(generator) = 1
        phasespace=1
        do vcomp=0,3
          kbeam(1,vcomp)=xconv(1)*pbeam(1,vcomp)
          kbeam(2,vcomp)=xconv(2)*pbeam(2,vcomp)
        enddo
        shat=xconv(1)*xconv(2)*energy*energy


c generate momenta

        call generate_momenta(random(9),kbeam,k,kcms,
     &      g(1,phasespace),channel,generator,switch)



#ifdef PRINTWEIGHTMAX
        if(lprintweightmax)then
          do vcomp=0,3
            do ext=1,nextgen(generator)
              pprint(ext,vcomp)=k(ext,vcomp)
            enddo
          enddo
        endif
#endif

        collfact=1d0

c technical cuts for subtraction branch
        if(ssub(generator).ne.0)then
          call dets(k,sk,nextgen(generator),switch)
          do ext=3,nextgen(generator)
             if (mccharge(ext,generator).ne.0) then


              if(sk(1,ext).lt.sk(1,2)*tcdeltas)then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
              kk=k(ext,1)*k(1,1)
     &          +k(ext,2)*k(1,2)
     &          +k(ext,3)*k(1,3)
              kvec=sqrt(k(ext,1)**2+k(ext,2)**2+k(ext,3)**2)


              if(kk.gt.tcdeltac*kvec*k(1,0))then
                rejectedcon(1,phasespace)=-ext-nextgen(generator)
                rejected(1)=-ext-nextgen(generator)
                return
              endif
            endif
          enddo
          if(k(1,0).le.tcdeltae*sqrt(sk(1,2)))then
            rejectedcon(1,phasespace)=-1
            rejected(1)=-1
            return
          endif


        endif


c slicing cuts
        if (ssli(generator).ne.0) then

c     boost momenta to cms
          do i1=0,3
            q(i1)=k(1,i1)+k(2,i1)
          enddo
          do ext=1,nextgen(generator)
            do vcomp=0,3
              q1(vcomp)=k(ext,vcomp)
            enddo
            call boost(shat,q,q1,1d0,switch)
            do vcomp=0,3
              kcms(ext,vcomp)=q1(vcomp)
            enddo
            ecms(ext,phasespace)=kcms(ext,0)
          enddo
          do ext=3,nextgen(generator)
            if (mccharge(ext,generator).ne.0) then
              kk=kcms(ext,1)*kcms(1,1)
     &            +kcms(ext,2)*kcms(1,2)
     &            +kcms(ext,3)*kcms(1,3)
              kvec=sqrt(kcms(ext,1)**2+kcms(ext,2)**2
     &            +kcms(ext,3)**2)
              if(kk.gt.(1d0-mcslideltac)*kvec*
     &            kcms(1,0))then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
            endif
          enddo
        endif

c input for effective collinear factors
        if (seff(generator).ne.0) then

c     boost momenta to cms
          do i1=0,3
            q(i1)=k(1,i1)+k(2,i1)
          enddo
          do ext=1,nextgen(generator)
            do vcomp=0,3
              q1(vcomp)=k(ext,vcomp)
            enddo
            call boost(shat,q,q1,1d0,switch)
            do vcomp=0,3
              kcms(ext,vcomp)=q1(vcomp)
            enddo
            ecms(ext,phasespace)=kcms(ext,0)
          enddo

          collfact=1d0
          collfact1=1d0
          do ext=3,nextgen(generator)
            if (mccharge(ext,generator).ne.0) then
              spk=k(ext,0)*k(1,0)-k(ext,1)*k(1,1)
     &            -k(ext,2)*k(1,2)-k(ext,3)*k(1,3)

c              write(*,*) 'integrand cut',spk,mceffcut*shat

              if(spk.lt.mceffcut*shat) then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
              omz=ecms(ext,phasespace)/ecms(1,phasespace)
              omz2=omz*omz
              z2=(1d0-omz)**2
c              s2th2=(1d0-kk/(kvec*kcms(1,0)))*.5d0
              aux=2d0*omz*spk
              collfact=collfact/(omz2+z2)
     &            *aux*(mcollreg2+aux*(omz2+z2))/(mcollreg2+aux)**2

            endif
          enddo
        endif


c recombination
        call recombination(pbeam,k,krec,generator,nextgen(generator),
     &    cut(phasespace),switch)
        if(switch.eq.0)then
          rejectedcon(1,phasespace)=cut(phasespace)
          rejected(1)=cut(phasespace)
        endif


c if event not cutted ...
        if(switch.eq.0)then
c calculate densities
          call density(k,g,generator,switch)


          if(switch.eq.0)then
            do cha=1,nchannel(generator)
c ADDED for testing AD 6.04.06
              gcha(cha,phasespace)=g(cha,phasespace)
              gsum(phasespace)=gsum(phasespace)+g(cha,phasespace)
     &            *alphaopt(cha,opt(generator),generator)
            enddo
            gsum(phasespace)=gsum(phasespace)/freqgencon(generator,1)
c ADDED for testing AD 6.04.06
            gsumm(phasespace)=gsum(phasespace)
            if(cut(phasespace).eq.0.and.switch.eq.0)then
              call crosssection(k,xconv(1),xconv(2),
     &            generator,phasespace,0,0,switch)
              weight(1,phasespace)=m2pdfpdf_0(phasespace)*wconv
     &          *factor(generator)/gsum(phasespace)
     &            *collfact
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
              mweight(nset,1,phasespace)=mm2pdfpdf_0(iref)*wconv
     &          *factor(generator)/gsum(phasespace)
     &            *collfact
      enddo
#endif


#ifdef SUBPROCESSES
              parton1=0
              do parton2=-4,4
              if (parton2.ne.0) then
                weightii(1,parton1,parton2,phasespace)
     &              =m2pdfpdfii_0(1,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
     &            *collfact
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                mweightii(nset,1,parton1,parton2,phasespace)
     &              =mm2pdfpdfii_0(iref,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
     &            *collfact
      enddo
#endif
              endif
              enddo

#endif

              do ext=1,nextgen(generator)
              do vcomp=0,3
                pevent(ext,vcomp,phasespace)=krec(ext,vcomp)
              enddo
              enddo
            endif
          endif
        endif


c subtraction terms
        if(ssub(generator).ne.0.and.switch.eq.0)then
          last=nextgen(generator)

c initial-state emitter (=gluon) with initial-state spectator
c sum over collinear final state particles
          do em=1,1
          do sp=2,2
          do co=3,nextgen(generator)
            pf= mcalphas*factor(generator)
          if(mccharge(co,generator).ne.0d0.and.em.ne.sp)then


            phasespace=phasespace+1
            x=(sk(em,sp)-sk(em,co)-sk(sp,co))/sk(em,sp)
            v=sk(em,co)/sk(em,sp)
            do vcomp=0,3
              ck(vcomp)=k(em,vcomp)+k(sp,vcomp)-k(co,vcomp)
              ckt(vcomp)=x*k(em,vcomp)+k(sp,vcomp)
            enddo
            do vcomp=0,3
              do j=3,nextgen(generator)
                pevent(j,vcomp,phasespace)=k(j,vcomp)
     &            -((1d0+x)*sk(em,j)+2d0*sk(sp,j)-sk(j,co))
     &              /(4d0*x+v-x*v)/sk(em,sp)*(ck(vcomp)+ckt(vcomp))
     &            +(sk(em,j)+sk(sp,j)-sk(j,co))/x/sk(em,sp)*ckt(vcomp)
              enddo
              pevent(em,vcomp,phasespace)=x*k(em,vcomp)
              pevent(sp,vcomp,phasespace)=k(sp,vcomp)
              if(co.eq.3)then
                pevent(3,vcomp,phasespace)=pevent(last,vcomp,phasespace)
              elseif(co.eq.4)then
                pevent(4,vcomp,phasespace)=pevent(3,vcomp,phasespace)
                pevent(3,vcomp,phasespace)=pevent(last,vcomp,phasespace)
              endif
              pevent(nextgen(generator),vcomp,phasespace)=0d0
            enddo
            call recombination(pbeam,pevent(1,0,phasespace),krec,
     &          generator,nextgen(generator)-1,cut(phasespace),switch)


            if(switch.eq.0)then
              if(cut(phasespace).eq.0) then
                rejected(1)=0
              elseif(rejected(1).gt.0
     &              .and.cut(phasespace).gt.rejected(1))then
                rejected(1)=cut(phasespace)
              endif
c              rejected(1)=min(rejected(1),cut(phasespace))
              rejectedcon(1,phasespace)=cut(phasespace)
            endif
            if(cut(phasespace).eq.0)then
              call crosssection(pevent(1,0,phasespace),
     &            xconv(1),xconv(2),
     &            generator,phasespace,co,sp,switch)
              if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
                wfacg=-8d0*pi*wconv*pf
     &            *(x*x+(1d0-x)**2)/(x*sk(em,co))
     &            /gsum(1)


                weight(1,phasespace)=
     &              wfacg*m2pdfpdftt_0(phasespace,co,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacg*mm2pdfpdftt_0(iref,co,sp)
      enddo
#endif





#ifdef SUBPROCESSES
                parton1=0
                do parton2=-4,4
                if (parton2.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacg
     &                *m2pdfpdfiitt_0(phasespace,parton1,parton2,co,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacg
     &                *mm2pdfpdfiitt_0(iref,parton1,parton2,co,sp)
      enddo
#endif
                endif
                enddo
#endif

              else
                write(*,*) ' generator 5: not implemented ssub = ',
     &              ssub(5)
                stop
              endif
            endif
          endif
          enddo
          enddo
          enddo
        endif

c        nphsp(generator)=phasespace
        nphsp(generator)=4

c            write(*,*) 'integrand: weights =',
c     &      (weight(phasespace),phasespace=1,nphsp(generator)),generator

***********************************************************************
*                                                                     *
*     tree-level process and                                          *
*     subtraction terms for initial gluon contributions 2             *
*                                                                     *
***********************************************************************
c       write(*,*) 'integrand sgen',generator,sgen(generator)
       else if(generator.eq.6.and.sgen(generator).ne.0
     &      .and.mod(nevt,iNumProcs*freqgencon(generator,1)).lt.
     &               iNumProcs)then
        ncontr(generator) = 1
        phasespace=1
        do vcomp=0,3
          kbeam(1,vcomp)=xconv(1)*pbeam(1,vcomp)
          kbeam(2,vcomp)=xconv(2)*pbeam(2,vcomp)
        enddo
        shat=xconv(1)*xconv(2)*energy*energy
        collfact=1d0

c generate momenta

        call generate_momenta(random(9),kbeam,k,kcms,
     &      g(1,phasespace),channel,generator,switch)
c        call generation(random(9),kbeam,k,g(1,phasespace),
c     &    channel,generator,switch)
c phase space point for testing

#ifdef PRINTWEIGHTMAX
        if(lprintweightmax)then
          do vcomp=0,3
            do ext=1,nextgen(generator)
              pprint(ext,vcomp)=k(ext,vcomp)
            enddo
          enddo
        endif
#endif

c technical cuts for subtraction branch
        if(ssub(generator).ne.0)then
          call dets(k,sk,nextgen(generator),switch)
          do ext=3,nextgen(generator)
             if (mccharge(ext,generator).ne.0) then

c              write(*,*) 'integrand techcuts 1',
c     &             ext,sk(2,ext),tcdeltas*sk(1,2),
c     &             tcdeltas

              if(sk(2,ext).lt.sk(1,2)*tcdeltas)then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
              kk=k(ext,1)*k(2,1)
     &          +k(ext,2)*k(2,2)
     &          +k(ext,3)*k(2,3)
              kvec=sqrt(k(ext,1)**2+k(ext,2)**2+k(ext,3)**2)

c              write(*,*) 'integrand techcuts 2',
c     &             ext,kk,tcdeltac*kvec*k(nextgen(generator),0)

              if(kk.gt.tcdeltac*kvec*k(2,0))then
                rejectedcon(1,phasespace)=-ext-nextgen(generator)
                rejected(1)=-ext-nextgen(generator)
                return
              endif
            endif
          enddo
          if(k(2,0).le.tcdeltae*sqrt(sk(1,2)))then
            rejectedcon(1,phasespace)=-2
            rejected(1)=-2
            return
          endif

c         write(*,*) 'integrand techcuts 3',
c     &             k(2,0),tcdeltae

        endif

c slicing cuts
        if (ssli(generator).ne.0) then

c     boost momenta to cms
          do i1=0,3
            q(i1)=k(1,i1)+k(2,i1)
          enddo
          do ext=1,nextgen(generator)
            do vcomp=0,3
              q1(vcomp)=k(ext,vcomp)
            enddo
            call boost(shat,q,q1,1d0,switch)
            do vcomp=0,3
              kcms(ext,vcomp)=q1(vcomp)
            enddo
            ecms(ext,phasespace)=kcms(ext,0)
          enddo
          do ext=3,nextgen(generator)
            if (mccharge(ext,generator).ne.0) then
              kk=kcms(ext,1)*kcms(2,1)
     &            +kcms(ext,2)*kcms(2,2)
     &            +kcms(ext,3)*kcms(2,3)
              kvec=sqrt(kcms(ext,1)**2+kcms(ext,2)**2
     &            +kcms(ext,3)**2)
              if(kk.gt.(1d0-mcslideltac)*kvec*
     &            kcms(2,0))then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
            endif
          enddo
c          if(kcms(2,0).le.mcslideltas*kcms(1,0))then
c            rejectedcon(1,phasespace)=-2
c            rejected(1)=-2
c            return
c          endif
        endif

c input for effective collinear factors
        if (seff(generator).ne.0) then

c     boost momenta to cms
          do i1=0,3
            q(i1)=k(1,i1)+k(2,i1)
          enddo
          do ext=1,nextgen(generator)
            do vcomp=0,3
              q1(vcomp)=k(ext,vcomp)
            enddo
            call boost(shat,q,q1,1d0,switch)
            do vcomp=0,3
              kcms(ext,vcomp)=q1(vcomp)
            enddo
            ecms(ext,phasespace)=kcms(ext,0)
          enddo

          collfact=1d0
          collfact1=1d0
          do ext=3,nextgen(generator)
            if (mccharge(ext,generator).ne.0) then
              spk=k(ext,0)*k(2,0)-k(ext,1)*k(2,1)
     &            -k(ext,2)*k(2,2)-k(ext,3)*k(2,3)
c              write(*,*) 'integrand spk',spk,mceffcut*shat,
c     &            spk.lt.mceffcut*shat
              if(spk.lt.mceffcut*shat)then
                rejectedcon(1,phasespace)=-ext
                rejected(1)=-ext
                return
              endif
              omz=ecms(ext,phasespace)/ecms(2,phasespace)
              omz2=omz*omz
              z2=(1d0-omz)**2
              aux=2d0*omz*spk
              collfact=collfact/(omz2+z2)
     &            *aux*(mcollreg2+aux*(omz2+z2))/(mcollreg2+aux)**2

            endif
          enddo
        endif


c recombination
        call recombination(pbeam,k,krec,generator,nextgen(generator),
     &    cut(phasespace),switch)
        if(switch.eq.0)then
          rejectedcon(1,phasespace)=cut(phasespace)
          rejected(1)=cut(phasespace)
        endif

c        write(*,*) 'integrand cut',generator,phasespace,
c     &      cut(phasespace)

c if event not cutted ...
        if(switch.eq.0)then
c calculate densities
          call density(k,g,generator,switch)
          if(switch.eq.0)then
            do cha=1,nchannel(generator)
c ADDED for testing AD 6.04.06
              gsum(phasespace)=gsum(phasespace)+g(cha,phasespace)
     &            *alphaopt(cha,opt(generator),generator)
            enddo
            gsum(phasespace)=gsum(phasespace)/freqgencon(generator,1)
c ADDED for testing AD 6.04.06
            gsumm(phasespace)=gsum(phasespace)
            if(cut(phasespace).eq.0.and.switch.eq.0)then
              call crosssection(k,xconv(1),xconv(2),
     &            generator,phasespace,0,0,switch)
              weight(1,phasespace)=m2pdfpdf_0(phasespace)*wconv
     &          *factor(generator)/gsum(phasespace)
     &                *collfact
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
              mweight(nset,1,phasespace)=mm2pdfpdf_0(iref)*wconv
     &          *factor(generator)/gsum(phasespace)
     &                *collfact
      enddo
#endif
#ifdef SUBPROCESSES
              parton2=0
              do parton1=-4,4
              if (parton1.ne.0) then
                weightii(1,parton1,parton2,phasespace)
     &              =m2pdfpdfii_0(1,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
     &                *collfact
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+1
                mweightii(nset,1,parton1,parton2,phasespace)
     &              =mm2pdfpdfii_0(iref,parton1,parton2)
     &              *wconv*factor(generator)/gsum(phasespace)
     &                *collfact
      enddo
#endif
              endif
              enddo

#endif

              do ext=1,nextgen(generator)
              do vcomp=0,3
                pevent(ext,vcomp,phasespace)=krec(ext,vcomp)
              enddo
              enddo
            endif
          endif
        endif

c subtraction terms
        if(ssub(generator).ne.0.and.switch.eq.0)then
          last=nextgen(generator)

c initial-state emitter (=photon) with initial-state spectator
c sum over collinear final state particles
          do em=2,2
          do sp=1,1
          do co=3,nextgen(generator)
            pf= mcalphas*factor(generator)
          if(mccharge(co,generator).ne.0d0.and.em.ne.sp)then
            phasespace=phasespace+1
            x=(sk(em,sp)-sk(em,co)-sk(sp,co))/sk(em,sp)
            v=sk(em,co)/sk(em,sp)
            do vcomp=0,3
              ck(vcomp)=k(em,vcomp)+k(sp,vcomp)-k(co,vcomp)
              ckt(vcomp)=x*k(em,vcomp)+k(sp,vcomp)
            enddo
            do vcomp=0,3
              do j=3,nextgen(generator)
                pevent(j,vcomp,phasespace)=k(j,vcomp)
     &            -((1d0+x)*sk(em,j)+2d0*sk(sp,j)-sk(j,co))
     &              /(4d0*x+v-x*v)/sk(em,sp)*(ck(vcomp)+ckt(vcomp))
     &            +(sk(em,j)+sk(sp,j)-sk(j,co))/x/sk(em,sp)*ckt(vcomp)
              enddo
              pevent(em,vcomp,phasespace)=x*k(em,vcomp)
              pevent(sp,vcomp,phasespace)=k(sp,vcomp)
              if(co.eq.3)then
                pevent(3,vcomp,phasespace)=pevent(4,vcomp,phasespace)
                pevent(4,vcomp,phasespace)=pevent(last,vcomp,phasespace)
              elseif(co.eq.4)then
                pevent(4,vcomp,phasespace)=pevent(last,vcomp,phasespace)
              endif
              pevent(nextgen(generator),vcomp,phasespace)=0d0
            enddo
c            call applycut(pbeam,pevent(1,0,phasespace),generator,
c     &        nextgen(generator)-1,cut(phasespace),switch)
            call recombination(pbeam,pevent(1,0,phasespace),krec,
     &          generator,nextgen(generator)-1,cut(phasespace),switch)

c            write(*,*) 'integrand cut',generator,phasespace,
c     &          cut(phasespace)

            if(switch.eq.0)then
              if(cut(phasespace).eq.0) then
                rejected(1)=0
              elseif(rejected(1).gt.0
     &              .and.cut(phasespace).gt.rejected(1))then
                rejected(1)=cut(phasespace)
              endif
c              rejected(1)=min(rejected(1),cut(phasespace))
              rejectedcon(1,phasespace)=cut(phasespace)
            endif
            if(cut(phasespace).eq.0)then
              call crosssection(pevent(1,0,phasespace),
     &            xconv(1),xconv(2),
     &            generator,phasespace,co,sp,switch)
              if(ssub(generator).eq.1.or.ssub(generator).eq.2)then
                wfacg=-8d0*pi*wconv*pf
     &            *(x*x+(1d0-x)**2)/(x*sk(em,co))
     &            /gsum(1)

                weight(1,phasespace)=
     &              wfacg*m2pdfpdftt_0(phasespace,co,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                mweight(nset,1,phasespace)=
     &              wfacg*mm2pdfpdftt_0(iref,co,sp)
      enddo
#endif
#ifdef SUBPROCESSES
                parton2=0
                do parton1=-4,4
                if (parton1.ne.0) then
                  weightii(1,parton1,parton2,phasespace)
     &                =wfacg
     &                *m2pdfpdfiitt_0(phasespace,parton1,parton2,co,sp)
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        iref=(nset-1)*maxps+phasespace
                  mweightii(nset,1,parton1,parton2,phasespace)
     &                =wfacg
     &                *mm2pdfpdfiitt_0(iref,parton1,parton2,co,sp)
      enddo
#endif
                endif
                enddo
#endif

              else
                write(*,*) ' generator 4: not implemented ssub = ',
     &              ssub(4)
                stop
              endif
            endif
          endif
          enddo
          enddo
          enddo
        endif

c        nphsp(generator)=phasespace
        nphsp(generator)=4

c            write(*,*) 'integrand: weights =',generator,
c     &      (weight(1,phasespace),phasespace=1,nphsp(generator))

      endif

c end separate generators


c optimization
      weighttot=0d0
      if(switch.gt.0)then
        do iphsp=1,nphsp(generator)
          do icon=1,ncontr(generator)
            weight(icon,iphsp)=0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
            mweight(nset,icon,iphsp)=0
      enddo
#endif
          enddo
        enddo
        weight_0=0d0
#ifdef multpdf
      do nset=1,pdfmemberto-pdfmemberfrom+1
        mweight_0(nset)=0d0
      enddo
#endif
      else
        do iphsp=1,nphsp(generator)
          do icon=1,ncontr(generator)
            weighttot=weighttot+weight(icon,iphsp)
          enddo
        enddo
      endif

c used for case where weighttot is always zero owing to switches
c if not weighttot should be zero only in exceptional cases
c eliminated 29.03.07 AD
c      if(weighttot.eq.0d0)weighttot=weight_0

c      write(*,*) 'integrand: final weights = ',weight_0,weighttot,
c     &    (weight(iweight),iweight=1,nphsp(generator))
      call optimization(weighttot,g(1,1),gsum(1),nevt,generator,
     *  nchannel(generator))
c      if(sconv(generator).ne.0.and.(generator.ne.1.or.ssub(1).ne.0))then
      if(sconv(generator).ne.0)then
        call rancum(rand1,weighttot,1,generator,switch)
        call rancum(rand2,weighttot,2,generator,switch)
      endif

      end

***********************************************************************
      subroutine dets(k,sk,lnext,switch)
***********************************************************************
*     calculation of invariants  sij = 2pi*pj!!!                      *
*     adapted from COFFERGG by Markus Roth                            *
*---------------------------------------------------------------------*
*     21.02.06  Ansgar Denner     last changed  16.01.07              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

c local variables
      real*8 sk(maxe,maxe),k(maxe,0:3)
      integer lnext,switch
      if(switch.gt.0)return
      do ext=1,lnext
      do ext2=ext+1,lnext
        sk(ext,ext2)=2d0*(k(ext,0)*k(ext2,0)-k(ext,1)*k(ext2,1)
     &    -k(ext,2)*k(ext2,2)-k(ext,3)*k(ext2,3))
        sk(ext2,ext)=sk(ext,ext2)
      enddo
      enddo
      end

***********************************************************************
      subroutine ranopt(rand,random,ranout,
     &    lweight,ln,nrand,generator,switch)
***********************************************************************
*     adaptive optimization                                           *
*     adapted from COFFERGG by Markus Roth                            *
*     divides interval from 0 to 1 into maxgr equal intervals         *
*     the interval is chosen according to its contribution using      *
*     the random number rand and weights alpha that are determined    *
*     after each noptran events                                       *
*     the output random number ranout is determined from random       *
*     that runs throug the interval determined by rand                *
*---------------------------------------------------------------------*
*     by Markus Roth         last changed   29.03.07   Ansgar Denner  *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

#ifdef mpiuse
      integer do_optimize
      common/do_optimize_common/do_optimize
#endif
      integer iNumProcs,iMyRank
      common/mpistuff/iNumProcs,iMyRank

c local variables
      real*8 lweight,rand,random,ranout
      integer ngrid,nrand,generator,ln,switch

c to avoid compiler warning
      data ngrid /0/

      if(switch.gt.0)return
c importance sampling
        if(rand.eq.beran(0,nrand,generator))then
          write(*,*) 'ranopt rand=beran',rand,
     &        beran(0,nrand,generator),0,nrand,generator
        endif
      do igrp=1,maxgr
        if(rand.ge.beran(igrp-1,nrand,generator).and.
     &    rand.le.beran(igrp,nrand,generator))then
          ngrid=igrp
        endif
      enddo
      ranout=(dble(ngrid-1)+random)/dble(maxgr)
c local density
      lweight=lweight/(dble(maxgr)*alran(ngrid,nrand,generator))
c optimization
#ifdef mpiuse
        if((iMyRank.EQ.0 .AND. do_optimize.EQ.1) .OR.
     &      iNumProcs.EQ.1) Then
#endif
      if(mod(ln,noptran(generator)).eq.0)then
        do igrp=1,maxgr
          alran(igrp,nrand,generator)=dabs(wran(igrp,nrand,generator))
          beran(igrp,nrand,generator)=beran(igrp-1,nrand,generator)
     &      +alran(igrp,nrand,generator)
        enddo
        do igrp=1,maxgr
          if(beran(maxgr,nrand,generator).ne.0d0)then
            alran(igrp,nrand,generator)=alminran/dble(maxgr)
     &        +(1d0-alminran)*alran(igrp,nrand,generator)
     &          /beran(maxgr,nrand,generator)
            beran(igrp,nrand,generator)=beran(igrp-1,nrand,generator)
     &        +alran(igrp,nrand,generator)
          else
            alran(igrp,nrand,generator)=1d0/dble(maxgr)
            beran(igrp,nrand,generator)=dble(igrp)/dble(maxgr)
          endif
        enddo
c        write(*,*) 'ranopt ',generator,ln,nrand,
c     &      (alran(igrp,nrand,generator),
c     &      igrp=1,maxgr)
      endif
#ifdef mpiuse
        endif
#endif
      end

***********************************************************************
      subroutine rancum(random,lweight,nrand,generator,switch)
***********************************************************************
*     cumulate weights for adaptive optimization                      *
*     adapted from COFFERGG by Markus Roth                            *
*---------------------------------------------------------------------*
*     30.01.06  Ansgar Denner     last changed  08.12.06              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

c local variables
      real*8 lweight,random
      integer nrand,ngrid,generator,switch

      if(switch.gt.0)return

c cumulate weight for optimization
      if(random.lt.1d0)then
        ngrid=int(random*dble(maxgr))+1
      else
c special case added 08.12.06 AD
        ngrid=maxgr
        write(*,*) 'rancum random = 1',random,
     &   lweight,nrand,generator,switch
      endif

      wran(ngrid,nrand,generator)=wran(ngrid,nrand,generator)
     &    +lweight
      end

***********************************************************************
      function dli2(x,switch)
***********************************************************************
*     dilogarithmus with real argument less than 1                    *
*     adapted from COFFERGG by Markus Roth                            *
*---------------------------------------------------------------------*
*     27.09.06  Ansgar Denner     last changed  27.09.06              *
***********************************************************************
      implicit none
c local variables
      real*8 x,dli2,dli2gen,pi2_6
      integer switch

      data pi2_6 /1.6449340668482264364724151666d0/

c mcoutput
      integer nout,numout,maxout,noutgen,nsteps,lnoutmc
      common/mcoutput/nout,numout,maxout,noutgen,nsteps,lnoutmc

      if(x.ge.-1d0.and.x.le.0.5d0)then
        dli2=dli2gen(x,switch)
      elseif(x.gt.0.5d0.and.x.lt.1d0)then
        dli2=pi2_6-dli2gen(1d0-x,switch)-dlog(x)*dlog(1d0-x)
      elseif(x.lt.-1d0)then
        dli2=-pi2_6-dli2gen(1d0/x,switch)-0.5d0*log(-x)**2
      elseif(x.eq.1d0)then
        dli2=pi2_6
      else
        write(*,'(a24,d16.10)')' dli2: no case found: x=',x
        stop
      endif
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     dilogarithm                                                 c
c                                                                 c
c     written by Markus Roth                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function dli2gen(x,switch)
      implicit none

#include "mcparams.h"
#include "mccommon.h"

c local variables
      integer n,nb
      parameter(nb=22)
      real*8 b(0:nb),error,x,dli2gen,x1,x2,xn,new,dli2new
      integer switch
      save b

c error is lower limit of the uncertainty, no message with error=0d0
      data b/1d0,-0.25d0,
     &       2.7777777777777777777777777777777776d-2,0d0,
     &       -2.7777777777777777777777777777777778d-4,0d0,
     &       4.7241118669690098261526832955404383d-6,0d0,
     &       -9.1857730746619635508524397413286296d-8,0d0,
     &       1.8978869988970999072009173019274030d-9,0d0,
     &       -4.0647616451442255268059093862919667d-11,0d0,
     &       8.9216910204564525552179873167527489d-13,0d0,
     &       -1.9939295860721075687236443477937898d-14,0d0,
     &       4.5189800296199181916504765528555929d-16,0d0,
     &       -1.0356517612181247014483411542218656d-17,0d0,
     &       2.3952186210261867457402837430009803d-19/
      x1=-dlog(1d0-x)
      x2=x1*x1
      xn=x2*x1
      dli2gen=b(0)*x1+b(1)*x2+b(2)*xn
      do n=4,nb,2
        xn=xn*x2
        new=b(n)*xn
        dli2new=dli2gen+new
        if(dli2new.eq.dli2gen)return
        dli2gen=dli2new
      enddo
      if(dabs(new/dli2gen).gt.error.and.error.ne.0d0)then
        error=dabs(new/dli2gen)
        switch=33
        if(numout.lt.maxout.and.lnoutmc.ge.5)then
 100        format(/" event = ",i9,", gen = ",i4,", channel = ",i4,
     &        ", exit = ",i4,(a))
            write(nout,100) nevt,cocha,switch
          write(nout,'(a25,d16.10)')' dli2gen: converges badly at ',x
          write(nout,'(a)')'         with error ',error
          numout=numout+1
        endif
        return
      endif
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     gamma function                                              c
c                                                                 c
c     written by Markus Roth                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function gamma(x)
      implicit none
c local variable
      integer maxa
      parameter(maxa=10)
      real*8 gamma,x,omx,a(maxa),prod,pi
      integer i1

      pi=4d0*datan(1d0)
c a(1)=gammae
      data a/0.5772156649015328606d0,0d0,
     &       0.4006856343865314285d0,0d0,
     &       0.2073855510286739853d0,0d0,
     &       0.1440498967688461181d0,0d0,
     &       0.1113342658695646905d0,0d0/
c good convergence for small values of 1-x
      omx=x-1d0
      gamma=dexp(0.5d0*dlog(omx*pi/dsin(pi*omx)))
      do i1=1,maxa,2
        prod=dexp(-(a(i1)*omx**i1))
        gamma=gamma*prod
        if(dabs(prod-1d0).lt.1d-10)return
      enddo
      write(*,'(a)')' gamma: converges badly'
      stop
      end

***********************************************************************
      subroutine histogram(filename,y,x,xminh,xmaxh,nbin,histo,levt,
     *  addpoint,lgen,step)
***********************************************************************
*     filling of generic histogram                                    *
*     filename: name of output file                                   *
*     y       : y-value to be filled in                               *
*     x       : x-value to be filled in                               *
*     xminh   : minimal x of histogram                                *
*     xmaxh   : minimal x of histogram                                *
*     nbin   : number of bins                                         *
*     histo   : number of histogram                                   *
*     levt    : number of event                                       *
*     addpoint: if=1 extra point at xmaxh is added                    *
*     gen     : generator                                             *
*     steps: flag for subroutine histogram                            *
*           steps=1: initialization                                   *
*           steps=2: start filling histograms                         *
*           steps=3: finish filling of histograms                     *
*           steps=4: print data for histograms                        *
*     adapted from COFFERGG by Markus Roth                            *
*---------------------------------------------------------------------*
*     31.10.06  Ansgar Denner     last changed  03.05.07              *
***********************************************************************
      implicit none

#include "mcparams.h"
#include "mccommon.h"

#ifdef mpiuse
      include "mpif.h"
      integer status(MPI_STATUS_SIZE)
      real*8 keep_haverage(maxh,maxb,0:maxg,maxcon),
     &       keep_hsigma(maxh,maxb,0:maxg,maxcon),
     &       keep_havborn(maxh,maxb),keep_hsiborn(maxh,maxb)
#endif

      integer iNumProcs,iMyRank,iErr
      common/mpistuff/iNumProcs,iMyRank

c local variable
      real*8 haverage(maxh,maxb,0:maxg,maxcon),
     &    hsigma(maxh,maxb,0:maxg,maxcon),
     &    ysum(maxh,maxb,maxcon)
      real*8 havborn(maxh,maxb),hsiborn(maxh,maxb)
      real*8 xminh,xmaxh,x,y(maxcon)
      real*8 hav(0:maxg,maxcon),hsi(0:maxg,maxcon)
      real*8 havtot,hsitot,havb,hsib,xshift
      real*8 havqcd,hsiqcd,havew,hsiew,havinp,hsiinp,having,hsiing
      real*8 havsum(0:maxg,maxcon),hsisum(0:maxg,maxcon)
      real*8 havtotsum,hsitotsum,havbsum,hsibsum
      integer i,i2,isum(maxh,maxps),imax(maxh)
      integer histo,nbin,levt,histout,step,addpoint,lgen,column
      integer histoutv,histoutr
      character*30 filename
      character*40 filenamev,filenamer
      save havtot,hsitot,haverage,hsigma,havborn,hsiborn,ysum,isum,imax
      save hav,hsi

      if(nhisto.lt.histo.or.lhisto(histo).ne.1)return
c initialization
      if(step.eq.1)then
        if(nbin.gt.maxb)then
          write(*,'(a20,i6)')' error: reset maxb >',nbin
          stop
        endif
        if(histo.gt.maxh)then
          write(*,'(a20,i6)')' error: reset maxh >',histo
          stop
        endif
        havtot=0d0
        hsitot=0d0
        do gen=0,maxg
          do icon=1,maxcon
            hav(gen,icon)=0d0
            hsi(gen,icon)=0d0
          enddo
        enddo
        do ihisto=1,maxh
          imax(ihisto)=0
          do ibin=1,maxb
            do gen=0,maxg
              do icon=1,ncontr(gen)
                ysum(ihisto,ibin,icon)=0d0
                haverage(ihisto,ibin,gen,icon)=0d0
                hsigma(ihisto,ibin,gen,icon)=0d0
              enddo
            enddo
            havborn(ihisto,ibin)=0d0
            hsiborn(ihisto,ibin)=0d0
          enddo
          do iphsp=1,maxps
            isum(ihisto,iphsp)=0
          enddo
        enddo
c input of single subevents
      elseif(step.eq.2)then
        if(x.ge.xminh.and.x.lt.xmaxh)then
c bin
          i=int((x-xminh)/(xmaxh-xminh)*nbin)+1
          do icon=1,ncontr(lgen)
            ysum(histo,i,icon)=y(icon)+ysum(histo,i,icon)
          enddo
c subevent #
          imax(histo)=imax(histo)+1
c bin for subevent
          isum(histo,imax(histo))=i
        endif
c input born event and final calculation of whole event
      elseif(step.eq.3)then
        if(x.ge.xminh.and.x.lt.xmaxh)then
          i=int((x-xminh)/(xmaxh-xminh)*nbin)+1
          havborn(histo,i)=y(1)+havborn(histo,i)
          hsiborn(histo,i)=y(1)*y(1)+hsiborn(histo,i)
        endif
        if(imax(histo).ge.1)then
          do iphsp=1,imax(histo)
            i=isum(histo,iphsp)
            do icon=1,ncontr(lgen)
              haverage(histo,i,lgen,icon)=haverage(histo,i,lgen,icon)
     &            +ysum(histo,i,icon)
              hsigma(histo,i,lgen,icon)=hsigma(histo,i,lgen,icon)
     &            +ysum(histo,i,icon)*ysum(histo,i,icon)
              ysum(histo,i,icon)=0d0
            enddo
          enddo
        endif
        imax(histo)=0
c output
      elseif(step.eq.4)then
        xshift=1d0

#ifdef mpiuse
	if(iNumProcs.NE.1) Then
	  if(iMyRank.EQ.0) Then
	    do i2=1,iNumProcs-1
	      call MPI_Recv(keep_haverage, maxh*maxb*(maxg+1)*maxcon,
     &             MPI_DOUBLE_PRECISION, i2, 101, MPI_COMM_WORLD,
     &             status, iErr)
	      call MPI_Recv(keep_hsigma, maxh*maxb*(maxg+1)*maxcon,
     &             MPI_DOUBLE_PRECISION, i2, 102, MPI_COMM_WORLD,
     &             status, iErr)
	      call MPI_Recv(keep_havborn, maxh*maxb,
     &             MPI_DOUBLE_PRECISION, i2, 102, MPI_COMM_WORLD,
     &             status, iErr)
	      call MPI_Recv(keep_hsiborn, maxh*maxb,
     &             MPI_DOUBLE_PRECISION, i2, 102, MPI_COMM_WORLD,
     &             status, iErr)
              do ibin=1,maxb
        	havborn(histo,ibin)=
     &                havborn(histo,ibin)+
     &                keep_havborn(histo,ibin)
        	hsiborn(histo,ibin)=
     &                hsiborn(histo,ibin)+
     &                keep_hsiborn(histo,ibin)
        	do gen=0,maxg
        	  do icon=1,ncontr(gen)
        	    haverage(histo,ibin,gen,icon)=
     &                   haverage(histo,ibin,gen,icon)+
     &                   keep_haverage(histo,ibin,gen,icon)
        	    hsigma(histo,ibin,gen,icon)=
     &                   hsigma(histo,ibin,gen,icon)+
     &                   keep_hsigma(histo,ibin,gen,icon)
		  enddo
		enddo
	      enddo
            enddo
	  else
	    call MPI_Send(haverage, maxh*maxb*(maxg+1)*maxcon,
     &           MPI_DOUBLE_PRECISION, 0, 101, MPI_COMM_WORLD, iErr)
	    call MPI_Send(hsigma, maxh*maxb*(maxg+1)*maxcon,
     &           MPI_DOUBLE_PRECISION, 0, 102, MPI_COMM_WORLD, iErr)
	    call MPI_Send(havborn, maxh*maxb,
     &           MPI_DOUBLE_PRECISION, 0, 102, MPI_COMM_WORLD, iErr)
	    call MPI_Send(hsiborn, maxh*maxb,
     &           MPI_DOUBLE_PRECISION, 0, 102, MPI_COMM_WORLD, iErr)
            do ibin=1,maxb
              havborn(histo,ibin)=0D0
              hsiborn(histo,ibin)=0D0
              do gen=0,maxg
        	do icon=1,ncontr(gen)
        	  haverage(histo,ibin,gen,icon)=0D0
        	  hsigma(histo,ibin,gen,icon)=0D0
		enddo
	      enddo
	    enddo
	  endif
	endif

	if(iMyRank.EQ.0) Then
#endif


        do i=35,2,-1
          if(filenamev(i:i).eq.' ')
     &        filenamev=filenamev(1:i-1)//filenamev(i+1:35)
          if(filenamer(i:i).eq.' ')
     &        filenamer=filenamer(1:i-1)//filenamer(i+1:35)
        enddo

        histout=25
        open(unit=histout,file=filename,status='unknown')
        write(histout,'("# histogram:",i3,", bins:",i4)')histo,nbin
        write(histout,'(a)')'# '


c 1000   format($,e12.6,2x)
c 1001   format($,2x,e12.6,2x,e12.6,2x,I10,2x)
 1002   format("# column ",I2,": ",a50)
 1003   format("# column ",I2,": ",a50,a20)
 1004   format("# column ",I2,": ",a57,"\n",13x,a27,I1)
 1005   format("# column ",I2,": kinematical quantity on x-axis")
 1006   format("# column ",I2,": ",a63)

        column=0
        write(histout,1005)column+1
        column=column+2
        write(histout,1002)column,
     &      "Monte Carlo average for the Born cross-section    "
        write(histout,1002)column+1,
     &      "Monte Carlo error   for the Born cross-section    "
        column=column+2
        write(histout,1002)column,
     &      "Monte Carlo average for the full cross-section    "
        write(histout,1002)column+1,
     &      "Monte Carlo error   for the full cross-section    "
        column=column+2
        write(histout,1002)column,
     &      "Monte Carlo average for electroweak corrections   "
        write(histout,1002)column+1,
     &      "Monte Carlo error   for electroweak corrections   "
        column=column+2
        write(histout,1002)column,
     &      "Monte Carlo average for QCD corrections           "
        write(histout,1002)column+1,
     &      "Monte Carlo error   for QCD corrections           "
        column=column+2
        write(histout,1002)column,
     &      "Monte Carlo average for incoming photon corrections   "
        write(histout,1002)column+1,
     &      "Monte Carlo error   for incoming photon corrections   "
        column=column+2
        write(histout,1002)column,
     &      "Monte Carlo average for incoming gluon corrections   "
        write(histout,1002)column+1,
     &      "Monte Carlo error   for incoming gluon corrections   "


        write(histout,'(a)') '# '

        havbsum=0d0
        hsibsum=0d0
        havtotsum=0d0
        hsitotsum=0d0
        do gen=0,maxg
          do icon=1,ncontr(gen)
            havsum(gen,icon)=0d0
            hsisum(gen,icon)=0d0
          enddo
        enddo
        do i=1,nbin
          havtot=0d0
          hsitot=0d0
          do gen=0,maxg
            do icon=1,ncontr(gen)
              hav(gen,icon)=haverage(histo,i,gen,icon)
     &            /dble(levt)/(xmaxh-xminh)*dble(nbin)
              hsi(gen,icon)=hsigma(histo,i,gen,icon)
     &            /(dble(levt)*(xmaxh-xminh)/dble(nbin))**2
              havtot=havtot+hav(gen,icon)
              hsitot=hsitot+hsi(gen,icon)
              hsi(gen,icon)=sqrt(dabs(hsi(gen,icon)
     &            -hav(gen,icon)*hav(gen,icon)/dble(levt)))
            enddo
          enddo
          hsitot=sqrt(dabs(hsitot-havtot*havtot/dble(levt)))
          havb=havborn(histo,i)/dble(levt)/(xmaxh-xminh)*dble(nbin)
          hsib=hsiborn(histo,i)/(dble(levt)*(xmaxh-xminh)/dble(nbin))**2
          hsib=sqrt(dabs(hsib-havb*havb/dble(levt)))

          havew=hav(0,2)+hav(0,3)+hav(1,1)
          hsiew=sqrt(hsi(0,2)**2+hsi(0,3)**2+hsi(1,1)**2)
          havqcd=hav(0,4)+hav(0,5)+hav(2,1)
          hsiqcd=sqrt(hsi(0,4)**2+hsi(0,5)**2+hsi(2,1)**2)
          havinp=hav(0,6)+hav(0,7)+hav(3,1)+hav(4,1)
          hsiinp=sqrt(hsi(0,6)**2+hsi(0,7)**2+hsi(3,1)**2+hsi(4,1)**2)
          having=hav(0,8)+hav(0,9)+hav(5,1)+hav(6,1)
          hsiing=sqrt(hsi(0,8)**2+hsi(0,9)**2+hsi(5,1)**2+hsi(6,1)**2)

          write(histout,'(20(e12.6,2x))')
     &        xminh+(xmaxh-xminh)/dble(nbin)*(dble(i)-xshift),
     &        havb,hsib,havtot,hsitot,havew,hsiew,havqcd,hsiqcd,
     &        havinp,hsiinp,having,hsiing



          havbsum=havbsum+havb
          hsibsum=hsibsum+hsib*hsib
          havtotsum=havtotsum+havtot
          hsitotsum=hsitotsum+hsitot*hsitot
        enddo
        havbsum=havbsum*(xmaxh-xminh)/dble(nbin)
        hsibsum=sqrt(hsibsum)*(xmaxh-xminh)/dble(nbin)
        havtotsum=havtotsum*(xmaxh-xminh)/dble(nbin)
        hsitotsum=sqrt(hsitotsum)*(xmaxh-xminh)/dble(nbin)
        do gen=0,maxg
          do icon=1,ncontr(gen)
            havsum(gen,icon)=havsum(gen,icon)*(xmaxh-xminh)/dble(nbin)
            hsisum(gen,icon)=sqrt(hsisum(gen,icon))
     &          *(xmaxh-xminh)/dble(nbin)
          enddo
        enddo

        if(addpoint.eq.1)then
          write(histout,'(20(e12.6,2x))')
     &        xmaxh,
     &        havb,hsib,havtot,hsitot,havew,hsiew,havqcd,hsiqcd,
     &        havinp,hsiinp,having,hsiing
          write(histout,'(a)') '# '
          write(histout,'(a31,1e12.6)')
     &        '# additional data point at x = ',xmaxh


      endif

      write(histout ,'(a)') '#'
      write(histout ,'(a)') '# useful for plotting with gnuplot'


      if(xshift.eq.0d0)then
        write(histout ,'(a)')'# x = right corner'
      elseif(xshift.eq.0.5d0)then
        write(histout ,'(a)')'# x = mid'
      else if(xshift.eq.1d0)then
        write(histout ,'(a)')'# x = left corner'
      endif
      close(histout)

 300  format(/a11,i3)
 400  format(a12,i2,a14,i2,a1,1g18.10,' +/- ',1g18.10)
 500  format(a31,1g18.10,' +/- ',1g18.10)
      write(nout,*)
      write(nout,300)' histogram',histo
      write(nout,500)' Born cross section:',
     &    havbsum,hsibsum
      write(nout,500)' complete cross section:',
     &    havtotsum,hsitotsum
      do gen=0,ngenerator
        do icon=1,ncontr(gen)
          if(havsum(gen,icon).ne.0d0)then
            write(nout,400)' generator',gen,
     &    ', contribution',icon,':',havsum(gen,icon),hsisum(gen,icon)
          endif
        enddo
      enddo

      endif

#ifdef mpiuse
      endif
#endif

      end


